@@ -6,6 +6,7 @@ use std::{
     sync::OnceLock,
 };
 
+/// write to stdout
 macro_rules! wout {
     ($($arg:tt)*) => ({
         use std::io::Write;
@@ -13,6 +14,7 @@ macro_rules! wout {
     });
 }
 
+/// write to stdout and log::info
 macro_rules! woutinfo {
     ($($arg:tt)*) => ({
         use std::io::Write;
@@ -23,6 +25,7 @@ macro_rules! woutinfo {
     });
 }
 
+/// write to stderr and log::error
 macro_rules! werr {
     ($($arg:tt)*) => ({
         use std::io::Write;
@@ -34,6 +37,7 @@ macro_rules! werr {
 }
 
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
+/// write to stderr and log::warn
 macro_rules! wwarn {
     ($($arg:tt)*) => ({
         use std::io::Write;
@@ -44,6 +48,7 @@ macro_rules! wwarn {
     });
 }
 
+/// write to stderr and log::info
 macro_rules! winfo {
     ($($arg:tt)*) => ({
         use std::io::Write;
@@ -54,6 +59,7 @@ macro_rules! winfo {
     });
 }
 
+/// write to stderr and log::error
 macro_rules! fail {
     ($e:expr) => {{
         use log::error;
@@ -63,6 +69,7 @@ macro_rules! fail {
     }};
 }
 
+/// write to stderr and log::error, using CliError::Other
 macro_rules! fail_clierror {
     ($($t:tt)*) => {{
         use log::error;
@@ -73,6 +80,40 @@ macro_rules! fail_clierror {
     }};
 }
 
+/// write to stderr and log::error, using CliError::IncorrectUsage
+macro_rules! fail_incorrectusage_clierror {
+    ($($t:tt)*) => {{
+        use log::error;
+        use crate::CliError;
+        let err = format!($($t)*);
+        error!("{err}");
+        Err(CliError::IncorrectUsage(err))
+    }};
+}
+
+/// write to stderr and log::error, using CliError::Encoding
+macro_rules! fail_encoding_clierror {
+    ($($t:tt)*) => {{
+        use log::error;
+        use crate::CliError;
+        let err = format!($($t)*);
+        error!("{err}");
+        Err(CliError::Encoding(err))
+    }};
+}
+
+/// write to stderr and log::error, using CliError::OutOfMemory
+macro_rules! fail_OOM_clierror {
+    ($($t:tt)*) => {{
+        use log::error;
+        use crate::CliError;
+        let err = format!($($t)*);
+        error!("{err}");
+        Err(CliError::OutOfMemory(err))
+    }};
+}
+
+/// write to stderr and log::error, using a format string
 macro_rules! fail_format {
     ($($t:tt)*) => {{
         use log::error;
@@ -89,6 +130,9 @@ pub enum QsvExitCode {
     Good           = 0,
     Bad            = 1,
     IncorrectUsage = 2,
+    NetworkError   = 3,
+    OutOfMemory    = 4,
+    EncodingError  = 5,
     Abort          = 255,
 }
 
@@ -106,6 +150,10 @@ pub enum CliError {
     Csv(csv::Error),
     Io(io::Error),
     NoMatch(),
+    IncorrectUsage(String),
+    Network(String),
+    OutOfMemory(String),
+    Encoding(String),
     Other(String),
 }
 
@@ -116,7 +164,11 @@ impl fmt::Display for CliError {
             CliError::Csv(ref e) => e.fmt(f),
             CliError::Io(ref e) => e.fmt(f),
             CliError::NoMatch() => f.write_str("no_match"),
-            CliError::Other(ref s) => f.write_str(s),
+            CliError::Other(ref s)
+            | CliError::IncorrectUsage(ref s)
+            | CliError::Encoding(ref s)
+            | CliError::OutOfMemory(ref s)
+            | CliError::Network(ref s) => f.write_str(s),
         }
     }
 }
@@ -171,6 +223,6 @@ impl From<serde_json::Error> for CliError {
 
 impl From<reqwest::Error> for CliError {
     fn from(err: reqwest::Error) -> CliError {
-        CliError::Other(err.to_string())
+        CliError::Network(err.to_string())
     }
 }
