@@ -1,16 +1,19 @@
 static USAGE: &str = r#"
-Apply a series of transformation functions to a given CSV column. This can be used to
+Apply a series of transformation functions to given CSV column/s. This can be used to
 perform typical data-wrangling tasks and/or to harmonize some values, etc.
 
-It has six subcommands:
- * operations - 36 string, format, currency, regex & NLP operators.
- * emptyreplace - replace empty cells with <--replacement> string.
- * datefmt - Formats a recognized date column to a specified format using <--formatstr>.
- * dynfmt - Dynamically constructs a new column from other columns using the <--formatstr> template.
- * geocode - geocodes a WGS84 location against a static copy of the Geonames cities database.
- * calcconv - parse and evaluate math expressions, with support for units and conversions.
+It has five subcommands:
+ 1. operations*   - 36 string, format, currency, regex & NLP operators.
+ 2. emptyreplace* - replace empty cells with <--replacement> string.
+ 3. datefmt*      - Formats recognized date/s (19 formats recognized) to
+                    a specified date format using <--formatstr>.
+ 4. dynfmt        - Dynamically constructs a new column from other columns using
+                    the <--formatstr> template.
+ 5. calcconv      - parse and evaluate math expressions, with support for units
+                    and conversions.
+    * subcommand is multi-column capable.
 
-OPERATIONS
+OPERATIONS (multi-column capable)
 Multiple operations can be applied, with the comma-delimited operation series
 applied in order:
 
@@ -22,6 +25,8 @@ applied in order:
 Operations support multi-column transformations. Just make sure the
 number of transformed columns with the --rename option is the same. e.g.:
 
+# trim and fold to uppercase the col1,col2 and col3 columns and rename them
+# to newcol1,newcol2 and newcol3
 $ qsv apply operations trim,upper col1,col2,col3 -r newcol1,newcol2,newcol3 file.csv
 
 It has 36 supported operations:
@@ -45,6 +50,7 @@ It has 36 supported operations:
   * replace: Replace all matches of a pattern (using --comparand)
       with a string (using --replacement) (Rust replace)
   * regex_replace: Replace all regex matches in --comparand w/ --replacement.
+      Specify <NULL> as --replacement to remove matches.
   * titlecase - capitalizes English text using Daring Fireball titlecase style
       https://daringfireball.net/2008/05/title_case
   * censor: profanity filter. Add additional comma-delimited profanities with --comparand.
@@ -136,7 +142,7 @@ You can also use this subcommand command to make a copy of a column:
 
   $ qsv apply operations copy col_to_copy -c col_copy file.csv
 
-EMPTYREPLACE
+EMPTYREPLACE (multi-column capable)
 Replace empty cells with <--replacement> string.
 Non-empty cells are not modified. See the `fill` command for more complex empty field operations.
 
@@ -147,17 +153,29 @@ $ qsv apply emptyreplace Measurement --replacement None file.csv
 
 Replace empty cells in file.csv Measurement column with 'Unknown Measurement'.
 
-$ qsv apply emptyreplace --replacement 'Unknown Measurement' file.csv
+$ qsv apply emptyreplace Measurement --replacement 'Unknown Measurement' file.csv
 
-DATEFMT
-Formats a recognized date column to a specified format using <--formatstr>.
+Replace empty cells in file.csv M1,M2 and M3 columns with 'None'.
+
+$ qsv apply emptyreplace M1,M2,M3 --replacement None file.csv
+
+Replace all empty cells in file.csv for columns that start with 'Measurement' with 'None'.
+
+$ qsv apply emptyreplace '/^Measurement/' --replacement None file.csv
+
+Replace all empty cells in file.csv for columns that start with 'observation'
+case insensitive with 'None'.
+
+$ qsv apply emptyreplace --replacement None '/(?i)^observation/' file.csv
+
+DATEFMT (multi-column capable)
+Formats recognized date/s to a specified format using <--formatstr>.
 See https://github.com/jqnatividad/belt/tree/main/dateparser#accepted-date-formats for
 recognized date formats.
 See https://docs.rs/chrono/latest/chrono/format/strftime/ for 
 accepted date formats for --formatstr.
 Defaults to ISO 8601/RFC 3339 format when --formatstr is not specified.
-
-Datefmt also supports multi-column formatting.
+( "%Y-%m-%dT%H:%M:%S%z" - e.g. 2001-07-08T00:34:60.026490+09:30 )
 
 Examples:
 Format dates in Open Date column to ISO 8601/RFC 3339 format:
@@ -168,6 +186,10 @@ Format multiple date columns in file.csv to ISO 8601/RFC 3339 format:
 
   $ qsv apply datefmt 'Open Date,Modified Date,Closed Date' file.csv
 
+Format all columns that end with "_date" case-insensitive in file.csv to ISO 8601/RFC 3339 format:
+
+  $ qsv apply datefmt '\(?i)_date$\' file.csv
+
 Format dates in OpenDate column using '%Y-%m-%d' format:
 
   $ qsv apply datefmt OpenDate --formatstr '%Y-%m-%d' file.csv
@@ -201,27 +223,6 @@ Create a new column 'FullName' from 'FirstName', 'MI', and 'LastName' columns:
 
   $ qsv apply dynfmt --formatstr 'Sir/Madam {FirstName} {MI}. {LastName}' -c FullName file.csv
 
-GEOCODE
-Geocodes to the nearest city center point given a location column
-[i.e. a column which contains a latitude, longitude WGS84 coordinate] against
-an embedded copy of the Geonames city database.
-
-The geocoded information is formatted based on --formatstr, returning
-it in 'city-state' format if not specified.
-
-Use the --new-column option if you want to keep the location column:
-
-Examples:
-Geocode file.csv Location column and set the geocoded value to a
-new column named City.
-
-$ qsv apply geocode Location --new-column City file.csv
-
-Geocode file.csv Location column with --formatstr=city-state and
-set the geocoded value a new column named City.
-
-$ qsv apply geocode Location --formatstr city-state --new-column City file.csv
-
 CALCCONV
 Parse and evaluate math expressions into a new column, with support for units and conversions.
 The math expression is built dynamically using the <--formatstr> template, similar to the DYNFMT
@@ -266,13 +267,14 @@ qsv apply operations <operations> [options] <column> [<input>]
 qsv apply emptyreplace --replacement=<string> [options] <column> [<input>]
 qsv apply datefmt [--formatstr=<string>] [options] <column> [<input>]
 qsv apply dynfmt --formatstr=<string> [options] --new-column=<name> [<input>]
-qsv apply geocode [--formatstr=<string>] [options] <column> [<input>]
 qsv apply calcconv --formatstr=<string> [options] --new-column=<name> [<input>]
 qsv apply --help
 
 apply arguments:
-The <column> argument can be a list of columns for the operations, emptyreplace &
-datefmt subcommands. See 'qsv select --help' for the format details.
+    <column>                        The column/s to apply the transformation to.
+                                    Note that the <column> argument supports multiple columns
+                                    for the operations, emptyreplace & datefmt subcommands.
+                                    See 'qsv select --help' for the format details.
 
     OPERATIONS subcommand:
         <operations>                The operation/s to apply.
@@ -293,13 +295,6 @@ datefmt subcommands. See 'qsv select --help' for the format details.
                                     See DYNFMT example above for more details.
         --new-column=<name>         Put the generated values in a new column.
 
-    GEOCODE subcommand:
-        --formatstr=<string>        The place format to use for the geocode operation.
-                                    See GEOCODE section in the --formatstr option below
-                                    for more details.
-        <column>                    The Location column (a column which contains BOTH latitude and 
-                                    longitude WGS84 coordinates) to geocode.
-
     CALCONV subcommand:
         --formatstr=<string>        The calculation/conversion expression to use.
         --new-column=<name>         Put the calculated/converted values in a new column.
@@ -335,22 +330,12 @@ apply options:
 
                                 DATEFMT: The date format to use. For formats, see
                                   https://docs.rs/chrono/latest/chrono/format/strftime/
-                                  Default to ISO 8601 / RFC 3339 date & time format.
+                                  Default to ISO 8601 / RFC 3339 date & time format -
+                                  "%Y-%m-%dT%H:%M:%S%z" - e.g. 2001-07-08T00:34:60.026490+09:30
                                   [default: %+]
 
                                 DYNFMT: the template to use to construct a new column.
 
-                                GEOCODE: the place format to use with the geocode subcommand.
-                                  The available formats are:
-                                  - 'city-state' (default) - e.g. Brooklyn, New York
-                                  - 'city-country' - Brooklyn, US
-                                  - 'city-state-country' | 'city-admin1-country' - Brooklyn, New York US
-                                  - 'city' - Brooklyn
-                                  - 'county' | 'admin2' - Kings County
-                                  - 'state' | 'admin1' - New York
-                                  - 'county-country' | 'admin2-country' - Kings County, US
-                                  - 'county-state-country' | 'admin2-admin1-country' - Kings County, New York US
-                                  - 'country' - US
     -j, --jobs <arg>            The number of jobs to run in parallel.
                                 When not set, the number of jobs is set to the number of CPUs detected.
     -b, --batch <size>          The number of rows per batch to load into memory, before running in parallel.
@@ -368,7 +353,6 @@ Common options:
 
 use std::{str::FromStr, sync::OnceLock};
 
-use cached::proc_macro::cached;
 use censor::{Censor, Sex, Zealous};
 use cpc::{eval, units::Unit};
 use data_encoding::BASE64;
@@ -383,7 +367,6 @@ use rayon::{
     prelude::IntoParallelRefIterator,
 };
 use regex::Regex;
-use reverse_geocoder::{Locations, ReverseGeocoder};
 use serde::Deserialize;
 use strsim::{
     damerau_levenshtein, hamming, jaro_winkler, normalized_damerau_levenshtein, osa_distance,
@@ -400,10 +383,13 @@ use crate::{
     config::{Config, Delimiter},
     regex_oncelock,
     select::SelectColumns,
-    util, CliResult,
+    util,
+    util::replace_column_value,
+    CliResult,
 };
 
-#[derive(EnumString)]
+#[derive(Clone, EnumString, PartialEq)]
+#[strum(use_phf)]
 #[strum(ascii_case_insensitive)]
 #[allow(non_camel_case_types)]
 enum Operations {
@@ -453,7 +439,6 @@ struct Args {
     cmd_datefmt:         bool,
     cmd_dynfmt:          bool,
     cmd_emptyreplace:    bool,
-    cmd_geocode:         bool,
     cmd_calcconv:        bool,
     arg_input:           Option<String>,
     flag_rename:         Option<String>,
@@ -472,8 +457,6 @@ struct Args {
 }
 
 static CENSOR: OnceLock<Censor> = OnceLock::new();
-static LOCS: OnceLock<Locations> = OnceLock::new();
-static GEOCODER: OnceLock<ReverseGeocoder> = OnceLock::new();
 static EUDEX_COMPARAND_HASH: OnceLock<eudex::Hash> = OnceLock::new();
 static REGEX_REPLACE: OnceLock<Regex> = OnceLock::new();
 static SENTIMENT_ANALYZER: OnceLock<SentimentIntensityAnalyzer> = OnceLock::new();
@@ -487,6 +470,8 @@ const DEFAULT_THRESHOLD: f64 = 0.9;
 // default number of decimal places to round to
 const DEFAULT_ROUND_PLACES: u32 = 3;
 
+const NULL_VALUE: &str = "<null>";
+
 // for thousands operator
 static INDIANCOMMA_POLICY: SeparatorPolicy = SeparatorPolicy {
     separator: ",",
@@ -495,28 +480,15 @@ static INDIANCOMMA_POLICY: SeparatorPolicy = SeparatorPolicy {
 };
 
 // valid subcommands
+#[derive(PartialEq)]
 enum ApplySubCmd {
     Operations,
     DateFmt,
     DynFmt,
-    Geocode,
     EmptyReplace,
     CalcConv,
 }
 
-#[inline]
-fn replace_column_value(
-    record: &csv::StringRecord,
-    column_index: usize,
-    new_value: &str,
-) -> csv::StringRecord {
-    record
-        .into_iter()
-        .enumerate()
-        .map(|(i, v)| if i == column_index { new_value } else { v })
-        .collect()
-}
-
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
     let rconfig = Config::new(&args.arg_input)
@@ -536,49 +508,50 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     if let Some(new_name) = args.flag_rename {
         let new_col_names = util::ColumnNameParser::new(&new_name).parse()?;
         if new_col_names.len() != sel.len() {
-            return fail!("Number of new columns does not match input column selection.");
+            return fail_incorrectusage_clierror!(
+                "Number of new columns does not match input column selection."
+            );
         }
         for (i, col_index) in sel.iter().enumerate() {
             headers = replace_column_value(&headers, *col_index, &new_col_names[i]);
         }
     }
 
-    if !rconfig.no_headers {
-        if let Some(new_column) = &args.flag_new_column {
-            headers.push_field(new_column);
-        }
-        wtr.write_record(&headers)?;
-    }
-
     // for dynfmt, safe_headers are the "safe" version of colnames - alphanumeric only,
     // all other chars replaced with underscore
     // dynfmt_fields are the columns used in the dynfmt --formatstr option
     // we prep it so we only populate the lookup vec with the index of these columns
     // so SimpleCurlyFormat is performant
-    let mut dynfmt_fields = Vec::with_capacity(10); // 10 is a reasonable default to save allocs
-    let mut dynfmt_template = args.flag_formatstr.clone();
-    if args.cmd_dynfmt || args.cmd_calcconv {
+    let dynfmt_template = if args.cmd_dynfmt || args.cmd_calcconv {
         if args.flag_no_headers {
-            return fail!("dynfmt/calcconv subcommand requires headers.");
+            return fail_incorrectusage_clierror!("dynfmt/calcconv subcommand requires headers.");
         }
+
+        let mut dynfmt_template_wrk = args.flag_formatstr.clone();
+        let mut dynfmt_fields = Vec::new();
+
         // first, get the fields used in the dynfmt template
-        let (safe_headers, _) = util::safe_header_names(&headers, false, false, None, "", true);
         let formatstr_re: &'static Regex = crate::regex_oncelock!(r"\{(?P<key>\w+)?\}");
         for format_fields in formatstr_re.captures_iter(&args.flag_formatstr) {
             dynfmt_fields.push(format_fields.name("key").unwrap().as_str());
         }
         // we sort the fields so we can do binary_search
         dynfmt_fields.sort_unstable();
+
         // now, get the indices of the columns for the lookup vec
+        let (safe_headers, _) = util::safe_header_names(&headers, false, false, None, "", true);
         for (i, field) in safe_headers.iter().enumerate() {
             if dynfmt_fields.binary_search(&field.as_str()).is_ok() {
                 let field_with_curly = format!("{{{field}}}");
                 let field_index = format!("{{{i}}}");
-                dynfmt_template = dynfmt_template.replace(&field_with_curly, &field_index);
+                dynfmt_template_wrk = dynfmt_template_wrk.replace(&field_with_curly, &field_index);
             }
         }
-        debug!("dynfmt_fields: {dynfmt_fields:?}  dynfmt_template: {dynfmt_template}");
-    }
+        debug!("dynfmt_fields: {dynfmt_fields:?}  dynfmt_template: {dynfmt_template_wrk}");
+        dynfmt_template_wrk
+    } else {
+        String::new()
+    };
 
     let mut ops_vec: Vec<Operations> = Vec::new();
 
@@ -594,8 +567,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             Err(e) => return Err(e),
         }
         ApplySubCmd::Operations
-    } else if args.cmd_geocode {
-        ApplySubCmd::Geocode
     } else if args.cmd_datefmt {
         ApplySubCmd::DateFmt
     } else if args.cmd_dynfmt {
@@ -605,9 +576,30 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     } else if args.cmd_calcconv {
         ApplySubCmd::CalcConv
     } else {
-        return fail!("Unknown apply subcommand.");
+        return fail_incorrectusage_clierror!("Unknown apply subcommand.");
     };
 
+    if !rconfig.no_headers {
+        if let Some(new_column) = &args.flag_new_column {
+            headers.push_field(new_column);
+        }
+        wtr.write_record(&headers)?;
+    }
+
+    // if there is a regex_replace operation and replacement is <NULL> case-insensitive,
+    // we set it to empty string
+    let flag_replacement = if apply_cmd == ApplySubCmd::Operations
+        && ops_vec.contains(&Operations::Regex_Replace)
+        && args.flag_replacement.to_lowercase() == NULL_VALUE
+    {
+        String::new()
+    } else {
+        args.flag_replacement
+    };
+    let flag_comparand = args.flag_comparand;
+    let flag_formatstr = args.flag_formatstr;
+    let flag_new_column = args.flag_new_column;
+
     // prep progress bar
     let show_progress =
         (args.flag_progressbar || util::get_envvar_flag("QSV_PROGRESSBAR")) && !rconfig.is_stdin();
@@ -664,20 +656,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             .map(|record_item| {
                 let mut record = record_item.clone();
                 match apply_cmd {
-                    ApplySubCmd::Geocode => {
-                        let mut cell = record[column_index].to_owned();
-                        if !cell.is_empty() {
-                            let search_result = search_cached(&cell, &args.flag_formatstr);
-                            if let Some(geocoded_result) = search_result {
-                                cell = geocoded_result;
-                            }
-                        }
-                        if args.flag_new_column.is_some() {
-                            record.push_field(&cell);
-                        } else {
-                            record = replace_column_value(&record, column_index, &cell);
-                        }
-                    },
                     ApplySubCmd::Operations => {
                         let mut cell = String::new();
                         for col_index in &*sel {
@@ -685,11 +663,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                             apply_operations(
                                 &ops_vec,
                                 &mut cell,
-                                &args.flag_comparand,
-                                &args.flag_replacement,
-                                &args.flag_formatstr,
+                                &flag_comparand,
+                                &flag_replacement,
+                                &flag_formatstr,
                             );
-                            if args.flag_new_column.is_some() {
+                            if flag_new_column.is_some() {
                                 record.push_field(&cell);
                             } else {
                                 record = replace_column_value(&record, *col_index, &cell);
@@ -701,9 +679,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         for col_index in &*sel {
                             record[*col_index].clone_into(&mut cell);
                             if cell.trim().is_empty() {
-                                cell = args.flag_replacement.clone();
+                                cell = flag_replacement.clone();
                             }
-                            if args.flag_new_column.is_some() {
+                            if flag_new_column.is_some() {
                                 record.push_field(&cell);
                             } else {
                                 record = replace_column_value(&record, *col_index, &cell);
@@ -718,7 +696,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                                 let parsed_date = parse_with_preference(&cell, prefer_dmy);
                                 if let Ok(format_date) = parsed_date {
                                     let formatted_date =
-                                        format_date.format(&args.flag_formatstr).to_string();
+                                        format_date.format(&flag_formatstr).to_string();
                                     if !args.flag_keep_zero_time
                                         && formatted_date.ends_with("T00:00:00+00:00")
                                     {
@@ -728,7 +706,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                                     }
                                 }
                             }
-                            if args.flag_new_column.is_some() {
+                            if flag_new_column.is_some() {
                                 record.push_field(&cell);
                             } else {
                                 record = replace_column_value(&record, *col_index, &cell);
@@ -748,7 +726,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                                 cell = formatted.to_string();
                             }
                         }
-                        if args.flag_new_column.is_some() {
+                        if flag_new_column.is_some() {
                             record.push_field(&cell);
                         } else {
                             record = replace_column_value(&record, column_index, &cell);
@@ -790,7 +768,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                             }
                         };
 
-                        if args.flag_new_column.is_some() {
+                        if flag_new_column.is_some() {
                             record.push_field(&result);
                         } else {
                             record = replace_column_value(&record, column_index, &result);
@@ -815,9 +793,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     } // end batch loop
 
     if show_progress {
-        if args.cmd_geocode {
-            util::update_cache_info!(progress, SEARCH_CACHED);
-        }
         util::finish_progress(&progress);
     }
     Ok(wtr.flush()?)
@@ -846,12 +821,14 @@ fn validate_operations(
 
     for op in operations {
         let Ok(operation) = Operations::from_str(op) else {
-            return fail_clierror!("Unknown '{op}' operation");
+            return fail_incorrectusage_clierror!("Unknown '{op}' operation");
         };
         match operation {
             Operations::Censor | Operations::Censor_Check | Operations::Censor_Count => {
                 if flag_new_column.is_none() {
-                    return fail!("--new_column (-c) is required for censor operations.");
+                    return fail_incorrectusage_clierror!(
+                        "--new_column (-c) is required for censor operations."
+                    );
                 }
                 if censor_invokes == 0
                     && CENSOR
@@ -870,13 +847,17 @@ fn validate_operations(
             },
             Operations::Copy => {
                 if flag_new_column.is_none() {
-                    return fail!("--new_column (-c) is required for copy operation.");
+                    return fail_incorrectusage_clierror!(
+                        "--new_column (-c) is required for copy operation."
+                    );
                 }
                 copy_invokes = copy_invokes.saturating_add(1);
             },
             Operations::Eudex => {
                 if flag_comparand.is_empty() || flag_new_column.is_none() {
-                    return fail!("--comparand (-C) and --new_column (-c) is required for eudex.");
+                    return fail_incorrectusage_clierror!(
+                        "--comparand (-C) and --new_column (-c) is required for eudex."
+                    );
                 }
                 if eudex_invokes == 0
                     && EUDEX_COMPARAND_HASH
@@ -889,12 +870,14 @@ fn validate_operations(
             },
             Operations::Mtrim | Operations::Mltrim | Operations::Mrtrim => {
                 if flag_comparand.is_empty() {
-                    return fail!("--comparand (-C) is required for match trim operations.");
+                    return fail_incorrectusage_clierror!(
+                        "--comparand (-C) is required for match trim operations."
+                    );
                 }
             },
             Operations::Regex_Replace => {
                 if flag_comparand.is_empty() || flag_replacement.is_empty() {
-                    return fail!(
+                    return fail_incorrectusage_clierror!(
                         "--comparand (-C) and --replacement (-R) are required for regex_replace \
                          operation."
                     );
@@ -913,7 +896,7 @@ fn validate_operations(
             },
             Operations::Replace => {
                 if flag_comparand.is_empty() || flag_replacement.is_empty() {
-                    return fail!(
+                    return fail_incorrectusage_clierror!(
                         "--comparand (-C) and --replacement (-R) are required for replace \
                          operation."
                     );
@@ -922,7 +905,9 @@ fn validate_operations(
             },
             Operations::Sentiment => {
                 if flag_new_column.is_none() {
-                    return fail!("--new_column (-c) is required for sentiment operation.");
+                    return fail_incorrectusage_clierror!(
+                        "--new_column (-c) is required for sentiment operation."
+                    );
                 }
                 sentiment_invokes = sentiment_invokes.saturating_add(1);
             },
@@ -933,7 +918,7 @@ fn validate_operations(
             | Operations::Simhm
             | Operations::Simod => {
                 if flag_comparand.is_empty() || flag_new_column.is_none() {
-                    return fail!(
+                    return fail_incorrectusage_clierror!(
                         "--comparand (-C) and --new_column (-c) is required for similarity \
                          operations."
                     );
@@ -973,7 +958,9 @@ fn validate_operations(
             },
             Operations::Whatlang => {
                 if flag_new_column.is_none() {
-                    return fail!("--new_column (-c) is required for whatlang language detection.");
+                    return fail_incorrectusage_clierror!(
+                        "--new_column (-c) is required for whatlang language detection."
+                    );
                 }
 
                 if whatlang_invokes == 0
@@ -1026,7 +1013,7 @@ fn validate_operations(
         || strip_invokes > 1
         || whatlang_invokes > 1
     {
-        return fail_clierror!(
+        return fail_incorrectusage_clierror!(
             "you can only use censor({censor_invokes}), copy({copy_invokes}), \
              eudex({eudex_invokes}), regex_replace({regex_replace_invokes}), \
              replace({replace_invokes}), sentiment({sentiment_invokes}), \
@@ -1132,7 +1119,7 @@ fn apply_operations(
                 *cell = censor.count(cell).to_string();
             },
             Operations::Thousands => {
-                if let Ok(num) = fast_float::parse::<f64, _>(&cell) {
+                if let Ok(num) = cell.parse::<f64>() {
                     let mut temp_string = num.separate_by_policy(*THOUSANDS_POLICY.get().unwrap());
 
                     // if there is a decimal separator (fractional part > 0.0), use the requested
@@ -1157,7 +1144,7 @@ fn apply_operations(
                 }
             },
             Operations::Round => {
-                if let Ok(num) = fast_float::parse::<f64, _>(&cell) {
+                if let Ok(num) = cell.parse::<f64>() {
                     *cell = util::round_num(num, *ROUND_PLACES.get().unwrap());
                 }
             },
@@ -1198,10 +1185,8 @@ fn apply_operations(
                 };
 
                 if let Ok(currency_value) = Currency::from_str(&cell_val) {
-                    let currency_wrk = currency_value.convert(
-                        fast_float::parse::<f64, _>(replacement).unwrap_or(1.0_f64),
-                        comparand,
-                    );
+                    let currency_wrk = currency_value
+                        .convert(replacement.parse::<f64>().unwrap_or(1.0_f64), comparand);
                     *cell = if formatstr.contains("euro") {
                         format!("{currency_wrk:e}")
                     } else {
@@ -1265,68 +1250,3 @@ fn apply_operations(
         }
     }
 }
-
-#[cached(
-    key = "String",
-    convert = r#"{ format!("{}", cell) }"#,
-    option = true,
-    sync_writes = false
-)]
-fn search_cached(cell: &str, formatstr: &str) -> Option<String> {
-    let geocoder =
-        GEOCODER.get_or_init(|| ReverseGeocoder::new(LOCS.get_or_init(Locations::from_memory)));
-
-    // regex for Location field. Accepts (lat, long) & lat, long
-    let locregex: &'static Regex =
-        regex_oncelock!(r"(?-u)([+-]?[0-9]+\.?[0-9]*|\.[0-9]+),\s*([+-]?[0-9]+\.?[0-9]*|\.[0-9]+)");
-
-    let loccaps = locregex.captures(cell);
-    loccaps.and_then(|loccaps| {
-        let lat = fast_float::parse(&loccaps[1]).unwrap_or_default();
-        let long = fast_float::parse(&loccaps[2]).unwrap_or_default();
-        if (-90.0..=90.0).contains(&lat) && (-180.0..=180.0).contains(&long) {
-            let search_result = geocoder.search((lat, long));
-            search_result.map(|locdetails| {
-                #[allow(clippy::match_same_arms)]
-                // match arms are evaluated in order,
-                // so we're optimizing for the most common cases first
-                match formatstr {
-                    "%+" | "city-state" => format!(
-                        "{name}, {admin1}",
-                        name = locdetails.record.name,
-                        admin1 = locdetails.record.admin1,
-                    ),
-                    "city-country" => format!(
-                        "{name}, {cc}",
-                        name = locdetails.record.name,
-                        cc = locdetails.record.cc
-                    ),
-                    "city-state-country" | "city-admin1-country" => format!(
-                        "{name}, {admin1} {cc}",
-                        name = locdetails.record.name,
-                        admin1 = locdetails.record.admin1,
-                        cc = locdetails.record.cc
-                    ),
-                    "city" => locdetails.record.name.to_string(),
-                    "county" | "admin2" => locdetails.record.admin2.to_string(),
-                    "state" | "admin1" => locdetails.record.admin1.to_string(),
-                    "county-country" | "admin2-country" => format!(
-                        "{admin2}, {cc}",
-                        admin2 = locdetails.record.admin2,
-                        cc = locdetails.record.cc
-                    ),
-                    "county-state-country" | "admin2-admin1-country" => format!(
-                        "{admin2}, {admin1} {cc}",
-                        admin2 = locdetails.record.admin2,
-                        admin1 = locdetails.record.admin1,
-                        cc = locdetails.record.cc
-                    ),
-                    "country" => locdetails.record.cc.to_string(),
-                    _ => locdetails.record.name.to_string(),
-                }
-            })
-        } else {
-            None
-        }
-    })
-}
