@@ -1,15 +1,18 @@
 static USAGE: &str = r#"
 applydp is a slimmed-down version of apply specifically created for Datapusher+.
-It "applies" a series of transformation functions to a given CSV column. This can be used to
+It "applies" a series of transformation functions to given CSV column/s. This can be used to
 perform typical data-wrangling tasks and/or to harmonize some values, etc.
 
 It has four subcommands:
- * operations - 18 string, format & regex operators.
- * emptyreplace - replace empty cells with <--replacement> string.
- * datefmt - Formats a recognized date column to a specified format using <--formatstr>.
- * dynfmt - Dynamically constructs a new column from other columns using the <--formatstr> template.
+ 1. operations*   - 18 string, format & regex operators.
+ 2. emptyreplace* - replace empty cells with <--replacement> string.
+ 3. datefmt*      - Formats recognized date/s (19 formats recognized) to
+                    a specified date format using <--formatstr>.
+ 4. dynfmt        - Dynamically constructs a new column from other columns using
+                    the <--formatstr> template.
+    * subcommand is multi-column capable.
 
-OPERATIONS
+OPERATIONS (multi-column capable)
 Multiple operations can be applied, with the comma-delimited operation series
 applied in order:
 
@@ -40,6 +43,7 @@ It has 18 supported operations:
   * replace: Replace all matches of a pattern (using --comparand)
       with a string (using --replacement) (Rust replace)
   * regex_replace: Replace all regex matches in --comparand w/ --replacement.
+      Specify <NULL> as --replacement to remove matches.
   * round: Round numeric values to the specified number of decimal places using
       Midpoint Nearest Even Rounding Strategy AKA "Bankers Rounding."
       Specify the number of decimal places with --formatstr (default: 3).
@@ -60,6 +64,10 @@ save it to a new column named uppercase_clean_surname.
 
   $ qsv applydp operations trim,upper surname -c uppercase_clean_surname file.csv
 
+Trim, squeeze, then transform to uppercase in place ALL fields that end with "_name" 
+  
+    $ qsv applydp operations trim,squeeze,upper \_name$\ file.csv
+
 Trim, then transform to uppercase the firstname and surname fields and
 rename the columns ufirstname and usurname.
 
@@ -78,7 +86,7 @@ You can also use this subcommand command to make a copy of a column:
 
   $ qsv applydp operations copy col_to_copy -c col_copy file.csv
 
-EMPTYREPLACE
+EMPTYREPLACE (multi-column capable)
 Replace empty cells with <--replacement> string.
 Non-empty cells are not modified. See the `fill` command for more complex empty field operations.
 
@@ -89,17 +97,29 @@ $ qsv applydp emptyreplace Measurement --replacement None file.csv
 
 Replace empty cells in file.csv Measurement column with 'Unknown Measurement'.
 
-$ qsv applydp emptyreplace --replacement 'Unknown Measurement' file.csv
+$ qsv applydp emptyreplace Measurement --replacement 'Unknown Measurement' file.csv
 
-DATEFMT
+Replace empty cells in file.csv M1,M2 and M3 columns with 'None'.
+
+$ qsv applydp emptyreplace M1,M2,M3 --replacement None file.csv
+
+Replace all empty cells in file.csv for columns that start with 'Measurement' with 'None'.
+
+$ qsv applydp emptyreplace '/^Measurement/' --replacement None file.csv
+
+Replace all empty cells in file.csv for columns that start with 'observation'
+case insensitive with 'None'.
+
+$ qsv applydp emptyreplace --replacement None '/(?i)^observation/' file.csv
+
+DATEFMT (multi-column capable)
 Formats a recognized date column to a specified format using <--formatstr>. 
 See https://github.com/jqnatividad/belt/tree/main/dateparser#accepted-date-formats for
 recognized date formats.
 See https://docs.rs/chrono/latest/chrono/format/strftime/ for
 accepted date formats for --formatstr.
 Defaults to ISO 8601/RFC 3339 format when --formatstr is not specified.
-
-Datefmt also supports multi-column formatting.
+( "%Y-%m-%dT%H:%M:%S%z" - e.g. 2001-07-08T00:34:60.026490+09:30 )
 
 Examples:
 Format dates in Open Date column to ISO 8601/RFC 3339 format:
@@ -110,6 +130,11 @@ Format multiple date columns in file.csv to ISO 8601/RFC 3339 format:
 
   $ qsv applydp datefmt 'Open Date,Modified Date,Closed Date' file.csv
 
+Format all columns that end with "_date" case-insensitive in file.csv to
+ISO 8601/RFC 3339 format:
+
+  $ qsv applydp datefmt '\(?i)_date$\' file.csv
+
 Format dates in OpenDate column using '%Y-%m-%d' format:
 
   $ qsv applydp datefmt OpenDate --formatstr '%Y-%m-%d' file.csv
@@ -154,28 +179,30 @@ qsv applydp dynfmt --formatstr=<string> [options] --new-column=<name> [<input>]
 qsv applydp --help
 
 apply arguments:
-The <column> argument can be a list of columns for the operations, emptyreplace &
-datefmt subcommands. See 'qsv select --help' for the format details.
+    <column>                    The column/s to apply the transformation to.
+                                Note that the <column> argument supports multiple columns
+                                for the operations, emptyreplace & datefmt subcommands.
+                                See 'qsv select --help' for the format details.
 
     OPERATIONS subcommand:
-        <operations>                The operation/s to apply.
-        <column>                    The column/s to apply the operations to.
+        <operations>            The operation/s to apply.
+        <column>                The column/s to apply the operations to.
 
     EMPTYREPLACE subcommand:
-        --replacement=<string>      The string to to use to replace empty values.
-        <column>                    The column/s to check for emptiness.
+        --replacement=<string>  The string to to use to replace empty values.
+        <column>                The column/s to check for emptiness.
 
     DATEFMT subcommand:
-        --formatstr=<string>        The date format to use for the datefmt operation.
-                                    See DATEFMT section in the --formatstr option below
-                                    for more details.
-        <column>                    The date column/s to apply the datefmt operation to.
+        --formatstr=<string>    The date format to use for the datefmt operation.
+                                See DATEFMT section in the --formatstr option below
+                                for more details.
+        <column>                The date column/s to apply the datefmt operation to.
 
     DYNFMT subcommand:
-        --formatstr=<string>        The template to use for the dynfmt operation.
-                                    See DYNFMT example above for more details.
-        --new-column=<name>         Put the generated values in a new column.
-        
+        --formatstr=<string>    The template to use for the dynfmt operation.
+                                See DYNFMT example above for more details.
+        --new-column=<name>     Put the generated values in a new column.
+
     <input>                     The input file to read from. If not specified, reads from stdin.
 
 applydp options:
@@ -195,7 +222,8 @@ applydp options:
 
                                 DATEFMT: The date format to use. For formats, see
                                   https://docs.rs/chrono/latest/chrono/format/strftime/
-                                  Default to ISO 8601 / RFC 3339 date & time format.
+                                  Default to ISO 8601 / RFC 3339 date & time format -
+                                  "%Y-%m-%dT%H:%M:%S%z" (e.g. 2001-07-08T00:34:60.026490+09:30)
                                   [default: %+]
 
                                 DYNFMT: the template to use to construct a new column.
@@ -232,10 +260,13 @@ use crate::{
     config::{Config, Delimiter},
     regex_oncelock,
     select::SelectColumns,
-    util, CliResult,
+    util,
+    util::replace_column_value,
+    CliResult,
 };
 
-#[derive(EnumString)]
+#[derive(Clone, EnumString, PartialEq)]
+#[strum(use_phf)]
 #[strum(ascii_case_insensitive)]
 #[allow(non_camel_case_types)]
 enum Operations {
@@ -288,18 +319,7 @@ static ROUND_PLACES: OnceLock<u32> = OnceLock::new();
 // default number of decimal places to round to
 const DEFAULT_ROUND_PLACES: u32 = 3;
 
-#[inline]
-fn replace_column_value(
-    record: &csv::StringRecord,
-    column_index: usize,
-    new_value: &str,
-) -> csv::StringRecord {
-    record
-        .into_iter()
-        .enumerate()
-        .map(|(i, v)| if i == column_index { new_value } else { v })
-        .collect()
-}
+const NULL_VALUE: &str = "<null>";
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
@@ -320,50 +340,52 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     if let Some(new_name) = args.flag_rename {
         let new_col_names = util::ColumnNameParser::new(&new_name).parse()?;
         if new_col_names.len() != sel.len() {
-            return fail!("Number of new columns does not match input column selection.");
+            return fail_incorrectusage_clierror!(
+                "Number of new columns does not match input column selection."
+            );
         }
         for (i, col_index) in sel.iter().enumerate() {
             headers = replace_column_value(&headers, *col_index, &new_col_names[i]);
         }
     }
 
-    if !rconfig.no_headers {
-        if let Some(new_column) = &args.flag_new_column {
-            headers.push_field(new_column);
-        }
-        wtr.write_record(&headers)?;
-    }
-
     // for dynfmt, safe_headers are the "safe" version of colnames - alphanumeric only,
     // all other chars replaced with underscore
     // dynfmt_fields are the columns used in the dynfmt --formatstr option
     // we prep it so we only populate the lookup vec with the index of these columns
     // so SimpleCurlyFormat is performant
-    let mut dynfmt_fields = Vec::with_capacity(10); // 10 is a reasonable default to save allocs
-    let mut dynfmt_template = args.flag_formatstr.clone();
-    if args.cmd_dynfmt {
+    let dynfmt_template = if args.cmd_dynfmt {
         if args.flag_no_headers {
-            return fail!("dynfmt/calcconv subcommand requires headers.");
+            return fail_incorrectusage_clierror!("dynfmt/calcconv subcommand requires headers.");
         }
+
+        let mut dynfmt_template_wrk = args.flag_formatstr.clone();
+        let mut dynfmt_fields = Vec::new();
+
         // first, get the fields used in the dynfmt template
-        let (safe_headers, _) = util::safe_header_names(&headers, false, false, None, "", true);
         let formatstr_re: &'static Regex = crate::regex_oncelock!(r"\{(?P<key>\w+)?\}");
         for format_fields in formatstr_re.captures_iter(&args.flag_formatstr) {
             dynfmt_fields.push(format_fields.name("key").unwrap().as_str());
         }
         // we sort the fields so we can do binary_search
         dynfmt_fields.sort_unstable();
+
         // now, get the indices of the columns for the lookup vec
+        let (safe_headers, _) = util::safe_header_names(&headers, false, false, None, "", true);
         for (i, field) in safe_headers.iter().enumerate() {
             if dynfmt_fields.binary_search(&field.as_str()).is_ok() {
                 let field_with_curly = format!("{{{field}}}");
                 let field_index = format!("{{{i}}}");
-                dynfmt_template = dynfmt_template.replace(&field_with_curly, &field_index);
+                dynfmt_template_wrk = dynfmt_template_wrk.replace(&field_with_curly, &field_index);
             }
         }
-        debug!("dynfmt_fields: {dynfmt_fields:?}  dynfmt_template: {dynfmt_template}");
-    }
+        debug!("dynfmt_fields: {dynfmt_fields:?}  dynfmt_template: {dynfmt_template_wrk}");
+        dynfmt_template_wrk
+    } else {
+        String::new()
+    };
 
+    #[derive(PartialEq)]
     enum ApplydpSubCmd {
         Operations,
         DateFmt,
@@ -395,6 +417,27 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         return fail!("Unknown applydp subcommand.");
     };
 
+    if !rconfig.no_headers {
+        if let Some(new_column) = &args.flag_new_column {
+            headers.push_field(new_column);
+        }
+        wtr.write_record(&headers)?;
+    }
+
+    // if there is a regex_replace operation and replacement is <NULL> case-insensitive,
+    // we set it to empty string
+    let flag_replacement = if applydp_cmd == ApplydpSubCmd::Operations
+        && ops_vec.contains(&Operations::Regex_Replace)
+        && args.flag_replacement.to_lowercase() == NULL_VALUE
+    {
+        String::new()
+    } else {
+        args.flag_replacement
+    };
+    let flag_comparand = args.flag_comparand;
+    let flag_formatstr = args.flag_formatstr;
+    let flag_new_column = args.flag_new_column;
+
     let prefer_dmy = args.flag_prefer_dmy || rconfig.get_dmy_preference();
 
     // amortize memory allocation by reusing record
@@ -447,10 +490,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                             applydp_operations(
                                 &ops_vec,
                                 &mut cell,
-                                &args.flag_comparand,
-                                &args.flag_replacement,
+                                &flag_comparand,
+                                &flag_replacement,
                             );
-                            if args.flag_new_column.is_some() {
+                            if flag_new_column.is_some() {
                                 record.push_field(&cell);
                             } else {
                                 record = replace_column_value(&record, *col_index, &cell);
@@ -462,9 +505,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         for col_index in sel.iter() {
                             record[*col_index].clone_into(&mut cell);
                             if cell.trim().is_empty() {
-                                cell = args.flag_replacement.clone();
+                                cell = flag_replacement.clone();
                             }
-                            if args.flag_new_column.is_some() {
+                            if flag_new_column.is_some() {
                                 record.push_field(&cell);
                             } else {
                                 record = replace_column_value(&record, *col_index, &cell);
@@ -479,7 +522,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                                 let parsed_date = parse_with_preference(&cell, prefer_dmy);
                                 if let Ok(format_date) = parsed_date {
                                     let formatted_date =
-                                        format_date.format(&args.flag_formatstr).to_string();
+                                        format_date.format(&flag_formatstr).to_string();
                                     if !args.flag_keep_zero_time
                                         && formatted_date.ends_with("T00:00:00+00:00")
                                     {
@@ -489,7 +532,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                                     }
                                 }
                             }
-                            if args.flag_new_column.is_some() {
+                            if flag_new_column.is_some() {
                                 record.push_field(&cell);
                             } else {
                                 record = replace_column_value(&record, *col_index, &cell);
@@ -509,7 +552,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                                 cell = formatted.to_string();
                             }
                         }
-                        if args.flag_new_column.is_some() {
+                        if flag_new_column.is_some() {
                             record.push_field(&cell);
                         } else {
                             record = replace_column_value(&record, column_index, &cell);
@@ -532,7 +575,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     Ok(wtr.flush()?)
 }
 
-// validate apply operations for required options
+// validate applydp operations for required options
 // and prepare operations enum vec
 fn validate_operations(
     operations: &Vec<&str>,
@@ -550,23 +593,27 @@ fn validate_operations(
 
     for op in operations {
         let Ok(operation) = Operations::from_str(op) else {
-            return fail_clierror!("Unknown '{op}' operation");
+            return fail_incorrectusage_clierror!("Unknown '{op}' operation");
         };
         match operation {
             Operations::Copy => {
                 if flag_new_column.is_none() {
-                    return fail!("--new_column (-c) is required for copy operation.");
+                    return fail_incorrectusage_clierror!(
+                        "--new_column (-c) is required for copy operation."
+                    );
                 }
                 copy_invokes = copy_invokes.saturating_add(1);
             },
             Operations::Mtrim | Operations::Mltrim | Operations::Mrtrim => {
                 if flag_comparand.is_empty() {
-                    return fail!("--comparand (-C) is required for match trim operations.");
+                    return fail_incorrectusage_clierror!(
+                        "--comparand (-C) is required for match trim operations."
+                    );
                 }
             },
             Operations::Regex_Replace => {
                 if flag_comparand.is_empty() || flag_replacement.is_empty() {
-                    return fail!(
+                    return fail_incorrectusage_clierror!(
                         "--comparand (-C) and --replacement (-R) are required for regex_replace \
                          operation."
                     );
@@ -584,7 +631,7 @@ fn validate_operations(
             },
             Operations::Replace => {
                 if flag_comparand.is_empty() || flag_replacement.is_empty() {
-                    return fail!(
+                    return fail_incorrectusage_clierror!(
                         "--comparand (-C) and --replacement (-R) are required for replace \
                          operation."
                     );
@@ -593,7 +640,9 @@ fn validate_operations(
             },
             Operations::Strip_Prefix | Operations::Strip_Suffix => {
                 if flag_comparand.is_empty() {
-                    return fail!("--comparand (-C) is required for strip operations.");
+                    return fail_incorrectusage_clierror!(
+                        "--comparand (-C) is required for strip operations."
+                    );
                 }
                 strip_invokes = strip_invokes.saturating_add(1);
             },
@@ -614,7 +663,7 @@ fn validate_operations(
         ops_vec.push(operation);
     }
     if copy_invokes > 1 || regex_replace_invokes > 1 || replace_invokes > 1 || strip_invokes > 1 {
-        return fail_clierror!(
+        return fail_incorrectusage_clierror!(
             "you can only use copy({copy_invokes}), regex_replace({regex_replace_invokes}), \
              replace({replace_invokes}), and strip({strip_invokes}) ONCE per operation series."
         );
@@ -689,7 +738,7 @@ fn applydp_operations(
                 *cell = regexreplace.replace_all(cell, replacement).to_string();
             },
             Operations::Round => {
-                if let Ok(num) = fast_float::parse(&cell) {
+                if let Ok(num) = cell.parse::<f64>() {
                     *cell = util::round_num(num, *ROUND_PLACES.get().unwrap());
                 }
             },
