@@ -9,7 +9,8 @@ reversed with the '--pad' flag.)
 Concatenating by rows can be done in two ways:
 
 'rows' subcommand: 
-   All CSV data must have the same number of columns and in the same order. 
+   All CSV data must have the same number of columns (unless --flexible is enabled)
+   and in the same order. 
    If you need to rearrange the columns or fix the lengths of records, use the
    'select' or 'fixlengths' commands. Also, only the headers of the *first* CSV
    data given are used. Headers in subsequent inputs are ignored. (This behavior
@@ -28,16 +29,34 @@ Usage:
     qsv cat columns [options] [<input>...]
     qsv cat --help
 
+cat arguments:
+    <input>...              The CSV file(s) to read. Use '-' for standard input.
+                            If input is a directory, all files in the directory will
+                            be read as input.
+                            If the input is a file with a '.infile-list' extension,
+                            the file will be read as a list of input files.
+                            If the input are snappy-compressed files(s), it will be
+                            decompressed automatically.
+
 cat options:
                              COLUMNS OPTION:
     -p, --pad                When concatenating columns, this flag will cause
                              all records to appear. It will pad each row if
                              other CSV data isn't long enough.
 
+                             ROWS OPTION:
+    --flexible               When concatenating rows, this flag turns off validation
+                             that the input and output CSVs have the same number of columns.
+                             This is faster, but may result in invalid CSV data.
+
                              ROWSKEY OPTIONS:
-    -g, --group              When concatenating with rowskey, use the file stem of each
-                             input file as a grouping value. A new column will be added
-                             to the beginning of each row, using --group-name.
+    -g, --group <grpkind>    When concatenating with rowskey, you can specify a grouping value
+                             which will be used as the first column in the output. This is useful
+                             when you want to know which file a row came from. Valid values are
+                             'fullpath', 'parentdirfname', 'parentdirfstem', 'fname', 'fstem' and 'none'.
+                             A new column will be added to the beginning of each row using --group-name.
+                             If 'none' is specified, no grouping column will be added.
+                             [default: none]
     -N, --group-name <arg>   When concatenating with rowskey, this flag provides the name
                              for the new grouping column. [default: file]
                              
@@ -51,8 +70,14 @@ Common options:
                            Must be a single character. (default: ,)
 "#;
 
+use std::{
+    path::{Path, PathBuf},
+    str::FromStr,
+};
+
 use indexmap::{IndexMap, IndexSet};
 use serde::Deserialize;
+use strum_macros::EnumString;
 use tempfile;
 
 use crate::{
@@ -65,17 +90,46 @@ struct Args {
     cmd_rows:        bool,
     cmd_rowskey:     bool,
     cmd_columns:     bool,
-    flag_group:      bool,
+    flag_group:      String,
     flag_group_name: String,
-    arg_input:       Vec<String>,
+    arg_input:       Vec<PathBuf>,
     flag_pad:        bool,
+    flag_flexible:   bool,
     flag_output:     Option<String>,
     flag_no_headers: bool,
     flag_delimiter:  Option<Delimiter>,
 }
 
+#[derive(Debug, EnumString, PartialEq)]
+#[strum(ascii_case_insensitive)]
+enum GroupKind {
+    FullPath,
+    ParentDirFName,
+    ParentDirFStem,
+    FName,
+    FStem,
+    None,
+}
+
+fn get_parentdir_and_file(path: &Path, stem_only: bool) -> String {
+    //safety: we know that this is a valid pathbuf
+    let file_info = if stem_only {
+        path.file_stem()
+    } else {
+        path.file_name()
+    }
+    .unwrap();
+
+    let parent_dir = path.parent().unwrap();
+
+    parent_dir.join(file_info).to_string_lossy().into_owned()
+}
+
 pub fn run(argv: &[&str]) -> CliResult<()> {
-    let args: Args = util::get_args(USAGE, argv)?;
+    let mut args: Args = util::get_args(USAGE, argv)?;
+
+    let tmpdir = tempfile::tempdir()?;
+    args.arg_input = util::process_input(args.arg_input, &tmpdir, "")?;
     if args.cmd_rows {
         args.cat_rows()
     } else if args.cmd_rowskey {
@@ -88,37 +142,76 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 }
 
 impl Args {
+    #[inline]
     fn configs(&self) -> CliResult<Vec<Config>> {
-        util::many_configs(&self.arg_input, self.flag_delimiter, self.flag_no_headers)
-            .map_err(From::from)
+        util::many_configs(
+            &self.arg_input,
+            self.flag_delimiter,
+            self.flag_no_headers,
+            // we can set flexible to true if we are using rowskey
+            // as we don't need to validate that the number of columns
+            // are the same across all files, increasing performance
+            self.flag_flexible || self.cmd_rowskey,
+        )
+        .map_err(From::from)
     }
 
     fn cat_rows(&self) -> CliResult<()> {
         let mut row = csv::ByteRecord::new();
-        let mut wtr = Config::new(&self.flag_output).writer()?;
-        for (i, conf) in self.configs()?.into_iter().enumerate() {
-            let mut rdr = conf.reader()?;
-            if i == 0 {
-                conf.write_headers(&mut rdr, &mut wtr)?;
-            }
+        let mut wtr = Config::new(&self.flag_output)
+            .flexible(self.flag_flexible)
+            .writer()?;
+        let mut rdr;
+
+        let mut configs = self.configs()?.into_iter();
+
+        // the first file is special, as it has the headers
+        // if --no-headers is set, we just write the first file
+        if let Some(conf) = configs.next() {
+            rdr = conf.reader()?;
+            conf.write_headers(&mut rdr, &mut wtr)?;
             while rdr.read_byte_record(&mut row)? {
                 wtr.write_byte_record(&row)?;
             }
         }
-        wtr.flush().map_err(From::from)
+
+        // the rest of the files are just written
+        // as fast as possible, as we don't need to
+        // worry about headers
+        for conf in configs {
+            rdr = conf.reader()?;
+            while rdr.read_byte_record(&mut row)? {
+                wtr.write_byte_record(&row)?;
+            }
+        }
+
+        Ok(wtr.flush()?)
     }
 
+    // this algorithm is largely inspired by https://github.com/vi/csvcatrow by @vi
+    // https://github.com/jqnatividad/qsv/issues/527
     fn cat_rowskey(&self) -> CliResult<()> {
-        // this algorithm is largely inspired by https://github.com/vi/csvcatrow by @vi
-        // https://github.com/jqnatividad/qsv/issues/527
+        // ahash is a faster hasher than the default one used by IndexSet and IndexMap
+        type AhashIndexSet<T> = IndexSet<T, ahash::RandomState>;
+        type AhashIndexMap<T, T2> = IndexMap<T, T2, ahash::RandomState>;
+
         if self.flag_no_headers {
-            return fail_clierror!(
+            return fail_incorrectusage_clierror!(
                 "cat rowskey does not support --no-headers, as we use column headers as keys."
             );
         }
-        let mut columns_global: IndexSet<Box<[u8]>> = IndexSet::with_capacity(32);
 
-        if self.flag_group {
+        let Ok(group_kind) = GroupKind::from_str(&self.flag_group) else {
+            return fail_incorrectusage_clierror!(
+                "Invalid grouping value `{}`. Valid values are 'fullpath', 'parentdirfname', \
+                 'parentdirfstem', 'fname', 'fstem' and 'none'.",
+                self.flag_group
+            );
+        };
+
+        let mut columns_global: AhashIndexSet<Box<[u8]>> = AhashIndexSet::default();
+
+        if group_kind != GroupKind::None {
             columns_global.insert(self.flag_group_name.as_bytes().to_vec().into_boxed_slice());
         }
 
@@ -132,12 +225,13 @@ impl Args {
         for conf in &self.configs()? {
             if conf.is_stdin() {
                 stdin_tempfilename = temp_dir.path().join("stdin");
-                let mut tmp_file = std::fs::File::create(&stdin_tempfilename)?;
+                let tmp_file = std::fs::File::create(&stdin_tempfilename)?;
+                let mut tmp_file = std::io::BufWriter::new(tmp_file);
                 std::io::copy(&mut std::io::stdin(), &mut tmp_file)?;
             }
             let mut rdr = conf.reader()?;
-            let h = rdr.byte_headers()?;
-            for field in h {
+            let header = rdr.byte_headers()?;
+            for field in header {
                 let fi = field.to_vec().into_boxed_slice();
                 columns_global.insert(fi);
             }
@@ -145,20 +239,25 @@ impl Args {
         let num_columns_global = columns_global.len();
 
         // Second pass, write all columns to a new file
-        let mut wtr = Config::new(&self.flag_output).writer()?;
+        // set flexible to true for faster writes
+        // as we know that all columns are already in columns_global and we don't need to
+        // validate that the number of columns are the same every time we write a row
+        let mut wtr = Config::new(&self.flag_output).flexible(true).writer()?;
+        let mut new_row = csv::ByteRecord::with_capacity(500, num_columns_global);
+
         for c in &columns_global {
-            wtr.write_field(c)?;
+            new_row.push_field(c);
         }
-        wtr.write_byte_record(&csv::ByteRecord::new())?;
+        wtr.write_byte_record(&new_row)?;
 
         // amortize allocations
-        #[allow(unused_assignments)]
-        let mut grouping_value = String::with_capacity(64);
+        let mut grouping_value = String::new();
         let mut conf_path;
         let mut rdr;
-        let mut h: &csv::ByteRecord;
-        let mut columns_of_this_file = IndexMap::with_capacity(num_columns_global);
-        let mut row: csv::ByteRecord;
+        let mut header: &csv::ByteRecord;
+        let mut columns_of_this_file: AhashIndexMap<Box<[u8]>, usize> = AhashIndexMap::default();
+        columns_of_this_file.reserve(num_columns_global);
+        let mut row: csv::ByteRecord = csv::ByteRecord::with_capacity(500, num_columns_global);
 
         for conf in self.configs()? {
             if conf.is_stdin() {
@@ -169,11 +268,11 @@ impl Args {
                 rdr = conf.reader()?;
                 conf_path = conf.path.clone();
             }
-            h = rdr.byte_headers()?;
+            header = rdr.byte_headers()?;
 
             columns_of_this_file.clear();
 
-            for (n, field) in h.iter().enumerate() {
+            for (n, field) in header.iter().enumerate() {
                 let fi = field.to_vec().into_boxed_slice();
                 if columns_of_this_file.contains_key(&fi) {
                     wwarn!(
@@ -185,37 +284,60 @@ impl Args {
                 columns_of_this_file.insert(fi, n);
             }
 
-            // use the file stem as the grouping value
-            // safety: we know that this is a file path and if the file path
-            // is not valid utf8, we convert it to lossy utf8
-            grouping_value = conf_path
-                .unwrap()
-                .file_stem()
-                .unwrap()
-                .to_string_lossy()
-                .to_string();
+            // safety: we know that this is a valid file path
+            let conf_pathbuf = conf_path.unwrap();
 
-            for current_row in rdr.byte_records() {
-                row = current_row?;
+            // set grouping_value
+            // safety: we know that this is a valid file path and if the file path
+            // is not utf8, we convert it to lossy utf8
+            match group_kind {
+                GroupKind::FullPath => {
+                    grouping_value.clear();
+                    grouping_value
+                        .push_str(&conf_pathbuf.canonicalize().unwrap().to_string_lossy());
+                },
+                GroupKind::ParentDirFName => {
+                    grouping_value = get_parentdir_and_file(&conf_pathbuf, false);
+                },
+                GroupKind::ParentDirFStem => {
+                    grouping_value = get_parentdir_and_file(&conf_pathbuf, true);
+                },
+                GroupKind::FName => {
+                    grouping_value.clear();
+                    grouping_value.push_str(&conf_pathbuf.file_name().unwrap().to_string_lossy());
+                },
+                GroupKind::FStem => {
+                    grouping_value.clear();
+                    grouping_value.push_str(&conf_pathbuf.file_stem().unwrap().to_string_lossy());
+                },
+                GroupKind::None => {},
+            }
+
+            let group_flag = group_kind != GroupKind::None;
+            let grouping_value_bytes = grouping_value.as_bytes();
+
+            while rdr.read_byte_record(&mut row)? {
+                new_row.clear();
                 for (col_idx, c) in columns_global.iter().enumerate() {
                     if let Some(idx) = columns_of_this_file.get(c) {
                         if let Some(d) = row.get(*idx) {
-                            wtr.write_field(d)?;
+                            new_row.push_field(d);
                         } else {
-                            wtr.write_field(b"")?;
+                            new_row.push_field(b"");
                         }
-                    } else if self.flag_group && col_idx == 0 {
+                    } else if group_flag && col_idx == 0 {
                         // we are in the first column, and --group is set
                         // so we write the grouping value
-                        wtr.write_field(&grouping_value)?;
+                        new_row.push_field(grouping_value_bytes);
                     } else {
-                        wtr.write_field(b"")?;
+                        new_row.push_field(b"");
                     }
                 }
-                wtr.write_byte_record(&csv::ByteRecord::new())?;
+                wtr.write_byte_record(&new_row)?;
             }
         }
-        wtr.flush().map_err(From::from)
+
+        Ok(wtr.flush()?)
     }
 
     fn cat_columns(&self) -> CliResult<()> {
@@ -264,6 +386,6 @@ impl Args {
             }
             wtr.write_byte_record(&record)?;
         }
-        wtr.flush().map_err(From::from)
+        Ok(wtr.flush()?)
     }
 }
