@@ -17,6 +17,9 @@ count options:
 
 Common options:
     -h, --help             Display this message
+    -f, --flexible         Do not validate if the CSV has different number of
+                           fields per record, increasing performance when counting
+                           without an index. Automatically enabled when --width is set.
     -n, --no-headers       When set, the first row will be included in
                            the count.
 "#;
@@ -31,6 +34,7 @@ struct Args {
     arg_input:           Option<String>,
     flag_human_readable: bool,
     flag_width:          bool,
+    flag_flexible:       bool,
     flag_no_headers:     bool,
 }
 
@@ -41,7 +45,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         // we also want to count the quotes when computing width
         .quoting(!args.flag_width)
         // and ignore differing column counts as well
-        .flexible(args.flag_width);
+        .flexible(args.flag_width || args.flag_flexible);
 
     // this comment left here for Logging.md example
     // log::debug!(
@@ -53,10 +57,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let (count, width) = if args.flag_width {
         count_input(&conf, args.flag_width)?
     } else {
-        match conf.indexed().unwrap_or_else(|_| {
+        let index_status = conf.indexed().unwrap_or_else(|_| {
             info!("index is stale");
             None
-        }) {
+        });
+        match index_status {
             Some(idx) => {
                 info!("index used");
                 (idx.count(), 0)
@@ -66,16 +71,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     };
 
     if args.flag_human_readable {
-        use thousands::Separable;
+        use indicatif::HumanCount;
 
         if args.flag_width {
-            woutinfo!(
-                "{};{}",
-                count.separate_with_commas(),
-                width.separate_with_commas()
-            );
+            woutinfo!("{};{}", HumanCount(count), HumanCount(width as u64));
         } else {
-            woutinfo!("{}", count.separate_with_commas());
+            woutinfo!("{}", HumanCount(count));
         }
     } else if args.flag_width {
         woutinfo!("{count};{width}");
@@ -89,20 +90,30 @@ fn count_input(
     conf: &Config,
     compute_width: bool,
 ) -> Result<(u64, usize), crate::clitypes::CliError> {
-    info!("counting...");
     let mut rdr = conf.reader()?;
     let mut count = 0_u64;
     let mut max_width = 0_usize;
-    let mut record_numfields = 0_usize;
+    let mut record_numdelimiters = 0_usize;
     let mut record = csv::ByteRecord::new();
 
     if compute_width {
+        let mut curr_width;
+
+        // read the first record to get the number of delimiters
+        // and the width of the first record
+        rdr.read_byte_record(&mut record)?;
+        max_width = record.as_slice().len();
+        count = 1;
+
+        // number of delimiters is number of fields minus 1
+        // we subtract 1 because the last field doesn't have a delimiter
+        record_numdelimiters = record.len().saturating_sub(1);
+
         while rdr.read_byte_record(&mut record)? {
             count += 1;
 
-            let curr_width = record.as_slice().len();
+            curr_width = record.as_slice().len();
             if curr_width > max_width {
-                record_numfields = record.len();
                 max_width = curr_width;
             }
         }
@@ -111,7 +122,7 @@ fn count_input(
             count += 1;
         }
     }
-    // record_numfields is a count of the delimiters
+    // record_numdelimiters is a count of the delimiters
     // which we also want to count when returning width
-    Ok((count, max_width + record_numfields))
+    Ok((count, max_width + record_numdelimiters))
 }
