@@ -2,7 +2,50 @@ static USAGE: &str = r#"
 Exports a specified Excel/ODS sheet to a CSV file.
 The first row of a sheet is assumed to be the header row.
 
-For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_excel.rs.
+Examples:
+
+Export the first sheet of an Excel file to a CSV file:
+    qsv excel input.xlsx > output.csv
+    qsv excel input.xlsx --output output.csv
+
+Export the first sheet of an ODS file to a CSV file:
+    qsv excel input.ods > output.csv
+    qsv excel input.ods -o output.csv
+
+Export the first sheet of an Excel file to a CSV file with different delimiters:
+    # semicolon
+    qsv excel input.xlsx -d ";" > output.csv
+    # tab
+    qsv excel input.xlsx -d "\t" > output.tsv
+
+Export a sheet by name (case-insensitive):
+    qsv excel --sheet "Sheet 3" input.xlsx
+
+Export a sheet by index:
+    # this exports the 3nd sheet (0-based index)
+    qsv excel -s 2 input.xlsx
+
+Export the last sheet (negative index)):
+    qsv excel -s -1 input.xlsx
+
+Export the second to last sheet:
+    qsv excel -s -2 input.xls
+
+Export a range of cells in the first sheet:
+    qsv excel --range C3:T25 input.xlsx
+
+Export a range of cells in the second sheet:
+    qsv excel --range C3:T25 -s 1 input.xlsx
+
+Export metadata for all sheets in CSV format:
+    qsv excel --metadata c input.xlsx
+
+Export metadata for all sheets in JSON format:
+    qsv excel --metadata j input.xlsx
+    # pretty-printed JSON
+    qsv excel --metadata J input.xlsx
+
+For more examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_excel.rs.
 
 Usage:
     qsv excel [options] [<input>]
@@ -13,7 +56,7 @@ Excel options:
                                Negative indices start from the end (-1 = last sheet). 
                                If the sheet cannot be found, qsv will read the first sheet.
                                [default: 0]
-    --metadata <c|j|J>         Outputs workbook metadata in CSV or JSON format: 
+    --metadata <c|j|J>         Outputs workbook metadata in CSV or JSON format:
                                  index, sheet_name, headers, num_columns, num_rows, safe_headers,
                                  safe_headers_count, unsafe_headers, unsafe_headers_count and
                                  duplicate_headers_count.
@@ -27,8 +70,8 @@ Excel options:
                                
                                In JSON(j) mode, the output is minified JSON.
                                In Pretty JSON(J) mode, the output is pretty-printed JSON.
-                               For both JSON modes, the filename and spreadsheet format are
-                               also included.
+                               For both JSON modes, the filename, the full file path, the workbook format
+                               and the number of sheets are also included.
                                
                                All other Excel options are ignored.
                                [default: none]
@@ -41,6 +84,8 @@ Excel options:
                                Note that if a date format is invalid, qsv will fall back and
                                return the date as if no date-format was specified.
      --range <range>           An Excel format range, like C:T or C3:T25, to extract to the CSV.
+     -j, --jobs <arg>          The number of jobs to run in parallel.
+                               When not set, the number of jobs is set to the number of CPUs detected.
 
 Common options:
     -h, --help                 Display this message
@@ -52,14 +97,15 @@ Common options:
 
 use std::{cmp, fmt::Write, path::PathBuf};
 
-use calamine::{open_workbook_auto, DataType, Range, Reader};
+use calamine::{open_workbook_auto, DataType, Range, Reader, SheetType};
+use indicatif::HumanCount;
 use log::info;
+use rayon::prelude::*;
 use serde::{Deserialize, Serialize};
-use thousands::Separable;
 
 use crate::{
     config::{Config, Delimiter},
-    util, CliResult,
+    util, CliError, CliResult,
 };
 
 #[derive(Deserialize)]
@@ -74,6 +120,7 @@ struct Args {
     flag_quiet:       bool,
     flag_date_format: Option<String>,
     flag_range:       String,
+    flag_jobs:        Option<usize>,
 }
 
 #[derive(PartialEq)]
@@ -88,6 +135,8 @@ enum MetadataMode {
 struct SheetMetadata {
     index:                   usize,
     name:                    String,
+    typ:                     String,
+    visible:                 String,
     headers:                 Vec<String>,
     num_columns:             usize,
     num_rows:                usize,
@@ -98,6 +147,12 @@ struct SheetMetadata {
     duplicate_headers_count: usize,
 }
 
+impl From<calamine::Error> for CliError {
+    fn from(e: calamine::Error) -> Self {
+        CliError::Other(format!("{e}"))
+    }
+}
+
 #[derive(Serialize, Deserialize)]
 struct MetadataStruct {
     filename:           String,
@@ -163,7 +218,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let path = &args.arg_input;
 
     let sce = PathBuf::from(path);
-    let mut ods_flag = false;
     let filename = sce
         .file_name()
         .and_then(std::ffi::OsStr::to_str)
@@ -175,11 +229,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .and_then(std::ffi::OsStr::to_str)
         .unwrap_or_default()
         .to_lowercase();
-    match format.as_str() {
-        "xls" | "xlsx" | "xlsm" | "xlsb" => (),
-        "ods" => ods_flag = true,
+    let ods_flag = match format.as_str() {
+        "xls" | "xlsx" | "xlsm" | "xlsb" => false,
+        "ods" => true,
         _ => {
-            return fail_clierror!(
+            return fail_incorrectusage_clierror!(
                 "\"{format}\" not supported. The excel command only supports the following file \
                  formats - xls, xlsx, xlsm, xlsb and ods."
             );
@@ -188,19 +242,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let requested_range = args.flag_range.to_lowercase();
 
-    let mut workbook = match open_workbook_auto(path) {
-        Ok(workbook) => workbook,
-        Err(e) => {
-            let es = e.to_string();
-            // password protected errors come in different flavors for Excel
-            if es.starts_with("Xls error: Cfb error")
-                || es.starts_with("Xlsx error: Zip error: invalid Zip archive")
-            {
-                return fail_clierror!("{path} may be a password-protected workbook: {e}.");
-            }
-            return fail_clierror!("Cannot open workbook: {e}.");
-        },
-    };
+    let mut workbook = open_workbook_auto(path)?;
 
     let sheet_names = workbook.sheet_names();
     if sheet_names.is_empty() {
@@ -210,7 +252,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         return fail!("No sheets found.");
     }
     let num_sheets = sheet_names.len();
-    let sheet_vec = sheet_names.to_owned();
 
     let mut wtr = Config::new(&args.flag_output)
         .flexible(args.flag_flexible)
@@ -225,7 +266,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         'J' => MetadataMode::PrettyJSON,
         'n' | 'N' => MetadataMode::None,
         _ => {
-            return fail_clierror!("Invalid mode: {}", args.flag_metadata);
+            return fail_incorrectusage_clierror!("Invalid mode: {}", args.flag_metadata);
         },
     };
 
@@ -233,18 +274,28 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let mut excelmetadata_struct = MetadataStruct {
             filename,
             canonical_filename,
-            format,
+            format: if ods_flag {
+                "ODS".to_string()
+            } else {
+                format!("Excel: {format}")
+            },
             num_sheets,
             sheet: vec![],
         };
         let mut metadata_record;
+        let sheet_vec = sheet_names;
 
         for (i, sheet_name) in sheet_vec.iter().enumerate() {
             let range = if let Some(result) = workbook.worksheet_range_at(i) {
                 match result {
                     Ok(result) => result,
                     Err(e) => {
-                        return fail_clierror!("Cannot retrieve range from {sheet_name}: {e}.");
+                        if workbook.sheets_metadata()[i].typ == SheetType::ChartSheet {
+                            // return an empty range for ChartSheet
+                            Range::empty()
+                        } else {
+                            return fail_clierror!("Cannot retrieve range from {sheet_name}: {e}.");
+                        }
                     },
                 }
             } else {
@@ -301,6 +352,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let sheetmetadata_struct = SheetMetadata {
                 index: i,
                 name: sheet_name.to_string(),
+                typ: format!("{:?}", workbook.sheets_metadata()[i].typ),
+                visible: format!("{:?}", workbook.sheets_metadata()[i].visible),
                 headers: header_vec,
                 num_columns,
                 num_rows,
@@ -319,6 +372,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 metadata_fields.extend_from_slice(&[
                     "index",
                     "sheet_name",
+                    "type",
+                    "visible",
                     "headers",
                     "num_columns",
                     "num_rows",
@@ -337,6 +392,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         sheetmetadata.index.to_string(),
                         sheetmetadata.name,
                         format!("{:?}", sheetmetadata.headers),
+                        sheetmetadata.typ,
+                        sheetmetadata.visible,
                         sheetmetadata.num_columns.to_string(),
                         sheetmetadata.num_rows.to_string(),
                         format!("{:?}", sheetmetadata.safe_headers),
@@ -379,12 +436,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         args.flag_sheet
     } else {
         // otherwise, if --sheet is a number, its a zero-based index, fetch it
-        if let Ok(sheet_index) = args.flag_sheet.parse::<i32>() {
+        if let Ok(sheet_index) = atoi_simd::parse::<i32>(args.flag_sheet.as_bytes()) {
             if sheet_index >= 0 {
                 if let Some(sheet_name) = sheet_names.get(sheet_index as usize) {
                     sheet_name.to_string()
                 } else {
-                    return fail_clierror!(
+                    return fail_incorrectusage_clierror!(
                         "sheet index {sheet_index} is greater than number of sheets {}",
                         sheet_names.len()
                     );
@@ -426,11 +483,15 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         return fail_clierror!("Cannot get sheet index for {sheet}");
     };
 
+    let sheet_type = workbook.sheets_metadata()[sheet_index].typ;
+    if sheet_type != SheetType::WorkSheet {
+        return fail_incorrectusage_clierror!(
+            "Can only export Worksheets. {sheet} is a {sheet_type:?}."
+        );
+    }
+
     let mut range = if let Some(result) = workbook.worksheet_range_at(sheet_index) {
-        match result {
-            Ok(result) => result,
-            Err(e) => return fail_clierror!("Cannot retrieve range from {sheet}: {e}"),
-        }
+        result?
     } else {
         Range::empty()
     };
@@ -453,188 +514,218 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let (row_count, col_count) = range.get_size();
 
-    if row_count > 0 {
-        let mut rows_iter = range.rows();
+    if row_count == 0 {
+        return fail_clierror!("\"{sheet}\" sheet is empty.");
+    }
+    // there are rows to export
+    let mut rows_iter = range.rows();
 
-        // use with_capacity to minimize reallocations
-        let mut record = csv::StringRecord::with_capacity(500, col_count);
-        let mut trimmed_record = csv::StringRecord::with_capacity(500, col_count);
+    // amortize allocations
+    let mut record = csv::StringRecord::with_capacity(500, col_count);
+    let mut trimmed_record = csv::StringRecord::with_capacity(500, col_count);
+    let mut col_name: String;
 
-        let mut col_name: String;
-
-        info!("exporting sheet ({sheet})... processing first row as header...");
-        let first_row = match rows_iter.next() {
-            Some(first_row) => first_row,
-            None => &[DataType::Empty],
+    // get the first row as header
+    info!("exporting sheet ({sheet})... processing first row as header...");
+    let first_row = match rows_iter.next() {
+        Some(first_row) => first_row,
+        None => &[DataType::Empty],
+    };
+    for cell in first_row {
+        col_name = match *cell {
+            DataType::String(ref s) => s.to_string(),
+            DataType::Empty => String::new(),
+            DataType::Error(ref _e) => String::new(),
+            DataType::Int(ref i) => i.to_string(),
+            DataType::DateTime(ref f) | DataType::Float(ref f) => f.to_string(),
+            DataType::Bool(ref b) => b.to_string(),
+            DataType::DateTimeIso(ref dt) => dt.to_string(),
+            DataType::DurationIso(ref d) => d.to_string(),
+            DataType::Duration(ref d) => d.to_string(),
         };
-        for cell in first_row {
-            col_name = match *cell {
-                DataType::String(ref s) => s.to_string(),
-                DataType::Empty => String::new(),
-                DataType::Error(ref _e) => String::new(),
-                DataType::Int(ref i) => i.to_string(),
-                DataType::DateTime(ref f) | DataType::Float(ref f) => f.to_string(),
-                DataType::Bool(ref b) => b.to_string(),
-                DataType::DateTimeIso(ref dt) => dt.to_string(),
-                DataType::DurationIso(ref d) => d.to_string(),
-                DataType::Duration(ref d) => d.to_string(),
-            };
-            record.push_field(&col_name);
-        }
+        record.push_field(&col_name);
+    }
 
-        let trim = args.flag_trim;
+    let trim = args.flag_trim;
 
-        if trim {
-            record.trim();
-            record.iter().for_each(|field| {
-                if field.contains('\n') {
-                    trimmed_record.push_field(&field.to_string().replace('\n', " "));
-                } else {
-                    trimmed_record.push_field(field);
-                }
-            });
-            wtr.write_record(&trimmed_record)?;
-        } else {
-            wtr.write_record(&record)?;
-        }
+    if trim {
+        record.trim();
+        record.iter().for_each(|field| {
+            if field.contains('\n') {
+                trimmed_record.push_field(&field.replace('\n', " "));
+            } else {
+                trimmed_record.push_field(field);
+            }
+        });
+        record.clone_from(&trimmed_record);
+    }
+    info!("header: {record:?}");
+    wtr.write_record(&record)?;
 
-        // process the rest of the rows
-        // first, amortize allocs by declaring mutable vars we'll use
-        // in the main processing loop here
-        let mut cell_date_flag: bool = false;
-        let mut float_val = 0_f64;
-        let mut float_flag: bool = false;
+    let date_format = if let Some(df) = args.flag_date_format {
+        df
+    } else {
+        String::new()
+    };
 
-        let date_format = if let Some(df) = args.flag_date_format {
-            df
-        } else {
-            String::new()
-        };
-        let mut work_date;
-        let mut ryu_buffer = ryu::Buffer::new();
-        let mut itoa_buffer = itoa::Buffer::new();
-        let mut formatted_date = String::new();
+    let mut rows = Vec::with_capacity(row_count);
 
-        // main processing loop
-        for row in rows_iter {
-            record.clear();
-            for cell in row {
-                match *cell {
-                    DataType::Empty => record.push_field(""),
-                    DataType::String(ref s) => record.push_field(s),
-                    DataType::Int(ref i) => {
-                        let mut buffer = itoa::Buffer::new();
-                        record.push_field(buffer.format(*i));
-                    },
-                    DataType::Float(ref f) => {
-                        float_val = *f;
-                        float_flag = true;
-                        cell_date_flag = false;
-                    },
-                    DataType::DateTime(ref f) => {
-                        float_val = *f;
-                        float_flag = true;
-                        cell_date_flag = true;
-                    },
-                    DataType::Error(ref e) => record.push_field(&format!("{e:?}")),
-                    DataType::Bool(ref b) => record.push_field(&b.to_string()),
-                    DataType::DateTimeIso(ref dt) => record.push_field(&dt.to_string()),
-                    DataType::DurationIso(ref d) => record.push_field(&d.to_string()),
-                    DataType::Duration(ref d) => record.push_field(ryu_buffer.format(*d)),
-                };
+    // process rest of the rows
+    for row in rows_iter {
+        rows.push(row);
+    }
 
-                // Dates are stored as floats in Excel, so if its a float value, we need to check
-                // if its a date. If its a date, we need to convert it to a string using the
-                // specified date format. If its not a date, we need to convert it to a string
-                // using ryu.
-                #[allow(clippy::cast_precision_loss)]
-                if float_flag {
-                    if cell_date_flag {
-                        // its a date, so convert it
-                        work_date = if float_val.fract() > 0.0 {
-                            // if it has a fractional part, then its a datetime
-                            if let Some(dt) = cell.as_datetime() {
-                                if date_format.is_empty() {
-                                    // no date format specified, so we'll just use the default
-                                    // format for the datetime
-                                    dt.to_string()
+    // set RAYON_NUM_THREADS
+    let ncpus = util::njobs(args.flag_jobs);
+
+    // set chunk_size to number of rows per core/thread
+    let chunk_size = row_count.div_ceil(ncpus);
+
+    let processed_rows: Vec<Vec<csv::StringRecord>> = rows
+        .par_chunks(chunk_size)
+        .map(|chunk| {
+            let mut record = csv::StringRecord::with_capacity(500, col_count);
+            let mut trimmed_record = csv::StringRecord::with_capacity(500, col_count);
+            let mut cell_date_flag: bool = false;
+            let mut float_val = 0_f64;
+            let mut float_flag: bool = false;
+            let mut work_date;
+            let mut ryu_buffer = ryu::Buffer::new();
+            let mut itoa_buffer = itoa::Buffer::new();
+            let mut formatted_date = String::new();
+
+            let mut processed_chunk: Vec<csv::StringRecord> = Vec::with_capacity(chunk_size);
+
+            for row in chunk {
+                for cell in *row {
+                    match *cell {
+                        DataType::Empty => record.push_field(""),
+                        DataType::String(ref s) => record.push_field(s),
+                        DataType::Int(ref i) => record.push_field(itoa_buffer.format(*i)),
+                        DataType::Float(ref f) => {
+                            float_val = *f;
+                            float_flag = true;
+                            cell_date_flag = false;
+                        },
+                        DataType::DateTime(ref f) => {
+                            float_val = *f;
+                            float_flag = true;
+                            cell_date_flag = true;
+                        },
+                        DataType::Error(ref e) => record.push_field(&format!("{e:?}")),
+                        DataType::Bool(ref b) => {
+                            record.push_field(if *b { "true" } else { "false" });
+                        },
+                        DataType::DateTimeIso(ref dt) => record.push_field(dt),
+                        DataType::DurationIso(ref d) => record.push_field(d),
+                        DataType::Duration(ref d) => record.push_field(ryu_buffer.format(*d)),
+                    };
+
+                    #[allow(clippy::cast_precision_loss)]
+                    if float_flag {
+                        if cell_date_flag {
+                            // its a date, so convert it
+                            work_date = if float_val.fract() > f64::EPSILON {
+                                // if it has a fractional part, then its a datetime
+                                if let Some(dt) = cell.as_datetime() {
+                                    if date_format.is_empty() {
+                                        // no date format specified, so we'll just use the
+                                        // default format for the datetime
+                                        dt.to_string()
+                                    } else {
+                                        // a date format was specified, so we'll use it
+                                        (formatted_date).clear();
+                                        if write!(formatted_date, "{}", dt.format(&date_format))
+                                            .is_ok()
+                                        {
+                                            // the format string was ok, so use to_string()
+                                            // to actually apply the DelayedFormat
+                                            formatted_date.to_string()
+                                        } else {
+                                            // if there was a format error, revert to the
+                                            // default format
+                                            dt.to_string()
+                                        }
+                                    }
                                 } else {
-                                    // a date format was specified, so we'll use it
-                                    (formatted_date).clear();
-                                    if write!(formatted_date, "{}", dt.format(&date_format)).is_ok()
+                                    format!("ERROR: Cannot convert {float_val} to datetime")
+                                }
+                            } else if let Some(d) = cell.as_date() {
+                                // if it has no fractional part and calamine can return it
+                                // as_date, then its a date
+                                if date_format.is_empty() {
+                                    d.to_string()
+                                } else {
+                                    formatted_date.clear();
+                                    if write!(formatted_date, "{}", d.format(&date_format)).is_ok()
                                     {
-                                        // the format string was ok, so use to_string() to
-                                        // actually apply the DelayedFormat
                                         formatted_date.to_string()
                                     } else {
-                                        // if there was a format error, revert to the default format
-                                        dt.to_string()
+                                        d.to_string()
                                     }
                                 }
                             } else {
-                                format!("ERROR: Cannot convert {float_val} to datetime")
-                            }
-                        } else if let Some(d) = cell.as_date() {
-                            // if it has no fractional part and calamine can return it as_date
-                            // then its a date
-                            if date_format.is_empty() {
-                                d.to_string()
-                            } else {
-                                formatted_date.clear();
-                                if write!(formatted_date, "{}", d.format(&date_format)).is_ok() {
-                                    formatted_date.to_string()
-                                } else {
-                                    d.to_string()
-                                }
-                            }
+                                format!("ERROR: Cannot convert {float_val} to date")
+                            };
+                            record.push_field(&work_date);
+                        // its not a date, so just push the ryu-formatted float value if its
+                        // not an integer or the candidate
+                        // integer is too big or too small to be an i64
+                        } else if float_val.fract().abs() > f64::EPSILON
+                            || float_val > i64::MAX as f64
+                            || float_val < i64::MIN as f64
+                        {
+                            record.push_field(ryu_buffer.format_finite(float_val));
                         } else {
-                            format!("ERROR: Cannot convert {float_val} to date")
-                        };
-                        record.push_field(&work_date);
-                    // its not a date, so just push the ryu-formatted float value if its not an
-                    // integer or the candidate integer is too big or too small to be an i64
-                    } else if float_val.fract().abs() > 0.0
-                        || float_val > i64::MAX as f64
-                        || float_val < i64::MIN as f64
-                    {
-                        record.push_field(ryu_buffer.format_finite(float_val));
-                    } else {
-                        // its an i64 integer. We can't use ryu to format it, because it will
-                        // be formatted as a float (have a ".0"). So we use itoa.
-                        record.push_field(itoa_buffer.format(float_val as i64));
+                            // its an i64 integer. We can't use ryu to format it, because it
+                            // will be formatted as a
+                            // float (have a ".0"). So we use itoa.
+                            record.push_field(itoa_buffer.format(float_val as i64));
+                        }
+                        // reset the float flag
+                        float_flag = false;
                     }
-                    // reset the float flag
-                    float_flag = false;
                 }
-            }
 
-            if trim {
-                record.trim();
-                trimmed_record.clear();
-                record.iter().for_each(|field| {
-                    if field.contains('\n') {
-                        trimmed_record.push_field(&field.to_string().replace('\n', " "));
-                    } else {
-                        trimmed_record.push_field(field);
-                    }
-                });
-                wtr.write_record(&trimmed_record)?;
-            } else {
-                wtr.write_record(&record)?;
+                if trim {
+                    record.trim();
+                    record.iter().for_each(|field| {
+                        if field.contains('\n') {
+                            trimmed_record.push_field(&field.replace('\n', " "));
+                        } else {
+                            trimmed_record.push_field(field);
+                        }
+                    });
+                    record.clone_from(&trimmed_record);
+                    trimmed_record.clear();
+                }
+
+                processed_chunk.push(record.clone());
+                record.clear();
             }
+            processed_chunk
+        })
+        .collect();
+
+    // rayon collect() guarantees original order,
+    // so we can just write results for each chunk in order
+    for processed_chunk in processed_rows {
+        for processed_row in processed_chunk {
+            wtr.write_record(&processed_row)?;
         }
     }
 
     wtr.flush()?;
 
     if !args.flag_quiet {
-        let end_msg = format!(
-            "{} {}-column rows exported from \"{sheet}\" sheet",
-            // don't count the header in row count
-            row_count.saturating_sub(1).separate_with_commas(),
-            col_count.separate_with_commas(),
+        winfo!(
+            "{}",
+            format!(
+                "{} {}-column rows exported from \"{sheet}\" sheet",
+                HumanCount(row_count.saturating_sub(1) as u64),
+                HumanCount(col_count as u64),
+            )
         );
-        winfo!("{end_msg}");
     }
 
     Ok(())
