@@ -7,7 +7,7 @@ Fetch is integrated with `jql` to directly parse out values from an API JSON res
 <url-column> needs to be a fully qualified URL path. Alternatively, you can dynamically
 construct URLs for each CSV record with the --url-template option (see Examples below).
 
-To use a proxy, please set env vars HTTP_PROXY and HTTPS_PROXY
+To use a proxy, please set env vars HTTP_PROXY, HTTPS_PROXY or ALL_PROXY
 (e.g. export HTTPS_PROXY=socks5://127.0.0.1:1086).
 
 Fetch caches responses to minimize traffic and maximize performance. By default, it uses
@@ -178,7 +178,8 @@ Common options:
                                appear as the header row in the output.
     -d, --delimiter <arg>      The field delimiter for reading CSV data.
                                Must be a single character. (default: ,)
-    -p, --progressbar          Show progress bars. Not valid for stdin.
+    -p, --progressbar          Show progress bars. Will also show the cache hit rate upon completion.
+                               Not valid for stdin.
 "#;
 
 use std::{fs, num::NonZeroU32, sync::OnceLock, thread, time};
@@ -197,7 +198,7 @@ use governor::{
 use indicatif::{HumanCount, MultiProgress, ProgressBar, ProgressDrawTarget};
 use log::{
     debug, error, info, log_enabled, warn,
-    Level::{Debug, Info, Trace, Warn},
+    Level::{Debug, Trace, Warn},
 };
 use rand::Rng;
 use redis;
@@ -320,7 +321,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let redis_client = match redis::Client::open(conn_str.to_string()) {
             Ok(rc) => rc,
             Err(e) => {
-                return fail_clierror!(r#"Invalid Redis connection string "{conn_str}": {e:?}"#)
+                return fail_incorrectusage_clierror!(
+                    r#"Invalid Redis connection string "{conn_str}": {e:?}"#
+                )
             },
         };
 
@@ -367,7 +370,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         true
     } else {
         if args.flag_pretty {
-            return fail!("The --pretty option requires the --new-column option.");
+            return fail_incorrectusage_clierror!(
+                "The --pretty option requires the --new-column option."
+            );
         }
         false
     };
@@ -385,7 +390,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut dynfmt_url_template = String::new();
     if let Some(ref url_template) = args.flag_url_template {
         if args.flag_no_headers {
-            return fail!("--url-template option requires column headers.");
+            return fail_incorrectusage_clierror!("--url-template option requires column headers.");
         }
         let str_headers = rdr.headers()?.clone();
         let mut dynfmt_fields = Vec::with_capacity(10); // 10 is a reasonable default to save allocs
@@ -414,9 +419,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let rate_limit = match args.flag_rate_limit {
         0 => NonZeroU32::new(u32::MAX).unwrap(),
         1..=1000 => NonZeroU32::new(args.flag_rate_limit).unwrap(),
-        _ => return fail!("Rate Limit should be between 0 to 1000 queries per second."),
+        _ => {
+            return fail_incorrectusage_clierror!(
+                "Rate Limit should be between 0 to 1000 queries per second."
+            )
+        },
     };
-    info!("RATE LIMIT: {rate_limit}");
+    debug!("RATE LIMIT: {rate_limit}");
 
     let http_headers: HeaderMap = {
         let mut map = HeaderMap::with_capacity(args.flag_http_header.len() + 1);
@@ -424,7 +433,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let vals: Vec<&str> = header.split(':').collect();
 
             if vals.len() != 2 {
-                return fail_clierror!(
+                return fail_incorrectusage_clierror!(
                     "{vals:?} is not a valid key-value pair. Expecting a key and a value \
                      separated by a colon."
                 );
@@ -435,14 +444,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let header_name: HeaderName =
                 match HeaderName::from_lowercase(k.to_lowercase().as_bytes()) {
                     Ok(h) => h,
-                    Err(e) => return fail_clierror!("Invalid header name: {e}"),
+                    Err(e) => return fail_incorrectusage_clierror!("Invalid header name: {e}"),
                 };
 
             // allocate new String for header value to put into map
             let v: String = String::from(vals[1].trim());
             let header_val: HeaderValue = match HeaderValue::from_str(v.as_str()) {
                 Ok(v) => v,
-                Err(e) => return fail_clierror!("Invalid header value: {e}"),
+                Err(e) => return fail_incorrectusage_clierror!("Invalid header value: {e}"),
             };
 
             map.append(header_name, header_val);
@@ -879,6 +888,27 @@ fn get_redis_response(
     }))
 }
 
+/// Get the value of a header from the ratelimit API response
+/// given its name or its x-name
+pub fn get_ratelimit_header_value<'a>(
+    api_respheader: &'a HeaderMap,
+    header: &'a str,
+    x_header: &'a str,
+) -> Option<&'a HeaderValue> {
+    api_respheader
+        .get(header)
+        .or_else(|| api_respheader.get(x_header))
+}
+
+/// Parse the value of a header from the ratelimit API response
+/// return sentinel value if the header is not found or if the value is not a valid u64
+/// return 1 if the value is 0
+pub fn parse_ratelimit_header_value(value: Option<&HeaderValue>, sentinel_value: u64) -> u64 {
+    value.map_or(sentinel_value, |v| {
+        atoi_simd::parse_pos::<u64>(v.to_str().unwrap().as_bytes()).unwrap_or(1)
+    })
+}
+
 #[inline]
 fn get_response(
     url: &str,
@@ -920,7 +950,7 @@ fn get_response(
             };
         },
     };
-    info!("Using URL: {valid_url}");
+    debug!("Using URL: {valid_url}");
 
     // wait until RateLimiter gives Okay or we timeout
     let mut limiter_total_wait: u64;
@@ -933,6 +963,8 @@ fn get_response(
     let mut api_status;
     let mut api_respheader = HeaderMap::new();
 
+    let debug_flag = log_enabled!(Debug);
+
     // request with --max-retries
     'retry: loop {
         // check the rate-limiter
@@ -941,15 +973,14 @@ fn get_response(
             limiter_total_wait += MINIMUM_WAIT_MS;
             thread::sleep(MIN_WAIT);
             if limiter_total_wait > governor_timeout_ms {
-                info!("rate limit timed out after {limiter_total_wait} ms");
+                debug!("rate limit timed out after {limiter_total_wait} ms");
                 break;
             } else if limiter_total_wait == MINIMUM_WAIT_MS {
-                info!("throttling...");
+                debug!("throttling...");
             }
         }
-        if log_enabled!(Info) && limiter_total_wait > 0 && limiter_total_wait <= governor_timeout_ms
-        {
-            info!("throttled for {limiter_total_wait} ms");
+        if debug_flag && limiter_total_wait > 0 && limiter_total_wait <= governor_timeout_ms {
+            debug!("throttled for {limiter_total_wait} ms");
         }
 
         // send the actual request
@@ -1025,40 +1056,31 @@ fn get_response(
                     || api_respheader.contains_key("x-ratelimit-limit")
                     || api_respheader.contains_key("retry-after")))
         {
-            let mut ratelimit_remaining = api_respheader.get("ratelimit-remaining");
-            if ratelimit_remaining.is_none() {
-                let temp_var = api_respheader.get("x-ratelimit-remaining");
-                if temp_var.is_some() {
-                    ratelimit_remaining = temp_var;
-                }
-            }
-            let mut ratelimit_reset = api_respheader.get("ratelimit-reset");
-            if ratelimit_reset.is_none() {
-                let temp_var = api_respheader.get("x-ratelimit-reset");
-                if temp_var.is_some() {
-                    ratelimit_reset = temp_var;
-                }
-            }
+            let ratelimit_remaining = get_ratelimit_header_value(
+                &api_respheader,
+                "ratelimit-remaining",
+                "x-ratelimit-remaining",
+            );
+
+            let ratelimit_reset =
+                get_ratelimit_header_value(&api_respheader, "ratelimit-reset", "x-ratelimit-reset");
 
             // some APIs add the "-second" suffix to ratelimit fields
-            let mut ratelimit_remaining_sec = api_respheader.get("ratelimit-remaining-second");
-            if ratelimit_remaining_sec.is_none() {
-                let temp_var = api_respheader.get("x-ratelimit-remaining-second");
-                if temp_var.is_some() {
-                    ratelimit_remaining_sec = temp_var;
-                }
-            }
-            let mut ratelimit_reset_sec = api_respheader.get("ratelimit-reset-second");
-            if ratelimit_reset_sec.is_none() {
-                let temp_var = api_respheader.get("x-ratelimit-reset-second");
-                if temp_var.is_some() {
-                    ratelimit_reset_sec = temp_var;
-                }
-            }
+            let ratelimit_remaining_sec = get_ratelimit_header_value(
+                &api_respheader,
+                "ratelimit-remaining-second",
+                "x-ratelimit-remaining-second",
+            );
+
+            let ratelimit_reset_sec = get_ratelimit_header_value(
+                &api_respheader,
+                "ratelimit-reset-second",
+                "x-ratelimit-reset-second",
+            );
 
             let retry_after = api_respheader.get("retry-after");
 
-            if log_enabled!(Debug) {
+            if debug_flag {
                 let rapidapi_proxy_response = api_respheader.get("X-RapidAPI-Proxy-Response");
 
                 debug!(
@@ -1071,40 +1093,13 @@ fn get_response(
 
             // if there's a ratelimit_remaining field in the response header, get it
             // otherwise, set remaining to sentinel value 9999
-            let remaining = ratelimit_remaining.map_or_else(
-                || {
-                    if let Some(ratelimit_remaining_sec) = ratelimit_remaining_sec {
-                        let remaining_sec_str = ratelimit_remaining_sec.to_str().unwrap();
-                        remaining_sec_str.parse::<u64>().unwrap_or(1)
-                    } else {
-                        9999_u64
-                    }
-                },
-                |ratelimit_remaining| {
-                    let remaining_str = ratelimit_remaining.to_str().unwrap();
-                    remaining_str.parse::<u64>().unwrap_or(1)
-                },
-            );
+            let remaining =
+                parse_ratelimit_header_value(ratelimit_remaining.or(ratelimit_remaining_sec), 9999);
 
             // if there's a ratelimit_reset field in the response header, get it
             // otherwise, set reset to sentinel value 0
-            let mut reset_secs = ratelimit_reset.map_or_else(
-                || {
-                    if let Some(ratelimit_reset_sec) = ratelimit_reset_sec {
-                        let reset_sec_str = ratelimit_reset_sec.to_str().unwrap();
-                        reset_sec_str.parse::<u64>().unwrap_or(1)
-                    } else {
-                        // sleep for at least 1 second if we get an API error,
-                        // even if there is no ratelimit_reset header,
-                        // otherwise return 0
-                        u64::from(error_flag)
-                    }
-                },
-                |ratelimit_reset| {
-                    let reset_str = ratelimit_reset.to_str().unwrap();
-                    reset_str.parse::<u64>().unwrap_or(1)
-                },
-            );
+            let mut reset_secs =
+                parse_ratelimit_header_value(ratelimit_reset.or(ratelimit_reset_sec), 0);
 
             // if there's a retry_after field in the response header, get it
             // and set reset to it
@@ -1115,7 +1110,8 @@ fn get_response(
                 // wait before retrying, which is a valid value
                 // however, we don't want to do date-parsing here, so we just
                 // wait timeout_secs seconds before retrying
-                reset_secs = retry_str.parse::<u64>().unwrap_or(timeout_secs);
+                reset_secs =
+                    atoi_simd::parse_pos::<u64>(retry_str.as_bytes()).unwrap_or(timeout_secs);
             }
 
             // if reset_secs > timeout, then just time out and skip the retries
@@ -1136,7 +1132,7 @@ fn get_response(
                 let pause_time =
                     (reset_secs * 1001) + (retries as u64 * rand::thread_rng().gen_range(10..30));
 
-                info!(
+                debug!(
                     "sleeping for {pause_time} ms until ratelimit is reset/retry_after has elapsed"
                 );
                 thread::sleep(time::Duration::from_millis(pause_time));
@@ -1147,7 +1143,7 @@ fn get_response(
                 break 'retry;
             }
             retries += 1;
-            info!("retrying {retries}...");
+            debug!("retrying {retries}...");
         } else {
             // there's no request error or ratelimits nor retry-after
             break 'retry;
@@ -1189,7 +1185,12 @@ impl From<jql_runner::errors::JqlRunnerError> for CliError {
     }
 }
 
-#[inline]
+#[cached(
+    size = 2_000_000,
+    key = "String",
+    convert = r#"{ format!("{}-{}", json, query) }"#,
+    result = true
+)]
 pub fn process_jql(json: &str, query: &str) -> CliResult<String> {
     let mut deserializer = serde_json::Deserializer::from_str(json);
 
