@@ -8,7 +8,7 @@ Fetchpost is integrated with `jql` to directly parse out values from an API JSON
 <url-column> needs to be a fully qualified URL path. It can be specified as a column name
 from which the URL value will be retrieved for each record, or as the URL literal itself.
 
-To use a proxy, please set env vars HTTP_PROXY and HTTPS_PROXY
+To use a proxy, please set env vars HTTP_PROXY, HTTPS_PROXY or ALL_PROXY
 (e.g. export HTTPS_PROXY=socks5://127.0.0.1:1086).
 
 Fetchpost caches responses to minimize traffic and maximize performance. By default, it uses
@@ -169,7 +169,8 @@ Common options:
                                appear as the header row in the output.
     -d, --delimiter <arg>      The field delimiter for reading CSV data.
                                Must be a single character. (default: ,)
-    -p, --progressbar          Show progress bars. Not valid for stdin.
+    -p, --progressbar          Show progress bars. Will also show the cache hit rate upon completion.
+                               Not valid for stdin.
 "#;
 
 use std::{fs, io::Write, num::NonZeroU32, sync::OnceLock, thread, time};
@@ -188,7 +189,7 @@ use governor::{
 use indicatif::{HumanCount, MultiProgress, ProgressBar, ProgressDrawTarget};
 use log::{
     debug, error, info, log_enabled, warn,
-    Level::{Debug, Info, Trace, Warn},
+    Level::{Debug, Trace, Warn},
 };
 use rand::Rng;
 use redis;
@@ -204,7 +205,7 @@ use simdutf8::basic::from_utf8;
 use url::Url;
 
 use crate::{
-    cmd::fetch::process_jql,
+    cmd::fetch::{get_ratelimit_header_value, parse_ratelimit_header_value, process_jql},
     config::{Config, Delimiter},
     select::SelectColumns,
     util, CliError, CliResult,
@@ -313,7 +314,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let redis_client = match redis::Client::open(conn_str.to_string()) {
             Ok(rc) => rc,
             Err(e) => {
-                return fail_clierror!(r#"Invalid Redis connection string "{conn_str}": {e:?}"#)
+                return fail_incorrectusage_clierror!(
+                    r#"Invalid Redis connection string "{conn_str}": {e:?}"#
+                )
             },
         };
 
@@ -360,7 +363,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         true
     } else {
         if args.flag_pretty {
-            return fail!("The --pretty option requires the --new-column option.");
+            return fail_incorrectusage_clierror!(
+                "The --pretty option requires the --new-column option."
+            );
         }
         false
     };
@@ -398,7 +403,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let rate_limit = match args.flag_rate_limit {
         0 => NonZeroU32::new(u32::MAX).unwrap(),
         1..=1000 => NonZeroU32::new(args.flag_rate_limit).unwrap(),
-        _ => return fail!("Rate Limit should be between 0 to 1000 queries per second."),
+        _ => {
+            return fail_incorrectusage_clierror!(
+                "Rate Limit should be between 0 to 1000 queries per second."
+            )
+        },
     };
     info!("RATE LIMIT: {rate_limit}");
 
@@ -408,7 +417,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let vals: Vec<&str> = header.split(':').collect();
 
             if vals.len() != 2 {
-                return fail_clierror!(
+                return fail_incorrectusage_clierror!(
                     "{vals:?} is not a valid key-value pair. Expecting a key and a value \
                      separated by a colon."
                 );
@@ -419,14 +428,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let header_name: HeaderName =
                 match HeaderName::from_lowercase(k.to_lowercase().as_bytes()) {
                     Ok(h) => h,
-                    Err(e) => return fail_clierror!("Invalid header name: {e}"),
+                    Err(e) => return fail_incorrectusage_clierror!("Invalid header name: {e}"),
                 };
 
             // allocate new String for header value to put into map
             let v: String = String::from(vals[1].trim());
             let header_val: HeaderValue = match HeaderValue::from_str(v.as_str()) {
                 Ok(v) => v,
-                Err(e) => return fail_clierror!("Invalid header value: {e}"),
+                Err(e) => return fail_incorrectusage_clierror!("Invalid header value: {e}"),
             };
 
             map.append(header_name, header_val);
@@ -937,7 +946,7 @@ fn get_response(
             };
         },
     };
-    info!("Using URL: {valid_url}");
+    debug!("Using URL: {valid_url}");
 
     // wait until RateLimiter gives Okay or we timeout
     let mut limiter_total_wait: u64;
@@ -950,6 +959,8 @@ fn get_response(
     let mut api_status;
     let mut api_respheader = HeaderMap::new();
 
+    let debug_flag = log_enabled!(Debug);
+
     // request with --max-retries
     'retry: loop {
         // check the rate-limiter
@@ -958,15 +969,14 @@ fn get_response(
             limiter_total_wait += MINIMUM_WAIT_MS;
             thread::sleep(MIN_WAIT);
             if limiter_total_wait > governor_timeout_ms {
-                info!("rate limit timed out after {limiter_total_wait} ms");
+                debug!("rate limit timed out after {limiter_total_wait} ms");
                 break;
             } else if limiter_total_wait == MINIMUM_WAIT_MS {
-                info!("throttling...");
+                debug!("throttling...");
             }
         }
-        if log_enabled!(Info) && limiter_total_wait > 0 && limiter_total_wait <= governor_timeout_ms
-        {
-            info!("throttled for {limiter_total_wait} ms");
+        if debug_flag && limiter_total_wait > 0 && limiter_total_wait <= governor_timeout_ms {
+            debug!("throttled for {limiter_total_wait} ms");
         }
 
         // send the actual request
@@ -1056,40 +1066,31 @@ fn get_response(
                     || api_respheader.contains_key("x-ratelimit-limit")
                     || api_respheader.contains_key("retry-after")))
         {
-            let mut ratelimit_remaining = api_respheader.get("ratelimit-remaining");
-            if ratelimit_remaining.is_none() {
-                let temp_var = api_respheader.get("x-ratelimit-remaining");
-                if temp_var.is_some() {
-                    ratelimit_remaining = temp_var;
-                }
-            }
-            let mut ratelimit_reset = api_respheader.get("ratelimit-reset");
-            if ratelimit_reset.is_none() {
-                let temp_var = api_respheader.get("x-ratelimit-reset");
-                if temp_var.is_some() {
-                    ratelimit_reset = temp_var;
-                }
-            }
+            let ratelimit_remaining = get_ratelimit_header_value(
+                &api_respheader,
+                "ratelimit-remaining",
+                "x-ratelimit-remaining",
+            );
+
+            let ratelimit_reset =
+                get_ratelimit_header_value(&api_respheader, "ratelimit-reset", "x-ratelimit-reset");
 
             // some APIs add the "-second" suffix to ratelimit fields
-            let mut ratelimit_remaining_sec = api_respheader.get("ratelimit-remaining-second");
-            if ratelimit_remaining_sec.is_none() {
-                let temp_var = api_respheader.get("x-ratelimit-remaining-second");
-                if temp_var.is_some() {
-                    ratelimit_remaining_sec = temp_var;
-                }
-            }
-            let mut ratelimit_reset_sec = api_respheader.get("ratelimit-reset-second");
-            if ratelimit_reset_sec.is_none() {
-                let temp_var = api_respheader.get("x-ratelimit-reset-second");
-                if temp_var.is_some() {
-                    ratelimit_reset_sec = temp_var;
-                }
-            }
+            let ratelimit_remaining_sec = get_ratelimit_header_value(
+                &api_respheader,
+                "ratelimit-remaining-second",
+                "x-ratelimit-remaining-second",
+            );
+
+            let ratelimit_reset_sec = get_ratelimit_header_value(
+                &api_respheader,
+                "ratelimit-reset-second",
+                "x-ratelimit-reset-second",
+            );
 
             let retry_after = api_respheader.get("retry-after");
 
-            if log_enabled!(Debug) {
+            if debug_flag {
                 let rapidapi_proxy_response = api_respheader.get("X-RapidAPI-Proxy-Response");
 
                 debug!(
@@ -1102,40 +1103,13 @@ fn get_response(
 
             // if there's a ratelimit_remaining field in the response header, get it
             // otherwise, set remaining to sentinel value 9999
-            let remaining = ratelimit_remaining.map_or_else(
-                || {
-                    if let Some(ratelimit_remaining_sec) = ratelimit_remaining_sec {
-                        let remaining_sec_str = ratelimit_remaining_sec.to_str().unwrap();
-                        remaining_sec_str.parse::<u64>().unwrap_or(1)
-                    } else {
-                        9999_u64
-                    }
-                },
-                |ratelimit_remaining| {
-                    let remaining_str = ratelimit_remaining.to_str().unwrap();
-                    remaining_str.parse::<u64>().unwrap_or(1)
-                },
-            );
+            let remaining =
+                parse_ratelimit_header_value(ratelimit_remaining.or(ratelimit_remaining_sec), 9999);
 
             // if there's a ratelimit_reset field in the response header, get it
             // otherwise, set reset to sentinel value 0
-            let mut reset_secs = ratelimit_reset.map_or_else(
-                || {
-                    if let Some(ratelimit_reset_sec) = ratelimit_reset_sec {
-                        let reset_sec_str = ratelimit_reset_sec.to_str().unwrap();
-                        reset_sec_str.parse::<u64>().unwrap_or(1)
-                    } else {
-                        // sleep for at least 1 second if we get an API error,
-                        // even if there is no ratelimit_reset header,
-                        // otherwise return 0
-                        u64::from(error_flag)
-                    }
-                },
-                |ratelimit_reset| {
-                    let reset_str = ratelimit_reset.to_str().unwrap();
-                    reset_str.parse::<u64>().unwrap_or(1)
-                },
-            );
+            let mut reset_secs =
+                parse_ratelimit_header_value(ratelimit_reset.or(ratelimit_reset_sec), 0);
 
             // if there's a retry_after field in the response header, get it
             // and set reset to it
@@ -1166,7 +1140,7 @@ fn get_response(
                 // more breathing room before we hit it again
                 let pause_time =
                     (reset_secs * 1001) + (retries as u64 * rand::thread_rng().gen_range(10..30));
-                info!(
+                debug!(
                     "sleeping for {pause_time} ms until ratelimit is reset/retry_after has elapsed"
                 );
 
@@ -1178,7 +1152,7 @@ fn get_response(
                 break 'retry;
             }
             retries += 1;
-            info!("retrying {retries}...");
+            debug!("retrying {retries}...");
         } else {
             // there's no request error or ratelimits nor retry-after
             break 'retry;
