@@ -16,7 +16,7 @@ For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_join
 
 Usage:
     qsv joinp [options] <columns1> <input1> <columns2> <input2>
-    qsv joinp --cross <input1> <input2>
+    qsv joinp --cross [--validate <arg>] <input1> <input2> [--output <file>]
     qsv joinp --help
 
 joinp arguments:
@@ -96,6 +96,7 @@ joinp options:
                            join, except we match on nearest key rather than
                            equal keys. Note that both CSV data sets will be SORTED
                            AUTOMATICALLY on the join columns.
+                           Particularly useful for time series data.
     --left_by <arg>        Do an 'asof_by' join - a special implementation of the asof
                            join that searches for the nearest keys within a subgroup
                            set by the asof_by columns. This specifies the column/s for
@@ -143,6 +144,11 @@ joinp options:
                              (e.g. 2022-02-29 -> 2022-02-28) instead of erroring.
 
                              OUTPUT FORMAT OPTIONS:
+   --sql-filter <SQL>        The SQL expression to apply against the join result.
+                             Ordinarily used to select columns and filter rows from 
+                             the join result. Be sure to select from the "join_result"
+                             table when formulating the SQL expression.
+                             (e.g. "select c1, c2 as colname from join_result where c2 > 20")
    --datetime-format <fmt>   The datetime format to use writing datetimes.
                              See https://docs.rs/chrono/latest/chrono/format/strftime/index.html
                              for the list of valid format specifiers.
@@ -151,7 +157,7 @@ joinp options:
    --float-precision <arg>   The number of digits of precision to use when writing floats.
                              (default: 6)
    --null-value <arg>        The string to use when writing null values.
-                             (default: <empty string>)                             
+                             (default: <empty string>)
 
 Common options:
     -h, --help             Display this message
@@ -162,6 +168,7 @@ Common options:
 "#;
 
 use std::{
+    env,
     fs::File,
     io::{self, Write},
     path::Path,
@@ -170,16 +177,17 @@ use std::{
 
 use polars::{
     datatypes::AnyValue,
-    frame::hash_join::{JoinType, JoinValidation},
     prelude::{
-        AsOfOptions, AsofStrategy, CsvWriter, LazyCsvReader, LazyFileListReader, LazyFrame,
-        SerWriter, SortOptions,
+        AsOfOptions, AsofStrategy, CsvWriter, IntoLazy, JoinType, JoinValidation, LazyCsvReader,
+        LazyFileListReader, LazyFrame, SerWriter, SortOptions,
     },
+    sql::SQLContext,
 };
 use serde::Deserialize;
 use smartstring;
+use tempfile::tempdir;
 
-use crate::{config::Delimiter, util, CliError, CliResult};
+use crate::{cmd::sqlp::compress_output_if_needed, config::Delimiter, util, CliError, CliResult};
 
 #[derive(Deserialize)]
 struct Args {
@@ -207,6 +215,7 @@ struct Args {
     flag_right_by:         Option<String>,
     flag_strategy:         Option<String>,
     flag_tolerance:        Option<String>,
+    flag_sql_filter:       Option<String>,
     flag_datetime_format:  Option<String>,
     flag_date_format:      Option<String>,
     flag_time_format:      Option<String>,
@@ -232,21 +241,24 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     if args.flag_asof {
         args.flag_try_parsedates = true;
     }
+
+    let tmpdir = tempdir()?;
     let join = args.new_join(
         args.flag_try_parsedates,
         args.flag_infer_len,
         args.flag_low_memory,
         args.flag_ignore_errors,
+        &tmpdir,
     )?;
 
     // safety: flag_validate is always is_some() as it has a default value
     args.flag_validate = Some(args.flag_validate.unwrap().to_lowercase());
     let validation = match args.flag_validate.as_deref() {
-        Some("none") | None => JoinValidation::ManyToMany,
+        Some("manytomany" | "none") | None => JoinValidation::ManyToMany,
         Some("onetomany") => JoinValidation::OneToMany,
         Some("manytoone") => JoinValidation::ManyToOne,
         Some("onetoone") => JoinValidation::OneToOne,
-        Some(s) => return fail_clierror!("Invalid join validation: {s}"),
+        Some(s) => return fail_incorrectusage_clierror!("Invalid join validation: {s}"),
     };
 
     let join_shape: (usize, usize) = match (
@@ -261,7 +273,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         (true, false, false, false, false, false) => join.run(JoinType::Left, validation, false),
         (false, true, false, false, false, false) => join.run(JoinType::Anti, validation, false),
         (false, false, true, false, false, false) => join.run(JoinType::Semi, validation, false),
-        (false, false, false, true, false, false) => join.run(JoinType::Outer, validation, false),
+        (false, false, false, true, false, false) => {
+            join.run(JoinType::Outer { coalesce: true }, validation, false)
+        },
         (false, false, false, false, true, false) => join.run(JoinType::Cross, validation, false),
         (false, false, false, false, false, true) => {
             // safety: flag_strategy is always is_some() as it has a default value
@@ -280,10 +294,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
             if strategy == AsofStrategy::Nearest {
                 if let Some(ref tolerance) = args.flag_tolerance {
-                    // If the tolerance is an integer, it is tolerance number of rows.
+                    // If the tolerance is a positive integer, it is tolerance number of rows.
                     // Otherwise, it is a tolerance date language spec.
-                    if let Ok(numeric_tolerance) = tolerance.parse::<i64>() {
-                        asof_options.tolerance = Some(AnyValue::Int64(numeric_tolerance));
+                    if let Ok(numeric_tolerance) = atoi_simd::parse_pos::<u64>(tolerance.as_bytes())
+                    {
+                        asof_options.tolerance = Some(AnyValue::UInt64(numeric_tolerance));
                     } else {
                         asof_options.tolerance_str = Some(tolerance.into());
                     }
@@ -309,7 +324,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             }
             join.run(JoinType::AsOf(asof_options), validation, true)
         },
-        _ => fail!("Please pick exactly one join operation."),
+        _ => fail_incorrectusage_clierror!("Please pick exactly one join operation."),
     }?;
 
     if !args.flag_quiet {
@@ -328,6 +343,7 @@ struct JoinStruct {
     delim:            u8,
     streaming:        bool,
     no_optimizations: bool,
+    sql_filter:       Option<String>,
     datetime_format:  Option<String>,
     date_format:      Option<String>,
     time_format:      Option<String>,
@@ -357,7 +373,7 @@ impl JoinStruct {
         let right_selcols_len = right_selcols.len();
 
         if left_selcols_len != right_selcols_len {
-            return fail_clierror!(
+            return fail_incorrectusage_clierror!(
                 "Both columns1 ({left_selcols:?}) and columns2 ({right_selcols:?}) must specify \
                  the same number of columns ({left_selcols_len } != {right_selcols_len})."
             );
@@ -380,18 +396,20 @@ impl JoinStruct {
                 comm_subplan_elim:   true,
                 comm_subexpr_elim:   true,
                 streaming:           self.streaming,
+                fast_projection:     true,
+                eager:               false,
             }
         };
         log::debug!("Optimization state: {optimization_state:?}");
 
-        let mut join_results = if jointype == JoinType::Cross {
+        let join_results = if jointype == JoinType::Cross {
             self.left_lf
                 .with_optimizations(optimization_state)
                 .join_builder()
                 .with(self.right_lf.with_optimizations(optimization_state))
                 .how(JoinType::Cross)
+                .allow_parallel(true)
                 .validate(validation)
-                .force_parallel(true)
                 .finish()
                 .collect()?
         } else {
@@ -408,15 +426,24 @@ impl JoinStruct {
                 .left_on(left_selcols)
                 .right_on(right_selcols)
                 .how(jointype)
+                .allow_parallel(true)
                 .validate(validation)
-                .force_parallel(true)
                 .finish()
                 .collect()?
         };
 
+        let mut results_df = if let Some(sql_filter) = &self.sql_filter {
+            let mut ctx = SQLContext::new();
+            ctx.register("join_result", join_results.lazy());
+            ctx.execute(sql_filter)
+                .and_then(polars::prelude::LazyFrame::collect)?
+        } else {
+            join_results
+        };
+
         // no need to use buffered writer here, as CsvWriter already does that
         let mut out_writer = match self.output {
-            Some(output_file) => {
+            Some(ref output_file) => {
                 let path = Path::new(&output_file);
                 Box::new(File::create(path).unwrap()) as Box<dyn Write>
             },
@@ -424,17 +451,20 @@ impl JoinStruct {
         };
 
         // shape is the number of rows and columns
-        let join_shape = join_results.shape();
+        let join_shape = results_df.shape();
 
         CsvWriter::new(&mut out_writer)
-            .has_header(true)
-            .with_delimiter(self.delim)
+            .include_header(true)
+            .with_separator(self.delim)
             .with_datetime_format(self.datetime_format)
             .with_date_format(self.date_format)
             .with_time_format(self.time_format)
             .with_float_precision(self.float_precision)
             .with_null_value(self.null_value)
-            .finish(&mut join_results)?;
+            .include_bom(util::get_envvar_flag("QSV_OUTPUT_BOM"))
+            .finish(&mut results_df)?;
+
+        compress_output_if_needed(self.output)?;
 
         Ok(join_shape)
     }
@@ -442,11 +472,12 @@ impl JoinStruct {
 
 impl Args {
     fn new_join(
-        &self,
+        &mut self,
         try_parsedates: bool,
         infer_len: usize,
         low_memory: bool,
         ignore_errors: bool,
+        tmpdir: &tempfile::TempDir,
     ) -> CliResult<JoinStruct> {
         let delim = if let Some(delimiter) = self.flag_delimiter {
             delimiter.as_byte()
@@ -454,36 +485,73 @@ impl Args {
             b','
         };
 
+        let comment_char = if let Ok(comment_char) = env::var("QSV_COMMENT_CHAR") {
+            Some(comment_char)
+        } else {
+            None
+        };
+
         let num_rows = if infer_len == 0 {
             None
         } else {
             Some(infer_len)
         };
 
-        let mut left_lf = LazyCsvReader::new(&self.arg_input1)
-            .has_header(true)
-            .with_missing_is_null(self.flag_nulls)
-            .with_delimiter(delim)
-            .with_infer_schema_length(num_rows)
-            .with_try_parse_dates(try_parsedates)
-            .low_memory(low_memory)
-            .with_ignore_errors(ignore_errors)
-            .finish()?;
+        // check if the input files exist
+        let input1_path = Path::new(&self.arg_input1);
+        if !input1_path.exists() {
+            return fail_clierror!("Input file {} does not exist.", self.arg_input1);
+        }
+        let input2_path = Path::new(&self.arg_input2);
+        if !input2_path.exists() {
+            return fail_clierror!("Input file {} does not exist.", self.arg_input2);
+        }
+
+        let mut left_lf = {
+            // check if the left input file is snappy compressed
+            // if so, we need to decompress it first
+            if input1_path.extension().and_then(std::ffi::OsStr::to_str) == Some("sz") {
+                let decompressed_path =
+                    util::decompress_snappy_file(&input1_path.to_path_buf(), tmpdir)?;
+                self.arg_input1 = decompressed_path;
+            }
+
+            LazyCsvReader::new(&self.arg_input1)
+                .has_header(true)
+                .with_missing_is_null(self.flag_nulls)
+                .with_comment_prefix(comment_char.as_deref())
+                .with_separator(delim)
+                .with_infer_schema_length(num_rows)
+                .with_try_parse_dates(try_parsedates)
+                .low_memory(low_memory)
+                .with_ignore_errors(ignore_errors)
+                .finish()?
+        };
 
         if let Some(filter_left) = &self.flag_filter_left {
             let filter_left_expr = polars::sql::sql_expr(filter_left)?;
             left_lf = left_lf.filter(filter_left_expr);
         }
 
-        let mut right_lf = LazyCsvReader::new(&self.arg_input2)
-            .has_header(true)
-            .with_missing_is_null(self.flag_nulls)
-            .with_delimiter(delim)
-            .with_infer_schema_length(num_rows)
-            .with_try_parse_dates(try_parsedates)
-            .low_memory(low_memory)
-            .with_ignore_errors(ignore_errors)
-            .finish()?;
+        let mut right_lf = {
+            // check if the right input file is snappy compressed
+            if input2_path.extension().and_then(std::ffi::OsStr::to_str) == Some("sz") {
+                let decompressed_path =
+                    util::decompress_snappy_file(&input2_path.to_path_buf(), tmpdir)?;
+                self.arg_input2 = decompressed_path;
+            }
+
+            LazyCsvReader::new(&self.arg_input2)
+                .has_header(true)
+                .with_missing_is_null(self.flag_nulls)
+                .with_comment_prefix(comment_char.as_deref())
+                .with_separator(delim)
+                .with_infer_schema_length(num_rows)
+                .with_try_parse_dates(try_parsedates)
+                .low_memory(low_memory)
+                .with_ignore_errors(ignore_errors)
+                .finish()?
+        };
 
         if let Some(filter_right) = &self.flag_filter_right {
             let filter_right_exprt = polars::sql::sql_expr(filter_right)?;
@@ -499,6 +567,7 @@ impl Args {
             delim,
             streaming: self.flag_streaming,
             no_optimizations: self.flag_no_optimizations,
+            sql_filter: self.flag_sql_filter.clone(),
             datetime_format: self.flag_datetime_format.clone(),
             date_format: self.flag_date_format.clone(),
             time_format: self.flag_time_format.clone(),
