@@ -106,11 +106,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         Some(uri) => {
             let path = if Url::parse(uri).is_ok() && uri.starts_with("http") {
                 // its a remote file, download it first
-                let temp_download_path = temp_download.path().to_str().unwrap().to_string();
-
                 let future = util::download_file(
                     uri,
-                    &temp_download_path,
+                    temp_download.path().to_path_buf(),
                     args.flag_progressbar && !args.cmd_check && !args.flag_quiet,
                     args.flag_user_agent,
                     Some(args.flag_timeout),
@@ -121,6 +119,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     },
                 );
                 tokio::runtime::Runtime::new()?.block_on(future)?;
+                // safety: temp_download is a NamedTempFile, so we know that it can be converted
+                let temp_download_path = temp_download.path().to_str().unwrap().to_string();
                 temp_download_path
             } else {
                 // its a local file
@@ -157,7 +157,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             jobs -= 1; // save one thread for other tasks
         }
 
-        compress(input_reader, output_writer, jobs)?;
+        compress(input_reader, output_writer, jobs, gzp::BUFSIZE * 2)?;
         let compressed_bytes = if let Some(path) = &args.flag_output {
             fs::metadata(path)?.len()
         } else {
@@ -193,7 +193,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         }
     } else if args.cmd_validate {
         if args.arg_input.is_none() {
-            return fail_clierror!("stdin is not supported by the snappy validate subcommand.");
+            return fail_incorrectusage_clierror!(
+                "stdin is not supported by the snappy validate subcommand."
+            );
         }
         let Ok(decompressed_bytes) = validate(input_reader) else {
             return fail_clierror!("Not a valid snappy file.");
@@ -236,10 +238,16 @@ pub fn compress<R: Read, W: Write + Send + 'static>(
     mut src: R,
     dst: W,
     jobs: usize,
+    buf_size: usize,
 ) -> CliResult<()> {
     let mut writer = ParCompressBuilder::<Snap>::new()
         .num_threads(jobs)?
-        .buffer_size(gzp::BUFSIZE)?
+        // the buffer size must be at least gzp::DICT_SIZE
+        .buffer_size(if buf_size < gzp::DICT_SIZE {
+            gzp::DICT_SIZE
+        } else {
+            buf_size
+        })?
         .pin_threads(Some(0))
         .from_writer(dst);
     io::copy(&mut src, &mut writer)?;
