@@ -2,12 +2,11 @@ static USAGE: &str = r#"
 Run blazing-fast Polars SQL queries against several CSVs - replete with joins, aggregations,
 grouping, sorting, and more - working on larger than memory CSV files.
 
-Polars SQL is a SQL dialect, converting SQL queries to fast Polars LazyFrame expressions
-(see https://pola-rs.github.io/polars-book/user-guide/sql/intro/).
+Polars SQL is a SQL dialect, converting SQL queries to fast Polars LazyFrame expressions.
 
 For a list of SQL functions and keywords supported by Polars SQL, see
-https://github.com/pola-rs/polars/blob/main/polars/polars-sql/src/functions.rs,
-https://github.com/pola-rs/polars/blob/main/polars/polars-sql/src/keywords.rs and
+https://github.com/pola-rs/polars/blob/rs-0.36.2/crates/polars-sql/src/functions.rs
+https://github.com/pola-rs/polars/blob/rs-0.36.2/crates/polars-sql/src/keywords.rs and
 https://github.com/pola-rs/polars/issues/7227
 
 Returns the shape of the query result (number of rows, number of columns) to stderr.
@@ -18,14 +17,32 @@ Example queries:
 
   qsv sqlp data.csv 'select col1, col2 as friendlyname from data' --format parquet --output data.parquet
 
+  # enclose column names with spaces in double quotes
+  qsv sqlp data.csv 'select "col 1", "col 2" from data'
+
   qsv sqlp data.csv data2.csv 'select * from data join data2 on data.colname = data2.colname'
 
+  qsv sqlp data.csv data2.csv 'SELECT col1 FROM data WHERE col1 IN (SELECT col2 FROM data2)'
+
+  qsv sqlp data1.csv data2.csv 'SELECT * FROM data1 UNION ALL BY NAME SELECT * FROM data2'
+
+  qsv sqlp tbl_a.csv tbl_b.csv tbl_c.csv "SELECT * FROM tbl_a \
+    RIGHT ANTI JOIN tbl_b USING (b) \
+    LEFT SEMI JOIN tbl_c USING (c)"
+
+  # use "_t_N" aliases to refer to input files, where N is the 1-based index
+  # of the input file/s. For example, _t_1 refers to the first input file, _t_2
+  # refers to the second input file, and so on.
   qsv sqlp data.csv data2.csv 'select * from _t_1 join _t_2 on _t_1.colname = _t_2.colname'
 
   qsv sqlp data.csv 'SELECT col1, count(*) AS cnt FROM data GROUP BY col1 ORDER BY cnt DESC, col1 ASC'
 
   qsv sqlp data.csv "select lower(col1), substr(col2, 2, 4) from data WHERE starts_with(col1, 'foo')"
 
+  qsv sqlp data.csv "select COALESCE(NULLIF(col2, ''), 'foo') from data"
+
+  qsv sqlp tbl1.csv "SELECT x FROM tbl1 WHERE x IN (SELECT y FROM tbl1)"
+
   # Use a SQL script to run a long, complex SQL query or to run SEVERAL SQL queries.
   # When running several queries, each query needs to be separated by a semicolon,
   # the last query will be returned as the result.
@@ -34,8 +51,8 @@ Example queries:
   qsv sqlp data.csv data2.csv data3.csv data4.csv script.sql --format json --output data.json
 
   # use Common Table Expressions (CTEs) using WITH to simplify complex queries
-  qsv sqlp people.csv "WITH millenials AS (SELECT * FROM people WHERE age >= 25 and age <= 40) \
-    SELECT * FROM millenials WHERE STARTS_WITH(name,'C')"
+  qsv sqlp people.csv "WITH millennials AS (SELECT * FROM people WHERE age >= 25 and age <= 40) \
+    SELECT * FROM millennials WHERE STARTS_WITH(name,'C')"
 
   # CASE statement
   qsv sqlp data.csv "select CASE WHEN col1 > 10 THEN 'foo' WHEN col1 > 5 THEN 'bar' ELSE 'baz' END from data"
@@ -59,6 +76,12 @@ Example queries:
   # case-insensitive regexp_like
   qsv sqlp data.csv "select * from data WHERE regexp_like(col1, '^foo', 'i') AND col2 > 10"
 
+  # regexp match using a literal pattern
+    qsv sqlp data.csv "select idx,val from data WHERE val regexp '^foo'"
+
+  # regexp match using patterns from another column
+    qsv sqlp data.csv "select idx,val from data WHERE val regexp pattern_col"
+
   # use Parquet, JSONL and Arrow files in SQL queries
   qsv sqlp data.csv "select * from data join read_parquet('data2.parquet') as t2 ON data.c1 = t2.c1"
   qsv sqlp data.csv "select * from data join read_ndjson('data2.jsonl') as t2 on data.c1 = t2.c1"
@@ -82,8 +105,9 @@ Usage:
 
 sqlp arguments:
     input                  The CSV file/s to query. Use '-' for standard input.
-                           If input is a directory, all CSV files in the directory will
-                           be used.
+                           If input is a directory, all files in the directory will be read as input.
+                           If the input is a file with a '.infile-list' extension, the
+                           file will be read as a list of files to use as input.
                            If the input are snappy compressed file(s), it will be
                            decompressed automatically.
                            Column headers are required. Use 'qsv rename _all_generic --no-headers'
@@ -102,11 +126,15 @@ sqlp options:
                                 json     JSON
                                 parquet  Apache Parquet
                                 arrow    Apache Arrow IPC
+                                avro     Apache Avro
                               (default: csv)
 
-                              POLARS CSV PARSING OPTIONS:
+                              POLARS CSV INPUT PARSING OPTIONS:
     --try-parsedates          Automatically try to parse dates/datetimes and time.
                               If parsing fails, columns remain as strings.
+                              Note that if dates are not well-formatted in your CSV,
+                              that you may want to try to set `--ignore-errors` to relax
+                              the CSV parsing of dates.
     --infer-len <arg>         The number of rows to scan when inferring the schema of the CSV.
                               Set to 0 to do a full table scan (warning: very slow).
                               (default: 250)
@@ -119,6 +147,9 @@ sqlp options:
                               will be skipped. If not set, the query will fail.
                               Only use this when debugging queries, as polars does batched
                               parsing and will skip the entire batch where the error occurred.
+    --rnull-values <arg>      The comma-delimited list of strings to consider as null values
+                              when READING CSV files.
+                              (default: <empty string>)                              
 
                               CSV OUTPUT FORMAT ONLY:
     --datetime-format <fmt>   The datetime format to use writing datetimes.
@@ -128,7 +159,7 @@ sqlp options:
     --time-format <fmt>       The time format to use writing times.
     --float-precision <arg>   The number of digits of precision to use when writing floats.
                               (default: 6)
-    --null-value <arg>        The string to use when writing null values.
+    --wnull-value <arg>       The string to use when WRITING null values.
                               (default: <empty string>)
 
                               PARQUET OUTPUT FORMAT ONLY:
@@ -158,16 +189,17 @@ use std::{
     env,
     fs::File,
     io,
-    io::{BufWriter, Read, Write},
+    io::{Read, Write},
     path::{Path, PathBuf},
     str::FromStr,
     time::Instant,
 };
 
 use polars::{
+    io::avro::AvroWriter,
     prelude::{
         CsvWriter, DataFrame, GzipLevel, IpcWriter, JsonWriter, LazyCsvReader, LazyFileListReader,
-        ParquetCompression, ParquetWriter, SerWriter, ZstdLevel,
+        NullValues, ParquetCompression, ParquetWriter, SerWriter, ZstdLevel,
     },
     sql::SQLContext,
 };
@@ -175,7 +207,6 @@ use serde::Deserialize;
 use tempfile;
 
 use crate::{
-    cmd::snappy::compress,
     config::{Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
     util,
     util::process_input,
@@ -191,7 +222,7 @@ struct Args {
     arg_sql:               String,
     flag_format:           String,
     flag_try_parsedates:   bool,
-    flag_infer_len:        usize,
+    flag_infer_len:        Option<usize>,
     flag_low_memory:       bool,
     flag_no_optimizations: bool,
     flag_ignore_errors:    bool,
@@ -199,7 +230,8 @@ struct Args {
     flag_date_format:      Option<String>,
     flag_time_format:      Option<String>,
     flag_float_precision:  Option<usize>,
-    flag_null_value:       String,
+    flag_rnull_values:     String,
+    flag_wnull_value:      String,
     flag_compression:      String,
     flag_compress_level:   Option<i32>,
     flag_statistics:       bool,
@@ -208,24 +240,25 @@ struct Args {
     flag_quiet:            bool,
 }
 
-#[derive(Default, Clone)]
+#[derive(Default, Clone, PartialEq)]
 enum OutputMode {
     #[default]
     Csv,
     Json,
     Parquet,
     Arrow,
+    Avro,
     None,
 }
 
 // shamelessly copied from
-// https://github.com/pola-rs/polars/blob/main/polars-cli/src/main.rs
+// https://github.com/pola-rs/polars-cli/blob/main/src/main.rs
 impl OutputMode {
     fn execute_query(
         &self,
         query: &str,
         ctx: &mut SQLContext,
-        delim: u8,
+        mut delim: u8,
         args: Args,
     ) -> CliResult<(usize, usize)> {
         let mut df = DataFrame::default();
@@ -242,6 +275,19 @@ impl OutputMode {
             let w = match args.flag_output {
                 Some(x) => {
                     let path = Path::new(&x);
+                    // if the output file ends with ".tsv" or ".tab", we use tab as the delimiter
+                    delim = if *self == OutputMode::Csv
+                        && path
+                            .extension()
+                            .map_or(false, |ext| ext.eq_ignore_ascii_case("tsv"))
+                        || path
+                            .extension()
+                            .map_or(false, |ext| ext.eq_ignore_ascii_case("tab"))
+                    {
+                        b'\t'
+                    } else {
+                        delim
+                    };
                     Box::new(File::create(path)?) as Box<dyn Write>
                 },
                 None => Box::new(io::stdout()) as Box<dyn Write>,
@@ -250,12 +296,13 @@ impl OutputMode {
 
             let out_result = match self {
                 OutputMode::Csv => CsvWriter::new(&mut w)
-                    .with_delimiter(delim)
+                    .with_separator(delim)
                     .with_datetime_format(args.flag_datetime_format)
                     .with_date_format(args.flag_date_format)
                     .with_time_format(args.flag_time_format)
                     .with_float_precision(args.flag_float_precision)
-                    .with_null_value(args.flag_null_value)
+                    .with_null_value(args.flag_wnull_value)
+                    .include_bom(util::get_envvar_flag("QSV_OUTPUT_BOM"))
                     .finish(&mut df),
                 OutputMode::Json => JsonWriter::new(&mut w).finish(&mut df),
                 OutputMode::Parquet => {
@@ -291,6 +338,7 @@ impl OutputMode {
                         .map(|_| ())
                 },
                 OutputMode::Arrow => IpcWriter::new(&mut w).finish(&mut df),
+                OutputMode::Avro => AvroWriter::new(&mut w).finish(&mut df),
                 OutputMode::None => Ok(()),
             };
 
@@ -316,6 +364,7 @@ impl FromStr for OutputMode {
             "json" => Ok(OutputMode::Json),
             "parquet" => Ok(OutputMode::Parquet),
             "arrow" => Ok(OutputMode::Arrow),
+            "avro" => Ok(OutputMode::Avro),
             _ => Err(format!("Invalid output mode: {s}")),
         }
     }
@@ -350,14 +399,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut args: Args = util::get_args(USAGE, argv)?;
 
     let tmpdir = tempfile::tempdir()?;
-    args.arg_input = process_input(
-        args.arg_input,
-        &tmpdir,
-        "No data on stdin. Please provide at least one input file or pipe data to stdin.",
-    )?;
+    args.arg_input = process_input(args.arg_input, &tmpdir, "")?;
 
-    if args.flag_null_value == "<empty string>" {
-        args.flag_null_value.clear();
+    let rnull_values = if args.flag_rnull_values == "<empty string>" {
+        vec![String::new()]
+    } else {
+        args.flag_rnull_values
+            .split(',')
+            .map(String::from)
+            .collect()
+    };
+
+    if args.flag_wnull_value == "<empty string>" {
+        args.flag_wnull_value.clear();
     };
 
     let output_mode: OutputMode = args.flag_format.parse().unwrap_or(OutputMode::Csv);
@@ -371,10 +425,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         b','
     };
 
-    let num_rows = if args.flag_infer_len == 0 {
-        None
+    let comment_char = if let Ok(comment_char) = env::var("QSV_COMMENT_CHAR") {
+        Some(comment_char)
     } else {
-        Some(args.flag_infer_len)
+        None
     };
 
     let optimization_state = if args.flag_no_optimizations {
@@ -392,12 +446,28 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             simplify_expr:       true,
             file_caching:        !args.flag_low_memory,
             slice_pushdown:      true,
-            comm_subplan_elim:   true,
+            comm_subplan_elim:   !args.flag_low_memory,
             comm_subexpr_elim:   true,
             streaming:           args.flag_low_memory,
+            fast_projection:     true,
+            eager:               false,
         }
     };
-    log::debug!("Optimization state: {optimization_state:?}");
+    // gated by log::log_enabled!(log::Level::Debug) to avoid the
+    // relatively expensive overhead of generating the debug string
+    // for the optimization state struct
+    let debuglog_flag = log::log_enabled!(log::Level::Debug);
+    if debuglog_flag {
+        log::debug!("Optimization state: {optimization_state:?}");
+        log::debug!(
+            "Delimiter: {delim} Infer_schema_len: {infer_len:?} try_parse_dates: {parse_dates} \
+             ignore_errors: {ignore_errors}, low_memory: {low_memory}",
+            infer_len = args.flag_infer_len,
+            parse_dates = args.flag_try_parsedates,
+            ignore_errors = args.flag_ignore_errors,
+            low_memory = args.flag_low_memory
+        );
+    }
 
     let mut ctx = SQLContext::new();
     let mut table_aliases = HashMap::with_capacity(args.arg_input.len());
@@ -416,22 +486,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             });
         table_aliases.insert(table_name.to_string(), format!("_t_{}", idx + 1));
 
-        if log::log_enabled!(log::Level::Debug) {
+        if debuglog_flag {
             log::debug!(
-                "Registering table: {table_name} as {alias} -  Delimiter: {delim} \
-                 Infer_schema_len: {num_rows:?} try_parse_dates: {parse_dates} ignore_errors: \
-                 {ignore_errors}, low_memory: {low_memory}",
+                "Registering table: {table_name} as {alias}",
                 alias = table_aliases.get(table_name).unwrap(),
-                parse_dates = args.flag_try_parsedates,
-                ignore_errors = args.flag_ignore_errors,
-                low_memory = args.flag_low_memory
             );
         }
         let lf = LazyCsvReader::new(table)
             .has_header(true)
             .with_missing_is_null(true)
-            .with_delimiter(delim)
-            .with_infer_schema_length(num_rows)
+            .with_comment_prefix(comment_char.as_deref())
+            .with_null_values(Some(NullValues::AllColumns(rnull_values.clone())))
+            .with_separator(delim)
+            .with_infer_schema_length(args.flag_infer_len)
             .with_try_parse_dates(args.flag_try_parsedates)
             .with_ignore_errors(args.flag_ignore_errors)
             .low_memory(args.flag_low_memory)
@@ -439,7 +506,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         ctx.register(table_name, lf.with_optimizations(optimization_state));
     }
 
-    if log::log_enabled!(log::Level::Debug) {
+    if debuglog_flag {
         let tables_in_context = ctx.get_tables();
         log::debug!("Table(s) registered in SQL Context: {tables_in_context:?}");
     }
@@ -455,14 +522,16 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         sql_script
             .split(';')
             .map(std::string::ToString::to_string)
-            .filter(|s| s.trim().len() > 0)
+            .filter(|s| !s.trim().is_empty())
             .collect()
     } else {
         // its not a sql script, just a single query
         vec![args.arg_sql.clone()]
     };
 
-    log::debug!("Executing query/ies: {queries:?}");
+    if debuglog_flag {
+        log::debug!("SQL query/ies: {queries:?}");
+    }
 
     let num_queries = queries.len();
     let last_query: usize = num_queries.saturating_sub(1);
@@ -483,7 +552,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             current_query = current_query.replace(table_alias, &(format!(r#""{table_name}""#)));
         }
 
-        if log::log_enabled!(log::Level::Debug) {
+        if debuglog_flag {
             log::debug!("Executing query {idx}: {current_query}");
             now = Instant::now();
         }
@@ -494,7 +563,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             // this is not the last query, we only execute the query, but don't write the output
             no_output.execute_query(&current_query, &mut ctx, delim, args.clone())?
         };
-        if log::log_enabled!(log::Level::Debug) {
+        if debuglog_flag {
             log::debug!(
                 "Query {idx} successfully executed in {elapsed:?} seconds: {query_result_shape:?}",
                 elapsed = now.elapsed().as_secs_f32()
@@ -502,13 +571,28 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         }
     }
 
-    if let Some(output) = args.flag_output {
-        // if the output ends with ".sz", we snappy compress the output
+    compress_output_if_needed(args.flag_output)?;
+
+    if !args.flag_quiet {
+        eprintln!("{query_result_shape:?}");
+    }
+
+    Ok(())
+}
+
+/// if the output ends with ".sz", we snappy compress the output
+/// and replace the original output with the compressed output
+pub fn compress_output_if_needed(
+    output_file: Option<String>,
+) -> Result<(), crate::clitypes::CliError> {
+    use crate::cmd::snappy::compress;
+
+    if let Some(output) = output_file {
         if std::path::Path::new(&output)
             .extension()
             .map_or(false, |ext| ext.eq_ignore_ascii_case("sz"))
         {
-            log::debug!("Compressing output with Snappy");
+            log::info!("Compressing output with Snappy");
 
             // we need to copy the output to a tempfile first, and then
             // compress the tempfile to the original output sz file
@@ -520,17 +604,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             // https://github.com/Stebalien/tempfile/issues/192
             let input_fname = tempfile.path().to_str().unwrap();
             let input = File::open(input_fname)?;
-            let output_sz_writer = BufWriter::with_capacity(
+            let output_sz_writer = std::fs::File::create(output)?;
+            compress(
+                input,
+                output_sz_writer,
+                util::max_jobs(),
                 DEFAULT_WTR_BUFFER_CAPACITY,
-                std::fs::File::create(output)?,
-            );
-            compress(input, output_sz_writer, util::max_jobs())?;
+            )?;
         }
-    }
-
-    if !args.flag_quiet {
-        eprintln!("{query_result_shape:?}");
-    }
-
+    };
     Ok(())
 }
