@@ -6,18 +6,19 @@ nullcount, sparsity, quartiles, interquartile range (IQR), lower/upper fences, s
 cardinality, mode/s & antimode/s, and median absolute deviation (MAD). Note that some
 statistics requires loading the entire file into memory, so they must be enabled explicitly. 
 
-By default, the following statistics are reported for *every* column in the CSV data:
+By default, the following "streaming" statistics are reported for *every* column:
 sum, min/max/range values, min/max length, mean, stddev, variance, nullcount & sparsity.
 The default set of statistics corresponds to statistics that can be computed efficiently
 on a stream of data (i.e., constant memory) and works with arbitrarily large CSV files.
 
-The following additional statistics require loading the entire file into memory:
+The following additional "non-streaming" statistics require loading the entire file into memory:
 cardinality, mode/antimode, median, MAD, quartiles and its related measures (IQR,
 lower/upper fences & skewness).
 
-Note that an Out-Of-Memory (OOM) check heuristic will prevent processing if the file
-is larger than the available memory minus a headroom buffer of 20% (adjustable using the
-QSV_FREEMEMORY_HEADROOM_PCT environment variable).
+When computing “non-streaming” statistics, an Out-Of-Memory (OOM) heuristic check is done.
+If the file is larger than the available memory minus a headroom buffer of 20% (which can be
+adjusted using the QSV_FREEMEMORY_HEADROOM_PCT environment variable), processing will be
+preemptively prevented.
 
 "Antimode" is the least frequently occurring non-zero value and is the opposite of mode.
 It returns "*ALL" if all the values are unique, and only returns a preview of the first
@@ -43,18 +44,26 @@ The date formats recognized and its sub-variants along with examples can be foun
 https://github.com/jqnatividad/belt/tree/main/dateparser#accepted-date-formats.
 
 Computing statistics on a large file can be made MUCH faster if you create an index for it
-first with 'qsv index' to enable multithreading.
+first with 'qsv index' to enable multithreading. With an index, the file is split into equal
+chunks and each chunk is processed in parallel. The number of chunks is determined by the
+number of logical CPUs detected. You can override this by setting the --jobs option.
 
-This command caches the results in <FILESTEM>.stats.csv and <FILESTEM>.stats.csv.bin
-(e.g., qsv stats nyc311.csv will create nyc311.stats.csv and nyc311.stats.csv.bin).
-The .bin file is the binary format of the computed stats. The arguments used to generate the
-cached stats are saved in <FILESTEM>.stats.csv.json.
+As stats is a central command in qsv, and can be expensive to compute, `stats` caches results
+in <FILESTEM>.stats.csv & if the --stats-binout option is used, <FILESTEM>.stats.csv.bin.sz 
+(e.g., qsv stats nyc311.csv will create nyc311.stats.csv & nyc311.stats.csv.bin.sz).
+The .bin.sz file is the snappy-compressed binary format of the computed stats.
+The arguments used to generate the cached stats are saved in <FILESTEM>.stats.csv.json.
 
 If stats have already been computed for the input file with similar arguments and the file
 hasn't changed, the stats will be loaded from the cache instead of recomputing it.
 
-For examples, see the "boston311" test files in https://github.com/jqnatividad/qsv/tree/master/resources/test
-and https://github.com/jqnatividad/qsv/blob/4529d51273218347fef6aca15ac24e22b85b2ec4/tests/test_stats.rs#L608.
+These cached stats are also used by other qsv commands (currently `schema` & `tojsonl`) to
+load the stats into memory faster. If the cached stats are not current (i.e., the input file
+is newer than the cached stats), the cached stats will be ignored and recomputed.
+
+For examples, see the "boston311" test files in 
+https://github.com/jqnatividad/qsv/tree/master/resources/test and
+https://github.com/jqnatividad/qsv/blob/4529d51273218347fef6aca15ac24e22b85b2ec4/tests/test_stats.rs#L608.
 
 Usage:
     qsv stats [options] [<input>]
@@ -121,11 +130,14 @@ stats options:
                               Note that a file handle is opened for each job.
                               When not set, the number of jobs is set to the
                               number of CPUs detected.
-    --stats-binout <file>     Write the stats to <file> in binary format.
-                              This is used internally by other qsv commands (currently `schema`
-                              and `tojsonl`) to load cached stats into memory without having to
-                              process/parse the CSV again.
-                              If <file> is "NONE", the stats cache will not be written.
+    --stats-binout            Write the stats in binary format. This is used internally
+                              by other qsv commands (currently `schema` & `tojsonl`) to
+                              load cached stats into memory faster. If set, the binary
+                              encoded stats will be written to <FILESTEM>.stats.csv.bin.sz.
+                              You can preemtively create the binary encoded stats file
+                              by using this option before running the `schema` and `tojsonl`
+                              commands and they will automatically load the binary encoded
+                              stats file if it exists.
 
 Common options:
     -h, --help             Display this message
@@ -163,7 +175,7 @@ use std::{
     borrow::ToOwned,
     default::Default,
     fmt, fs, io,
-    io::{BufWriter, Write},
+    io::Write,
     iter::repeat,
     path::{Path, PathBuf},
     str,
@@ -173,6 +185,7 @@ use std::{
     },
 };
 
+use gzp::{par::compress::ParCompressBuilder, snap::Snap};
 use itertools::Itertools;
 use qsv_dateparser::parse_with_preference;
 use serde::{Deserialize, Serialize};
@@ -208,11 +221,11 @@ pub struct Args {
     pub flag_prefer_dmy:      bool,
     pub flag_force:           bool,
     pub flag_jobs:            Option<usize>,
+    pub flag_stats_binout:    bool,
     pub flag_output:          Option<String>,
     pub flag_no_headers:      bool,
     pub flag_delimiter:       Option<Delimiter>,
     pub flag_memcheck:        bool,
-    pub flag_stats_binout:    Option<String>,
 }
 
 // this struct is used to serialize/deserialize the stats to
@@ -309,6 +322,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         record_count:         0,
         date_generated:       String::new(),
         compute_duration_ms:  0,
+        // save the qsv version in the stats.csv.json file
+        // so cached stats are automatically invalidated
+        // when the qsv version changes
         qsv_version:          env!("CARGO_PKG_VERSION").to_string(),
     };
 
@@ -353,13 +369,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let mut compute_stats = true;
 
-    let mut stats_binout = String::new();
-    let write_stats_cache = if let Some(s) = args.flag_stats_binout.clone() {
-        stats_binout = s;
-        stats_binout.to_lowercase() != "none"
-    } else {
-        true
-    };
+    let write_stats_binout = args.flag_stats_binout;
 
     if let Some(path) = fconfig.path.clone() {
         let path_file_stem = path.file_stem().unwrap().to_str().unwrap();
@@ -367,7 +377,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         // check if <FILESTEM>.stats.csv file already exists.
         // If it does, check if it was compiled using the same args.
         // However, if the --force flag is set,
-        // regenerate the stats even if the args are the same.
+        // recompute the stats even if the args are the same.
         if stats_file.exists() && !args.flag_force {
             let stats_args_json_file = stats_file.with_extension("csv.json");
             let existing_stats_args_json_str =
@@ -375,8 +385,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     Ok(s) => s,
                     Err(e) => {
                         log::warn!(
-                            "Could not read {path_file_stem}.stats.csv.json: {e:?}, \
-                             regenerating..."
+                            "Could not read {path_file_stem}.stats.csv.json: {e:?}, recomputing..."
                         );
                         fs::remove_file(&stats_file)?;
                         fs::remove_file(&stats_args_json_file)?;
@@ -384,6 +393,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     },
                 };
 
+            let time_saved: u64;
             // deserialize the existing stats args json
             let existing_stats_args_json: StatsArgs =
                 match serde_json::from_str::<StatsArgs>(&existing_stats_args_json_str) {
@@ -394,13 +404,15 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         stat_args.canonical_stats_path = String::new();
                         stat_args.record_count = 0;
                         stat_args.date_generated = String::new();
+                        time_saved = stat_args.compute_duration_ms;
                         stat_args.compute_duration_ms = 0;
                         stat_args
                     },
                     Err(e) => {
+                        time_saved = 0;
                         log::warn!(
                             "Could not serialize {path_file_stem}.stats.csv.json: {e:?}, \
-                             regenerating..."
+                             recomputing..."
                         );
                         fs::remove_file(&stats_file)?;
                         fs::remove_file(&stats_args_json_file)?;
@@ -432,12 +444,15 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         && existing_stats_args_json.flag_nulls == current_stats_args.flag_nulls
                         && existing_stats_args_json.qsv_version == current_stats_args.qsv_version)
             {
-                log::info!("{path_file_stem}.stats.csv already exists and is current, skipping...",);
+                log::info!(
+                    "{path_file_stem}.stats.csv already exists and is current. Skipping compute \
+                     and using cached stats instead - {time_saved} secs saved...",
+                );
                 compute_stats = false;
             } else {
                 log::info!(
                     "{path_file_stem}.stats.csv already exists, but is older than the input file \
-                     or the args have changed, regenerating...",
+                     or the args have changed, recomputing...",
                 );
                 fs::remove_file(&stats_file)?;
             }
@@ -458,8 +473,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
             // we need to count the number of records in the file to calculate sparsity
             let record_count = RECORD_COUNT.get_or_init(|| util::count_rows(&fconfig).unwrap());
-
             log::info!("scanning {record_count} records...");
+
             let (headers, stats) = match fconfig.indexed()? {
                 None => args.sequential_stats(&args.flag_dates_whitelist),
                 Some(idx) => {
@@ -477,7 +492,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             }?;
 
             // clone a copy of stats so we can binary encode it to disk later
-            if write_stats_cache {
+            if write_stats_binout {
                 stats_for_encoding = stats.clone();
             }
 
@@ -496,7 +511,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             }
 
             // update the stats args json file
-            if write_stats_cache {
+            if write_stats_binout {
                 current_stats_args.canonical_input_path =
                     path.canonicalize()?.to_str().unwrap().to_string();
                 current_stats_args.record_count = *record_count;
@@ -510,7 +525,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     if let Some(pb) = stdin_tempfile_path {
         // remove the temp file we created to store stdin
-        log::info!("deleting stdin temp file");
         std::fs::remove_file(pb)?;
     }
 
@@ -545,34 +559,37 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             fs::copy(currstats_filename.clone(), stats_pathbuf.clone())?;
         }
 
-        // only create the stats args json and binary encoded files if we computed the stats
-        if compute_stats && write_stats_cache {
-            // save the stats args to "<FILESTEM>.stats.csv.json"
-            stats_pathbuf.set_extension("csv.json");
-            // write empty file first so we can canonicalize it
-            std::fs::File::create(stats_pathbuf.clone())?;
-            current_stats_args.canonical_stats_path = stats_pathbuf
-                .clone()
-                .canonicalize()?
-                .to_str()
-                .unwrap()
-                .to_string();
-            std::fs::write(
-                stats_pathbuf.clone(),
-                // safety: we know that current_stats_args is JSON serializable
-                serde_json::to_string_pretty(&current_stats_args).unwrap(),
-            )?;
+        // save the stats args to "<FILESTEM>.stats.csv.json"
+        stats_pathbuf.set_extension("csv.json");
+        // write empty file first so we can canonicalize it
+        std::fs::File::create(stats_pathbuf.clone())?;
+        current_stats_args.canonical_stats_path = stats_pathbuf
+            .clone()
+            .canonicalize()?
+            .to_str()
+            .unwrap()
+            .to_string();
+        std::fs::write(
+            stats_pathbuf.clone(),
+            // safety: we know that current_stats_args is JSON serializable
+            serde_json::to_string_pretty(&current_stats_args).unwrap(),
+        )?;
 
-            // binary encode the stats to "<FILESTEM>.stats.csv.bin"
-            let mut stats_pathbuf = path.clone();
-            stats_pathbuf.set_extension("stats.csv.bin");
+        // only create the binary encoded files if we computed the stats
+        // and the user specified --stats-binout
+        if compute_stats && write_stats_binout {
+            // binary encode the stats to "<FILESTEM>.stats.csv.bin.sz"
+            let mut stats_pathbuf = path;
+            stats_pathbuf.set_extension("stats.csv.bin.sz");
             // we do the binary encoding inside a block so that the encoded_file
-            // gets dropped/flushed before we copy it to the output file
+            // automatically gets dropped/flushed before we copy it to the output file
             {
-                let encoded_file = BufWriter::with_capacity(
-                    DEFAULT_WTR_BUFFER_CAPACITY * 2,
-                    fs::File::create(stats_pathbuf.clone())?,
-                );
+                let encoded_file = ParCompressBuilder::<Snap>::new()
+                    .num_threads(util::max_jobs())?
+                    .buffer_size(DEFAULT_WTR_BUFFER_CAPACITY * 2)?
+                    .pin_threads(Some(0))
+                    .from_writer(fs::File::create(stats_pathbuf.clone())?);
+
                 if let Err(e) = bincode::serialize_into(encoded_file, &stats_for_encoding) {
                     return fail_clierror!(
                         "Failed to write binary encoded stats {}: {e:?}",
@@ -581,17 +598,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 }
             }
         }
-
-        // if the user specified --stats-binout, copy the binary stats to that file as well
-        if write_stats_cache && !stats_binout.is_empty() {
-            let mut stats_pathbuf = path;
-            stats_pathbuf.set_extension("stats.csv.bin");
-            if let Err(e) = fs::copy(stats_pathbuf, stats_binout.clone()) {
-                return fail_clierror!(
-                    "Failed to copy binary encoded stats to {stats_binout}: {e:?}"
-                );
-            }
-        }
     }
 
     if stdout_output_flag {
@@ -712,13 +718,16 @@ impl Args {
         // safety: because we're using iterators and INFER_DATE_FLAGS has the same size,
         // we know we don't need to bounds check
         unsafe {
+            let mut i;
             for row in it {
-                for (i, field) in sel.select(&row.unwrap_unchecked()).enumerate() {
+                i = 0;
+                for field in sel.select(&row.unwrap_unchecked()) {
                     stats.get_unchecked_mut(i).add(
                         field,
                         *infer_date_flags.get_unchecked(i),
                         infer_boolean,
                     );
+                    i += 1;
                 }
             }
         }
@@ -884,6 +893,8 @@ fn init_date_inference(
         log::info!("inferring dates for ALL fields with DMY preference: {dmy_preferred}");
         vec![true; headers.len()]
     } else {
+        let mut header_str = String::new();
+        let mut date_found = false;
         let whitelist = whitelist_lower
             .split(',')
             .map(str::trim)
@@ -891,8 +902,8 @@ fn init_date_inference(
         headers
             .iter()
             .map(|header| {
-                let header_str = from_bytes::<String>(header).unwrap().to_lowercase();
-                let date_found = whitelist
+                util::to_lowercase_into(&from_bytes::<String>(header).unwrap(), &mut header_str);
+                date_found = whitelist
                     .iter()
                     .any(|whitelist_item| header_str.contains(whitelist_item));
                 if date_found {
@@ -907,7 +918,7 @@ fn init_date_inference(
 
     INFER_DATE_FLAGS
         .set(infer_date_flags)
-        .map_err(|e| format!("Cannot init date inference flags: {:?}", e))?;
+        .map_err(|e| format!("Cannot init date inference flags: {e:?}"))?;
     Ok(())
 }
 
@@ -950,7 +961,7 @@ pub struct Stats {
 fn timestamp_ms_to_rfc3339(timestamp: i64, typ: FieldType) -> String {
     use chrono::{DateTime, NaiveDateTime, Utc};
 
-    let date_val = DateTime::<Utc>::from_utc(
+    let date_val = DateTime::<Utc>::from_naive_utc_and_offset(
         NaiveDateTime::from_timestamp_millis(timestamp).unwrap_or_default(),
         Utc,
     )
@@ -1040,6 +1051,8 @@ impl Stats {
                     };
                 }
             },
+            // do nothing for String type
+            TString => {},
             TFloat | TInteger => {
                 if sample_type == TNull {
                     if self.which.include_nulls {
@@ -1063,8 +1076,6 @@ impl Stats {
                     }
                 }
             },
-            // do nothing for String type
-            TString => {},
             TDateTime | TDate => {
                 if sample_type == TNull {
                     if self.which.include_nulls {
@@ -1230,7 +1241,7 @@ impl Stats {
         // sum
         if let Some(sum) = self.sum.as_ref().and_then(|sum| sum.show(typ)) {
             if typ == FieldType::TFloat {
-                if let Ok(f64_val) = fast_float::parse::<f64, _>(&sum) {
+                if let Ok(f64_val) = sum.parse::<f64>() {
                     pieces.push(util::round_num(f64_val, round_places));
                 } else {
                     pieces.push(format!("ERROR: Cannot convert {sum} to a float."));
@@ -1488,15 +1499,15 @@ impl FieldType {
             || current_type == FieldType::TInteger
             || current_type == FieldType::TNull
         {
-            if let Ok(int_val) = string.parse::<i64>() {
+            if let Ok(int_val) = atoi_simd::parse::<i64>(sample) {
                 // leading zero, its a string (e.g. zip codes)
-                if string.starts_with('0') && int_val != 0 {
+                if string.as_bytes()[0] == b'0' && int_val != 0 {
                     return (TString, None);
                 }
                 return (TInteger, None);
             }
 
-            if fast_float::parse::<f64, _>(string).is_ok() {
+            if string.parse::<f64>().is_ok() {
                 return (TFloat, None);
             }
         }
@@ -1606,7 +1617,7 @@ impl TypedSum {
                     // so we don't panic on overflow/underflow, use saturating_add
                     self.integer = self
                         .integer
-                        .saturating_add(from_bytes::<i64>(sample).unwrap());
+                        .saturating_add(atoi_simd::parse::<i64>(sample).unwrap());
                 }
             },
             _ => {},
@@ -1673,19 +1684,19 @@ impl TypedMinMax {
         match typ {
             TString | TNull => {},
             TFloat => {
-                let n = fast_float::parse::<f64, _>(from_utf8(sample).unwrap()).unwrap();
+                let n = from_utf8(sample).unwrap().parse::<f64>().unwrap();
 
                 self.floats.add(n);
                 self.integers.add(n as i64);
             },
             TInteger => {
-                let n = from_utf8(sample).unwrap().parse::<i64>().unwrap();
+                let n = atoi_simd::parse::<i64>(sample).unwrap();
                 self.integers.add(n);
                 #[allow(clippy::cast_precision_loss)]
                 self.floats.add(n as f64);
             },
             TDate | TDateTime => {
-                let n = from_utf8(sample).unwrap().parse::<i64>().unwrap();
+                let n = atoi_simd::parse::<i64>(sample).unwrap();
                 self.dates.add(n);
             },
         }
