@@ -1,19 +1,63 @@
 static USAGE: &str = r#"
-Validate CSV data with JSON Schema Validation, putting invalid records into a separate file.
-Uses https://json-schema.org/draft/2020-12/json-schema-validation.html.
-When run without JSON Schema, only a simple CSV check (RFC 4180) is performed.
+Validates CSV data using two modes:
 
-You can create a JSON Schema from a CSV file using the `qsv schema` command.
+JSON SCHEMA VALIDATION MODE:
+This mode is invoked if a JSON Schema file is provided.
 
-Example output files from `mydata.csv`. If piped from stdin, then filename is `stdin.csv`.
+The CSV data is validated against the JSON Schema. If the CSV data is valid, no output
+files are created and the command returns an exit code of 0.
 
-* mydata.csv.valid
-* mydata.csv.invalid
-* mydata.csv.validation-errors.tsv
+If invalid records are found, they are put into an "invalid" file, with the rest of the
+records put into a "valid"" file.
 
-The JSON Schema can be a local file or a URL.
+A "validation-errors.tsv" report is also created with the following columns:
 
-Returns exitcode 0 when the CSV file is valid, exitcode > 0 otherwise.
+  * row_number: the row number of the invalid record
+  * field: the field name of the invalid field
+  * error: a validation error message detailing why the field is invalid
+
+It uses the JSON Schema Validation Specification (draft 2020-12) to validate the CSV.
+It validates not only the structure of the file, but the data types and domain/range of the
+fields as well. See https://json-schema.org/draft/2020-12/json-schema-validation.html
+
+You can create a JSON Schema file from a reference CSV file using the `qsv schema` command.
+Once the schema is created, you can fine-tune it to your needs and use it to validate other CSV
+files that have the same structure.
+
+Be sure to select a “training” CSV file that is representative of the data you want to validate
+when creating a schema. The data types, domain/range and regular expressions inferred from the
+reference CSV file should be appropriate for the data you want to validate. 
+
+Typically, after creating a schema, you should edit it to fine-tune each field's inferred
+validation rules.
+
+For example, if we created a JSON schema file called "reference.schema.json" using the `schema` command.
+And want to validate "mydata.csv" which we know has validation errors, the output files from running
+`qsv validate mydata.csv reference.schema.json` are: 
+
+  * mydata.csv.valid
+  * mydata.csv.invalid
+  * mydata.csv.validation-errors.tsv
+
+With an exit code of 1 to indicate a validation error.
+
+If we validate another CSV file, "mydata2.csv", which we know is valid, there are no output files,
+and the exit code is 0.
+
+If piped from stdin, the filenames will use `stdin.csv` as the base filename. For example:
+`cat mydata.csv | qsv validate reference.schema.json`
+
+   * stdin.csv.valid
+   * stdin.csv.invalid
+   * stdin.csv.validation-errors.tsv
+
+RFC 4180 VALIDATION MODE:
+If run without a JSON Schema file, the CSV is validated if it complies with qsv's interpretation of
+the RFC 4180 CSV standard (see https://github.com/jqnatividad/qsv#rfc-4180-csv-standard).
+
+It also confirms if the CSV is UTF-8 encoded.
+
+For both modes, returns exit code 0 when the CSV file is valid, exitcode > 0 otherwise.
 If all records are valid, no output files are produced.
 
 For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_validate.rs.
@@ -22,7 +66,14 @@ Usage:
     qsv validate [options] [<input>] [<json-schema>]
     qsv validate --help
 
+Validate arguments:
+    <input>                    Input CSV file to validate. If not provided, will read from stdin.
+    <json-schema>              JSON Schema file to validate against. If not provided, `validate`
+                               will run in RFC 4180 validation mode.
+                               The file can be a local file or a URL.
+
 Validate options:
+    --trim                     Trim leading and trailing whitespace from fields before validating.
     --fail-fast                Stops on first error.
     --valid <suffix>           Valid record output file suffix. [default: valid]
     --invalid <suffix>         Invalid record output file suffix. [default: invalid]
@@ -47,6 +98,7 @@ Common options:
     -d, --delimiter <arg>      The field delimiter for reading CSV data.
                                Must be a single character. [default: ,]
     -p, --progressbar          Show progress bars. Not valid for stdin.
+    -Q, --quiet                Do not display validation summary message.
 "#;
 
 use std::{
@@ -61,6 +113,7 @@ use std::{
 };
 
 use csv::ByteRecord;
+use indicatif::HumanCount;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 use indicatif::{ProgressBar, ProgressDrawTarget};
 use itertools::Itertools;
@@ -73,7 +126,6 @@ use rayon::{
 use serde::{Deserialize, Serialize};
 use serde_json::{json, value::Number, Map, Value};
 use simdutf8::basic::from_utf8;
-use thousands::Separable;
 
 use crate::{
     config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
@@ -88,6 +140,7 @@ static TIMEOUT_SECS: AtomicU16 = AtomicU16::new(15);
 #[derive(Deserialize)]
 #[allow(dead_code)]
 struct Args {
+    flag_trim:        bool,
     flag_fail_fast:   bool,
     flag_valid:       Option<String>,
     flag_invalid:     Option<String>,
@@ -98,18 +151,27 @@ struct Args {
     flag_no_headers:  bool,
     flag_delimiter:   Option<Delimiter>,
     flag_progressbar: bool,
+    flag_quiet:       bool,
     arg_input:        Option<String>,
     arg_json_schema:  Option<String>,
     flag_timeout:     u16,
 }
 
+enum JSONtypes {
+    String,
+    Number,
+    Integer,
+    Boolean,
+    Unsupported,
+}
+
 #[derive(Serialize, Deserialize)]
 struct RFC4180Struct {
     delimiter_char: char,
     header_row:     bool,
     quote_char:     char,
     num_records:    u64,
-    num_fields:     usize,
+    num_fields:     u64,
     fields:         Vec<String>,
 }
 
@@ -132,6 +194,208 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let mut rdr = rconfig.reader()?;
 
+    // if no JSON Schema supplied, only let csv reader RFC4180-validate csv file
+    if args.arg_json_schema.is_none() {
+        // just read csv file and let csv reader report problems
+        // since we're using csv::StringRecord, this will also detect non-utf8 sequences
+
+        let flag_json = args.flag_json || args.flag_pretty_json;
+        let flag_pretty_json = args.flag_pretty_json;
+
+        // first, let's validate the header row
+        let mut header_msg = String::new();
+        let mut header_len = 0_usize;
+        let mut field_vec: Vec<String> = Vec::new();
+        if !args.flag_no_headers {
+            let fields_result = rdr.headers();
+            match fields_result {
+                Ok(fields) => {
+                    header_len = fields.len();
+                    field_vec.reserve(header_len);
+                    for field in fields {
+                        field_vec.push(field.to_string());
+                    }
+                    let field_list = field_vec.join(r#"", ""#);
+                    header_msg = format!(
+                        "{} columns (\"{field_list}\") and ",
+                        HumanCount(header_len as u64)
+                    );
+                },
+                Err(e) => {
+                    // we're returning a JSON error for the header,
+                    // so we have more machine-friendly details
+                    if flag_json {
+                        // there's a UTF-8 error, so we report utf8 error metadata
+                        if let csv::ErrorKind::Utf8 { pos, err } = e.kind() {
+                            let header_error = json!({
+                                "errors": [{
+                                    "title" : "Header UTF-8 validation error",
+                                    "detail" : format!("{e}"),
+                                    "meta": {
+                                        "record_position": format!("{pos:?}"),
+                                        "record_error": format!("{err}"),
+                                    }
+                                }]
+                            });
+                            let json_error = if flag_pretty_json {
+                                serde_json::to_string_pretty(&header_error).unwrap()
+                            } else {
+                                header_error.to_string()
+                            };
+
+                            return fail_encoding_clierror!("{json_error}");
+                        }
+                        // it's not a UTF-8 error, so we report a generic
+                        // header validation error
+                        let header_error = json!({
+                            "errors": [{
+                                "title" : "Header Validation error",
+                                "detail" : format!("{e}"),
+                            }]
+                        });
+                        let json_error = if flag_pretty_json {
+                            serde_json::to_string_pretty(&header_error).unwrap()
+                        } else {
+                            header_error.to_string()
+                        };
+                        return fail_encoding_clierror!("{json_error}");
+                    }
+                    // we're not returning a JSON error, so we can use
+                    // a user-friendly error message with suggestions
+                    if let csv::ErrorKind::Utf8 { pos, err } = e.kind() {
+                        return fail_encoding_clierror!(
+                            "non-utf8 sequence detected in header, position {pos:?}.\n{err}\nUse \
+                             `qsv input` to fix formatting and to handle non-utf8 sequences.\n
+                             You may also want to transcode your data to UTF-8 first using `iconv` \
+                             or `recode`."
+                        );
+                    }
+                    // its not a UTF-8 error, report a generic header validation error
+                    return fail_clierror!("Header Validation error: {e}.");
+                },
+            }
+        }
+
+        // Now, let's validate the rest of the records the fastest way possible.
+        // We do this by using csv::ByteRecord, which does not validate utf8
+        // making for higher througput and lower memory usage compared to csv::StringRecord
+        // which validates each field SEPARATELY as a utf8 string.
+        // Combined with simdutf8::basic::from_utf8(), we utf8-validate the entire record in one go
+        // as a slice of bytes, this approach is much faster than csv::StringRecord's
+        // per-field validation.
+        let mut record = csv::ByteRecord::with_capacity(500, header_len);
+        let mut result;
+        let mut record_idx: u64 = 0;
+
+        'rfc4180_check: loop {
+            result = rdr.read_byte_record(&mut record);
+            if let Err(e) = result {
+                // read_byte_record() does not validate utf8, so we know this is not a utf8 error
+                if flag_json {
+                    // we're returning a JSON error, so we have more machine-friendly details
+                    // using the JSON API error format
+
+                    let validation_error = json!({
+                        "errors": [{
+                            "title" : "Validation error",
+                            "detail" : format!("{e}"),
+                            "meta": {
+                                "last_valid_record": format!("{record_idx}"),
+                            }
+                        }]
+                    });
+
+                    let json_error = if flag_pretty_json {
+                        serde_json::to_string_pretty(&validation_error).unwrap()
+                    } else {
+                        validation_error.to_string()
+                    };
+
+                    return fail!(json_error);
+                }
+
+                // we're not returning a JSON error, so we can use a
+                // user-friendly error message with a fixlengths suggestion
+                if let csv::ErrorKind::UnequalLengths {
+                    expected_len: _,
+                    len: _,
+                    pos: _,
+                } = e.kind()
+                {
+                    return fail_clierror!(
+                        "Validation error: {e}.\nUse `qsv fixlengths` to fix record length issues."
+                    );
+                }
+                return fail_clierror!("Validation error: {e}.\nLast valid record: {record_idx}");
+            }
+
+            // use SIMD accelerated UTF-8 validation, validate the entire record in one go
+            if simdutf8::basic::from_utf8(record.as_slice()).is_err() {
+                // there's a UTF-8 error, so we report utf8 error metadata
+                if flag_json {
+                    let validation_error = json!({
+                        "errors": [{
+                            "title" : "UTF-8 validation error",
+                            "detail" : "Cannot parse CSV record as UTF-8",
+                            "meta": {
+                                "last_valid_record": format!("{record_idx}"),
+                            }
+                        }]
+                    });
+
+                    let json_error = if flag_pretty_json {
+                        serde_json::to_string_pretty(&validation_error).unwrap()
+                    } else {
+                        validation_error.to_string()
+                    };
+                    return fail_encoding_clierror!("{json_error}");
+                }
+                // we're not returning a JSON error, so we can use a
+                // user-friendly error message with utf8 transcoding suggestions
+                return fail_encoding_clierror!(
+                    "non-utf8 sequence at record {record_idx}.\nUse `qsv input` to fix formatting \
+                     and to handle non-utf8 sequences.\nYou may also want to transcode your data \
+                     to UTF-8 first using `iconv` or `recode`."
+                );
+            }
+
+            if result.is_ok_and(|more_data| !more_data) {
+                // we've read the CSV to the end, so break out of loop
+                break 'rfc4180_check;
+            }
+            record_idx += 1;
+        }
+
+        // if we're here, we know the CSV is valid
+        let msg = if flag_json {
+            let rfc4180 = RFC4180Struct {
+                delimiter_char: rconfig.get_delimiter() as char,
+                header_row:     !rconfig.no_headers,
+                quote_char:     rconfig.quote as char,
+                num_records:    record_idx,
+                num_fields:     header_len as u64,
+                fields:         field_vec,
+            };
+
+            if flag_pretty_json {
+                serde_json::to_string_pretty(&rfc4180).unwrap()
+            } else {
+                serde_json::to_string(&rfc4180).unwrap()
+            }
+        } else {
+            format!(
+                "Valid: {header_msg}{} records detected.",
+                HumanCount(record_idx)
+            )
+        };
+        if !args.flag_quiet {
+            woutinfo!("{msg}");
+        }
+
+        // we're done when validating without a schema
+        return Ok(());
+    }
+
     // prep progress bar
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     let progress = ProgressBar::with_draw_target(None, ProgressDrawTarget::stderr_with_hz(5));
@@ -151,122 +415,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         progress.set_draw_target(ProgressDrawTarget::hidden());
     }
 
-    // if no JSON Schema supplied, only let csv reader RFC4180-validate csv file
-    if args.arg_json_schema.is_none() {
-        // just read csv file and let csv reader report problems
-
-        let mut header_msg = String::new();
-        let mut header_len = 0;
-        let mut field_vec = vec![];
-        if !args.flag_no_headers {
-            let fields_result = rdr.headers();
-            match fields_result {
-                Ok(fields) => {
-                    header_len = fields.len();
-                    for field in fields {
-                        field_vec.push(field.to_string());
-                    }
-                    let field_list = field_vec.join(r#"", ""#);
-                    header_msg = format!(
-                        "{} columns (\"{field_list}\") and ",
-                        header_len.separate_with_commas()
-                    );
-                },
-                Err(e) => {
-                    if args.flag_json || args.flag_pretty_json {
-                        let header_error = json!({
-                            "errors": [{
-                                "title" : "Cannot read header",
-                                "detail" : format!("{e}")
-                            }]
-                        });
-                        let json_error = if args.flag_pretty_json {
-                            serde_json::to_string_pretty(&header_error).unwrap()
-                        } else {
-                            header_error.to_string()
-                        };
-
-                        return fail!(json_error);
-                    }
-                    return fail_clierror!("Cannot read header ({e}).");
-                },
-            }
-        }
-
-        let mut record_idx: u64 = 0;
-        for result in rdr.records() {
-            #[cfg(any(feature = "feature_capable", feature = "lite"))]
-            if show_progress {
-                progress.inc(1);
-            }
-
-            if let Err(e) = result {
-                if args.flag_json || args.flag_pretty_json {
-                    let validation_error = json!({
-                        "errors": [{
-                            "title" : "Validation error",
-                            "detail" : format!("{e}"),
-                            "meta": {
-                                "last_valid_record": format!("{record_idx}"),
-                            }
-                        }]
-                    });
-
-                    let json_error = if args.flag_pretty_json {
-                        serde_json::to_string_pretty(&validation_error).unwrap()
-                    } else {
-                        validation_error.to_string()
-                    };
-
-                    return fail!(json_error);
-                }
-                return fail_clierror!(
-                    r#"Validation error: {e}.
-Last valid record: {record_idx}
-Use `qsv fixlengths` to fix record length issues.
-Use `qsv input` to fix formatting and to transcode to utf8 if required."#
-                );
-            }
-            record_idx += 1;
-        }
-
-        #[cfg(any(feature = "feature_capable", feature = "lite"))]
-        if show_progress {
-            progress.set_message(format!(
-                " validated {} records.",
-                progress.length().unwrap().separate_with_commas()
-            ));
-            util::finish_progress(&progress);
-        }
-
-        let msg = if args.flag_json || args.flag_pretty_json {
-            let rfc4180 = RFC4180Struct {
-                delimiter_char: rconfig.get_delimiter() as char,
-                header_row:     !rconfig.no_headers,
-                quote_char:     rconfig.quote as char,
-                num_records:    record_idx,
-                num_fields:     header_len,
-                fields:         field_vec,
-            };
-
-            if args.flag_pretty_json {
-                serde_json::to_string_pretty(&rfc4180).unwrap()
-            } else {
-                serde_json::to_string(&rfc4180).unwrap()
-            }
-        } else {
-            format!(
-                "Valid: {header_msg}{} records detected.",
-                record_idx.separate_with_commas()
-            )
-        };
-        woutinfo!("{msg}");
-
-        return Ok(());
-    }
-
     let headers = rdr.byte_headers()?.clone();
-    let headers_len = headers.len();
+    let header_len = headers.len();
 
     // parse and compile supplied JSON Schema
     let (schema_json, schema_compiled): (Value, JSONSchema) =
@@ -293,12 +443,22 @@ Use `qsv input` to fix formatting and to transcode to utf8 if required."#
             },
         };
 
-    debug!("compiled schema: {:?}", &schema_compiled);
+    if log::log_enabled!(log::Level::Debug) {
+        // only log if debug is enabled
+        // as it can be quite large and expensive to deserialize the schema
+        debug!("schema json: {:?}", &schema_json);
+    }
+
+    // set this once, as this is used repeatedly in a hot loop
+    NULL_TYPE.set(Value::String("null".to_string())).unwrap();
+
+    // get JSON types for each column in CSV file
+    let header_types = get_json_types(&headers, &schema_json)?;
 
     // how many rows read and processed as batches
-    let mut row_number: u32 = 0;
+    let mut row_number: u64 = 0;
     // how many invalid rows found
-    let mut invalid_count: u32 = 0;
+    let mut invalid_count: u64 = 0;
 
     // amortize memory allocation by reusing record
     let mut record = csv::ByteRecord::new();
@@ -308,28 +468,25 @@ Use `qsv input` to fix formatting and to transcode to utf8 if required."#
     let mut validation_results = Vec::with_capacity(batch_size);
     let mut valid_flags: Vec<bool> = Vec::with_capacity(batch_size);
     let mut validation_error_messages: Vec<String> = Vec::with_capacity(50);
+    let flag_trim = args.flag_trim;
 
     // set RAYON_NUM_THREADS
     util::njobs(args.flag_jobs);
 
-    // set this once, as this is used repeatedly in a hot loop
-    NULL_TYPE.set(Value::String("null".to_string())).unwrap();
-
     // main loop to read CSV and construct batches for parallel processing.
     // each batch is processed via Rayon parallel iterator.
     // loop exits when batch is empty.
     'batch_loop: loop {
+        let mut buffer = itoa::Buffer::new();
         for _ in 0..batch_size {
             match rdr.read_byte_record(&mut record) {
                 Ok(has_data) => {
                     if has_data {
                         row_number += 1;
-                        let mut buffer = itoa::Buffer::new();
                         record.push_field(buffer.format(row_number).as_bytes());
-
-                        // non-allocating trimming in place is much faster on the record level
-                        // with our csv fork than doing per field std::str::trim which is allocating
-                        record.trim();
+                        if flag_trim {
+                            record.trim();
+                        }
                         batch.push(record.clone());
                     } else {
                         // nothing else to add to batch
@@ -351,15 +508,7 @@ Use `qsv input` to fix formatting and to transcode to utf8 if required."#
         // validation_results vector should have same row count and in same order as input CSV
         batch
             .par_iter()
-            .map(|record| {
-                do_json_validation(
-                    &headers,
-                    headers_len,
-                    record,
-                    &schema_json,
-                    &schema_compiled,
-                )
-            })
+            .map(|record| do_json_validation(&header_types, header_len, record, &schema_compiled))
             .collect_into_vec(&mut validation_results);
 
         // write to validation error report, but keep Vec<bool> to gen valid/invalid files later
@@ -392,7 +541,7 @@ Use `qsv input` to fix formatting and to transcode to utf8 if required."#
     if show_progress {
         progress.set_message(format!(
             " validated {} records.",
-            progress.length().unwrap().separate_with_commas()
+            HumanCount(progress.length().unwrap())
         ));
         util::finish_progress(&progress);
     }
@@ -426,7 +575,7 @@ Use `qsv input` to fix formatting and to transcode to utf8 if required."#
         let fail_fast_msg = if args.flag_fail_fast {
             format!(
                 "fail-fast enabled. stopped after row {}.\n",
-                row_number.separate_with_commas()
+                HumanCount(row_number)
             )
         } else {
             String::new()
@@ -434,12 +583,14 @@ Use `qsv input` to fix formatting and to transcode to utf8 if required."#
 
         return fail_clierror!(
             "{fail_fast_msg}{} out of {} records invalid.",
-            invalid_count.separate_with_commas(),
-            row_number.separate_with_commas()
+            HumanCount(invalid_count),
+            HumanCount(row_number)
         );
     }
 
-    winfo!("All {} records valid.", row_number.separate_with_commas());
+    if !args.flag_quiet {
+        winfo!("All {} records valid.", HumanCount(row_number));
+    }
     Ok(())
 }
 
@@ -517,18 +668,16 @@ fn write_error_report(input_path: &str, validation_error_messages: Vec<String>)
 
 /// if given record is valid, return None, otherwise, error file entry string
 fn do_json_validation(
-    headers: &ByteRecord,
-    headers_len: usize,
+    header_types: &[(String, JSONtypes)],
+    header_len: usize,
     record: &ByteRecord,
-    schema_json: &Value,
     schema_compiled: &JSONSchema,
 ) -> Option<String> {
     // row number was added as last column. We use can do unwrap safely since we know its there
-    let row_number_string = from_utf8(record.get(headers_len).unwrap()).unwrap();
+    let row_number_string = from_utf8(record.get(header_len).unwrap()).unwrap();
 
-    // debug!("instance[{row_number}]: {instance:?}");
     validate_json_instance(
-        &(match to_json_instance(headers, headers_len, record, schema_json) {
+        &(match to_json_instance(header_types, header_len, record) {
             Ok(obj) => obj,
             Err(e) => {
                 return Some(format!("{row_number_string}\t<RECORD>\t{e}"));
@@ -550,146 +699,134 @@ fn do_json_validation(
     })
 }
 
-/// convert CSV Record into JSON instance by referencing Type from Schema
+/// convert CSV Record into JSON instance by referencing JSON types
 #[inline]
 fn to_json_instance(
-    headers: &ByteRecord,
-    headers_len: usize,
+    header_types: &[(String, JSONtypes)],
+    header_len: usize,
     record: &ByteRecord,
-    schema: &Value,
-) -> Result<Value, String> {
+) -> CliResult<Value> {
+    let mut json_object_map: Map<String, Value> = Map::with_capacity(header_len);
+
+    for ((key, json_type), value) in header_types.iter().zip(record.iter()) {
+        if value.is_empty() {
+            json_object_map.insert(key.to_string(), Value::Null);
+            continue;
+        }
+
+        let Ok(value_str) = from_utf8(value) else {
+            let s = String::from_utf8_lossy(value);
+            return fail_encoding_clierror!("CSV value is not valid UTF-8: {s}");
+        };
+
+        match *json_type {
+            JSONtypes::String => {
+                json_object_map.insert(key.clone(), Value::String(value_str.to_owned()));
+            },
+            JSONtypes::Number => {
+                if let Ok(float) = value_str.parse::<f64>() {
+                    json_object_map
+                        .insert(key.clone(), Value::Number(Number::from_f64(float).unwrap()));
+                } else {
+                    return fail_clierror!(
+                        "Can't cast into Number. key: {key}, value: {value_str}"
+                    );
+                }
+            },
+            JSONtypes::Integer => {
+                if let Ok(int) = atoi_simd::parse::<i64>(value_str.as_bytes()) {
+                    json_object_map.insert(key.clone(), Value::Number(Number::from(int)));
+                } else {
+                    return fail_clierror!(
+                        "Can't cast into Integer. key: {key}, value: {value_str}"
+                    );
+                }
+            },
+            JSONtypes::Boolean => {
+                if let Ok(boolean) = value_str.parse::<bool>() {
+                    json_object_map.insert(key.clone(), Value::Bool(boolean));
+                } else {
+                    return fail_clierror!(
+                        "Can't cast into Boolean. key: {key}, value: {value_str}"
+                    );
+                }
+            },
+            JSONtypes::Unsupported => {
+                unreachable!("we should never get an unsupported JSON type");
+            },
+        }
+    }
+
+    Ok(Value::Object(json_object_map))
+}
+
+/// get JSON types for each column in CSV file
+/// returns a Vector of tuples of column/header name (String) & JSON type (JSONtypes enum)
+#[inline]
+fn get_json_types(headers: &ByteRecord, schema: &Value) -> CliResult<Vec<(String, JSONtypes)>> {
     // make sure schema has expected structure
     let Some(schema_properties) = schema.get("properties") else {
-        return fail!("JSON Schema missing 'properties' object");
+        return fail_clierror!("JSON Schema missing 'properties' object");
     };
 
-    // map holds individual CSV fields converted as serde_json::Value
-    // we use with_capacity to minimize allocs
-    let mut json_object_map: Map<String, Value> = Map::with_capacity(headers_len);
-
     // safety: we set NULL_TYPE in main() and it's never changed
     let null_type = NULL_TYPE.get().unwrap();
 
     let mut key_string: String;
-    let mut value_string: String;
     let mut field_def: &Value;
     let mut field_type_def: &Value;
-    let mut json_type: u8;
+    let mut json_type: JSONtypes;
+    let mut header_types: Vec<(String, JSONtypes)> = Vec::with_capacity(headers.len());
 
     // iterate over each CSV field and convert to JSON type
-    for (i, header) in headers.iter().enumerate() {
+    for header in headers {
         // convert csv header to string. It's the key in the JSON object
         key_string = if let Ok(s) = from_utf8(header) {
             s.to_owned()
         } else {
             let s = String::from_utf8_lossy(header);
-            return fail!(format!("CSV header is not valid UTF-8: {s}"));
+            return fail_encoding_clierror!("CSV header is not valid UTF-8: {s}");
         };
 
-        // convert csv value to string; no trimming reqd as it's done on the record level beforehand
-        let Some(value) = record.get(i) else {
-            return fail!("CSV record is missing value for header '{key_string}' at index {i}");
-        };
-        value_string = if let Ok(s) = from_utf8(value) {
-            s.to_owned()
-        } else {
-            let s = String::from_utf8_lossy(value);
-            return fail!(format!("CSV value is not valid UTF-8: {s}"));
-        };
-
-        // if value_string is empty, then just put an empty JSON String
-        if value_string.is_empty() {
-            json_object_map.insert(key_string, Value::Null);
-            continue;
-        }
-
-        // get json type from schema; defaults to STRING if not specified
-        field_def = schema_properties.get(&key_string).unwrap_or(&Value::Null);
-
+        field_def = schema_properties
+            .get(key_string.clone())
+            .unwrap_or(&Value::Null);
         field_type_def = field_def.get("type").unwrap_or(&Value::Null);
 
-        // we do json_type as a u8 so match will be a tad faster below
         json_type = match field_type_def {
-            Value::String(s) => s.as_bytes()[0],
+            Value::String(s) => match s.as_str() {
+                "string" => JSONtypes::String,
+                "number" => JSONtypes::Number,
+                "integer" => JSONtypes::Integer,
+                "boolean" => JSONtypes::Boolean,
+                _ => JSONtypes::Unsupported,
+            },
             Value::Array(vec) => {
-                // if can't find usable type info, defaults to "string"
-                let mut return_val = b's';
-
-                // grab the first entry that's not a "null", since it just means value is optional
+                let mut return_val = JSONtypes::String;
                 for val in vec {
                     if *val == *null_type {
-                        // keep looking
                         continue;
                     }
                     return_val = if let Some(s) = val.as_str() {
-                        s.as_bytes()[0]
+                        match s {
+                            "string" => JSONtypes::String,
+                            "number" => JSONtypes::Number,
+                            "integer" => JSONtypes::Integer,
+                            "boolean" => JSONtypes::Boolean,
+                            _ => JSONtypes::Unsupported,
+                        }
                     } else {
-                        return fail!("type info should be a JSON string");
+                        JSONtypes::String
                     };
                 }
-
                 return_val
             },
-            _ => {
-                // default to JSON String
-                b's'
-            },
+            _ => JSONtypes::String,
         };
 
-        // dbg!(i, &header_string, &value_string, &json_type);
-
-        // matching against a u8, rather than an str should be a tad faster
-        match json_type {
-            b's' => {
-                // string
-                json_object_map.insert(key_string, Value::String(value_string));
-            },
-            b'n' => {
-                // number
-                if let Ok(float) = fast_float::parse(&value_string) {
-                    json_object_map.insert(
-                        key_string,
-                        // safety: we know it's a valid f64 from the fast_float::parse() above
-                        Value::Number(Number::from_f64(float).unwrap()),
-                    );
-                } else {
-                    return fail_format!(
-                        "Can't cast into Float. key: {key_string}, value: {value_string}, json \
-                         type: number"
-                    );
-                }
-            },
-            b'i' => {
-                // integer
-                if let Ok(int) = value_string.parse::<i64>() {
-                    json_object_map.insert(key_string, Value::Number(Number::from(int)));
-                } else {
-                    return fail_format!(
-                        "Can't cast into Integer. key: {key_string}, value: {value_string}, json \
-                         type: integer"
-                    );
-                }
-            },
-            b'b' => {
-                // boolean
-                if let Ok(boolean) = value_string.parse::<bool>() {
-                    json_object_map.insert(key_string, Value::Bool(boolean));
-                } else {
-                    return fail_format!(
-                        "Can't cast into Boolean. key: {key_string}, value: {value_string}, json \
-                         type: boolean"
-                    );
-                }
-            },
-            _ => {
-                unreachable!("we should never get an unknown json type");
-            },
-        }
+        header_types.push((key_string, json_type));
     }
-
-    // dbg!(&json_object_map);
-
-    Ok(Value::Object(json_object_map))
+    Ok(header_types)
 }
 
 #[cfg(test)]
@@ -757,11 +894,12 @@ mod tests_for_csv_to_json_conversion {
 
         let mut rdr = csv::Reader::from_reader(csv.as_bytes());
         let headers = rdr.byte_headers().unwrap().clone();
+        let header_types = get_json_types(&headers, &schema_json()).unwrap();
         let mut record = rdr.byte_records().next().unwrap().unwrap();
         record.trim();
 
         assert_eq!(
-            to_json_instance(&headers, headers.len(), &record, &schema_json())
+            to_json_instance(&header_types, headers.len(), &record)
                 .expect("can't convert csv to json instance"),
             json!({
                 "A": "hello",
@@ -788,19 +926,16 @@ mod tests_for_csv_to_json_conversion {
 
         let mut rdr = csv::Reader::from_reader(csv.as_bytes());
         let headers = rdr.byte_headers().unwrap().clone();
+        let header_types = get_json_types(&headers, &schema_json()).unwrap();
 
         let result = to_json_instance(
-            &headers,
+            &header_types,
             headers.len(),
             &rdr.byte_records().next().unwrap().unwrap(),
-            &schema_json(),
         );
         assert!(&result.is_err());
-        let error = result.err().unwrap();
-        assert_eq!(
-            "Can't cast into Integer. key: C, value: 3.0e8, json type: integer",
-            error
-        );
+        let error = result.err().unwrap().to_string();
+        assert_eq!("Can't cast into Integer. key: C, value: 3.0e8", error);
     }
 }
 
@@ -887,10 +1022,11 @@ mod tests_for_schema_validation {
 
         let mut rdr = csv::Reader::from_reader(csv.as_bytes());
         let headers = rdr.byte_headers().unwrap().clone();
+        let header_types = get_json_types(&headers, &schema_json()).unwrap();
 
         let record = &rdr.byte_records().next().unwrap().unwrap();
 
-        let instance = to_json_instance(&headers, headers.len(), record, &schema_json()).unwrap();
+        let instance = to_json_instance(&header_types, headers.len(), record).unwrap();
 
         let result = validate_json_instance(&instance, &compiled_schema());
 
@@ -905,10 +1041,11 @@ mod tests_for_schema_validation {
 
         let mut rdr = csv::Reader::from_reader(csv.as_bytes());
         let headers = rdr.byte_headers().unwrap().clone();
+        let header_types = get_json_types(&headers, &schema_json()).unwrap();
 
         let record = &rdr.byte_records().next().unwrap().unwrap();
 
-        let instance = to_json_instance(&headers, headers.len(), record, &schema_json()).unwrap();
+        let instance = to_json_instance(&header_types, headers.len(), record).unwrap();
 
         let result = validate_json_instance(&instance, &compiled_schema());
 
