@@ -3,6 +3,7 @@ use std::{
     env, fs,
     io::{self, Read},
     path::{Path, PathBuf},
+    sync::atomic::{AtomicBool, Ordering},
 };
 
 use log::{debug, info, warn};
@@ -15,10 +16,10 @@ use crate::{
     util, CliResult,
 };
 
-// rdr default is 8k in csv crate, we're doubling it
-pub const DEFAULT_RDR_BUFFER_CAPACITY: usize = 16 * (1 << 10);
-// previous wtr default in xsv is 32k, we're doubling it
-pub const DEFAULT_WTR_BUFFER_CAPACITY: usize = 64 * (1 << 10);
+// rdr default is 8k in csv crate, we're making it 128k
+pub const DEFAULT_RDR_BUFFER_CAPACITY: usize = 128 * (1 << 10);
+// previous wtr default in xsv is 32k, we're making it 512k
+pub const DEFAULT_WTR_BUFFER_CAPACITY: usize = 512 * (1 << 10);
 
 // number of rows for qsv_sniffer to sample
 const DEFAULT_SNIFFER_SAMPLE: usize = 100;
@@ -26,7 +27,10 @@ const DEFAULT_SNIFFER_SAMPLE: usize = 100;
 // file size at which we warn user that a large file has not been indexed
 const NO_INDEX_WARNING_FILESIZE: u64 = 100_000_000; // 100MB
 
-#[derive(Clone, Copy, Debug)]
+// so we don't have to keep checking if the index has been created
+static AUTO_INDEXED: AtomicBool = AtomicBool::new(false);
+
+#[derive(Clone, Copy, Debug, PartialEq)]
 pub struct Delimiter(pub u8);
 
 /// Delimiter represents values that can be passed from the command line that
@@ -67,7 +71,7 @@ impl<'de> Deserialize<'de> for Delimiter {
     }
 }
 
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub struct Config {
     pub path:          Option<PathBuf>, // None implies <stdin>
     idx_path:          Option<PathBuf>,
@@ -83,7 +87,7 @@ pub struct Config {
     quoting:           bool,
     pub preamble_rows: u64,
     trim:              csv::Trim,
-    autoindex:         bool,
+    autoindex_size:    u64,
     prefer_dmy:        bool,
     comment:           Option<u8>,
     snappy:            bool, // flag to enable snappy compression/decompression
@@ -171,7 +175,10 @@ impl Config {
             quoting: true,
             preamble_rows: preamble,
             trim: csv::Trim::None,
-            autoindex: util::get_envvar_flag("QSV_AUTOINDEX"),
+            autoindex_size: std::env::var("QSV_AUTOINDEX_SIZE")
+                .unwrap_or_else(|_| "0".to_owned())
+                .parse()
+                .unwrap_or(0),
             prefer_dmy: util::get_envvar_flag("QSV_PREFER_DMY"),
             comment: None,
             snappy,
@@ -354,74 +361,92 @@ impl Config {
                     return;
                 };
                 debug!("autoindex of {path_buf:?} successful.");
+                AUTO_INDEXED.store(true, Ordering::Relaxed);
             },
             Err(e) => debug!("autoindex of {path_buf:?} failed: {e}"),
         }
     }
 
+    /// Check if the index file exists and is newer than the CSV file.
+    /// If so, return the CSV file handle and the index file handle. If not, return None.
+    /// Unless the CSV's file size >= QSV_AUTOINDEX_SIZE, then we'll create an index automatically.
+    /// This will also automatically update stale indices (i.e. the CSV is newer than the index )
     pub fn index_files(&self) -> io::Result<Option<(csv::Reader<fs::File>, fs::File)>> {
-        let (csv_file, idx_file) = match (&self.path, &self.idx_path) {
-            (&None, &None) => return Ok(None),
-            (&None, &Some(_)) => {
-                return Err(io::Error::new(
-                    io::ErrorKind::InvalidInput,
-                    "Cannot use <stdin> with indexes",
-                ));
-            },
-            (Some(p), &None) => {
-                // We generally don't want to report an error here, since we're
-                // passively trying to find an index, so we just log the warning...
-                let idx_file = match fs::File::open(util::idx_path(p)) {
-                    Err(e) => {
-                        if self.autoindex && !self.snappy {
-                            // however, if QSV_AUTOINDEX is set, we create the index automatically
-                            self.autoindex_file();
-                            fs::File::open(util::idx_path(p))?
-                        } else {
-                            warn!("No index file found - {p:?}: {e}");
+        let mut data_modified = 0_u64;
+        let data_fsize;
+        let mut idx_path_work = PathBuf::new();
 
-                            let (_, data_fsize) = util::file_metadata(&p.metadata()?);
+        // the auto_indexed flag is set when an index is created automatically with
+        // autoindex_file(). We use this flag to avoid checking if the index exists every
+        // time this function is called. If the index was already auto-indexed, we can just
+        // use it & return immediately.
+        let auto_indexed = AUTO_INDEXED.load(Ordering::Relaxed);
 
-                            // If the CSV file is larger than NO_INDEX_WARNING_FILESIZE,
-                            // log a warning that the user should consider creating an index file
-                            // for faster access.
-                            if data_fsize > NO_INDEX_WARNING_FILESIZE {
-                                use thousands::Separable;
+        let (csv_file, mut idx_file) = if auto_indexed {
+            (
+                fs::File::open(self.path.clone().unwrap())?,
+                fs::File::open(util::idx_path(&self.path.clone().unwrap()))?,
+            )
+        } else {
+            match (&self.path, &self.idx_path) {
+                (&None, &None) => return Ok(None),
+                (&None, &Some(_)) => {
+                    return Err(io::Error::new(
+                        io::ErrorKind::InvalidInput,
+                        "Cannot use <stdin> with indexes",
+                    ));
+                },
+                (Some(p), Some(ip)) => (fs::File::open(p)?, fs::File::open(ip)?),
+                (Some(p), &None) => {
+                    // We generally don't want to report an error here, since we're
+                    // passively trying to find an index.
+
+                    (data_modified, data_fsize) = util::file_metadata(&p.metadata()?);
+                    idx_path_work = util::idx_path(p);
+                    let idx_file = match fs::File::open(&idx_path_work) {
+                        Err(_) => {
+                            // the index file doesn't exist
+                            if self.snappy {
+                                // cannot index snappy compressed files
+                                return Ok(None);
+                            } else if self.autoindex_size > 0 && data_fsize >= self.autoindex_size {
+                                // if CSV file size >= QSV_AUTOINDEX_SIZE, and
+                                // its not a snappy file, create an index automatically
+                                self.autoindex_file();
+                                fs::File::open(&idx_path_work)?
+                            } else if data_fsize >= NO_INDEX_WARNING_FILESIZE {
+                                // warn user that the CSV file is large and not indexed
+                                use indicatif::HumanCount;
 
                                 warn!(
                                     "The {} MB CSV file is larger than the {} MB \
                                      NO_INDEX_WARNING_FILESIZE threshold. Consider creating an \
-                                     index file for faster access.",
-                                    (data_fsize * 100).separate_with_commas(),
-                                    (NO_INDEX_WARNING_FILESIZE * 100).separate_with_commas()
+                                     index file as it will make qsv commands much faster.",
+                                    HumanCount(data_fsize * 100),
+                                    HumanCount(NO_INDEX_WARNING_FILESIZE * 100)
                                 );
+                                return Ok(None);
+                            } else {
+                                // CSV not greater than QSV_AUTOINDEX_SIZE, and not greater than
+                                // NO_INDEX_WARNING_FILESIZE, so we don't create an index
+                                return Ok(None);
                             }
-                            return Ok(None);
-                        }
-                    },
-                    Ok(f) => f,
-                };
-                (fs::File::open(p)?, idx_file)
-            },
-            (Some(p), Some(ip)) => (fs::File::open(p)?, fs::File::open(ip)?),
+                        },
+                        Ok(f) => f,
+                    };
+                    (fs::File::open(p)?, idx_file)
+                },
+            }
         };
         // If the CSV data was last modified after the index file was last
-        // modified, then return an error and demand the user regenerate the index.
-        // Unless QSV_AUTOINDEX is set, in which case, we'll recreate the
-        // stale index automatically
-        let (data_modified, _) = util::file_metadata(&csv_file.metadata()?);
+        // modified, recreate the stale index automatically
         let (idx_modified, _) = util::file_metadata(&idx_file.metadata()?);
         if data_modified > idx_modified {
-            if self.autoindex && !self.snappy {
-                info!("index stale... autoindexing...");
-                self.autoindex_file();
-            } else {
-                return Err(io::Error::new(
-                    io::ErrorKind::Other,
-                    "The CSV file was modified after the index file. Please re-create the index.",
-                ));
-            }
+            info!("index stale... autoindexing...");
+            self.autoindex_file();
+            idx_file = fs::File::open(&idx_path_work)?;
         }
+
         let csv_rdr = self.from_reader(csv_file);
         Ok(Some((csv_rdr, idx_file)))
     }
@@ -503,11 +528,15 @@ impl Config {
     }
 
     #[allow(clippy::wrong_self_convention)]
-    pub fn from_writer<W: io::Write>(&self, wtr: W) -> csv::Writer<W> {
+    pub fn from_writer<W: io::Write>(&self, mut wtr: W) -> csv::Writer<W> {
         let wtr_capacitys = env::var("QSV_WTR_BUFFER_CAPACITY")
             .unwrap_or_else(|_| DEFAULT_WTR_BUFFER_CAPACITY.to_string());
         let wtr_buffer: usize = wtr_capacitys.parse().unwrap_or(DEFAULT_WTR_BUFFER_CAPACITY);
 
+        if util::get_envvar_flag("QSV_OUTPUT_BOM") {
+            wtr.write_all("\u{FEFF}".as_bytes()).unwrap();
+        }
+
         csv::WriterBuilder::new()
             .flexible(self.flexible)
             .delimiter(self.delimiter)
