@@ -5,6 +5,7 @@
         clippy::cast_possible_wrap,
         clippy::cast_sign_loss,
         // things are often more readable this way
+        clippy::needless_raw_string_hashes,
         clippy::cast_lossless,
         clippy::module_name_repetitions,
         clippy::type_complexity,
@@ -12,7 +13,6 @@
         // correctly used
         clippy::derive_partial_eq_without_eq,
         clippy::enum_glob_use,
-        let_underscore_drop,
         clippy::result_unit_err,
         // not practical
         clippy::similar_names,
@@ -20,18 +20,17 @@
         clippy::struct_excessive_bools,
         // preference
         clippy::doc_markdown,
-        clippy::unseparated_literal_suffix,
         clippy::unnecessary_wraps,
         // false positive
         clippy::needless_doctest_main,
-        // performance
-        clippy::len_zero,
         // noisy
         clippy::missing_errors_doc,
-        clippy::must_use_candidate,
         clippy::use_self,
         clippy::cognitive_complexity,
         clippy::option_if_let_else,
+    ),
+    warn(
+        clippy::missing_asserts_for_indexing,
     )
 )]
 
@@ -40,6 +39,7 @@ use std::{env, io, time::Instant};
 
 extern crate qsv_docopt as docopt;
 use docopt::Docopt;
+use rand::Rng;
 use serde::Deserialize;
 
 use crate::clitypes::{CliError, CliResult, QsvExitCode, CURRENT_COMMAND};
@@ -75,7 +75,7 @@ Options:
     -v, --version        Print version info, mem allocator, features installed, 
                          max_jobs, num_cpus, build info then exit
 
-* sponsored by datHere - Data Infrastructure Engineering
+sponsored by datHere - Data Infrastructure Engineering (https://qsv.datHere.com)
 "#;
 
 #[derive(Deserialize)]
@@ -126,8 +126,9 @@ fn main() -> QsvExitCode {
 
     enabled_commands.push_str("    frequency   Show frequency tables\n");
 
-    #[cfg(all(feature = "generate", not(feature = "lite")))]
-    enabled_commands.push_str("    generate    Generate test data by profiling a CSV\n");
+    #[cfg(all(feature = "geocode", not(feature = "lite")))]
+    enabled_commands
+        .push_str("    geocode     Geocodes a location against the Geonames cities database.\n");
 
     enabled_commands.push_str(
         "    headers     Show header names
@@ -210,9 +211,10 @@ fn main() -> QsvExitCode {
     if args.flag_list {
         wout!("Installed commands ({num_commands}):");
         wout!(
-            "{enabled_commands}\n
-sponsored by datHere - Data Infrastructure Engineering
-        "
+            r#"{enabled_commands}
+
+sponsored by datHere - Data Infrastructure Engineering (https://qsv.datHere.com)
+"#
         );
         util::log_end(qsv_args, now);
         return QsvExitCode::Good;
@@ -232,13 +234,20 @@ sponsored by datHere - Data Infrastructure Engineering
     match args.arg_command {
         None => {
             werr!(
-                "qsv is a suite of CSV command line utilities.
+                r#"qsv is a suite of CSV command line utilities.
 
-Please choose one of the following {num_commands} commands:\n{enabled_commands}\n
-sponsored by datHere - Data Infrastructure Engineering
-"
+Please choose one of the following {num_commands} commands:
+{enabled_commands}
+
+sponsored by datHere - Data Infrastructure Engineering (https://qsv.datHere.com)
+"#
             );
-            _ = util::qsv_check_for_update(true, false);
+
+            // if no command is specified, auto-check for updates 10% of the time
+            let mut rng = rand::thread_rng(); //DevSkim: ignore DS148264
+            if rng.gen_range(0..10) == 0 {
+                _ = util::qsv_check_for_update(true, false);
+            }
             util::log_end(qsv_args, now);
             QsvExitCode::Good
         },
@@ -248,22 +257,27 @@ sponsored by datHere - Data Infrastructure Engineering
                 QsvExitCode::Good
             },
             Err(CliError::Flag(err)) => {
-                werr!("{err}");
+                werr!("usage error: {err}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::IncorrectUsage
+            },
+            Err(CliError::IncorrectUsage(err)) => {
+                werr!("usage error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::IncorrectUsage
             },
             Err(CliError::Csv(err)) => {
-                werr!("{err}");
+                werr!("csv error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::Bad
             },
             Err(CliError::Io(ref err)) if err.kind() == io::ErrorKind::BrokenPipe => {
-                werr!("Broken pipe: {err}");
+                werr!("broken pipe error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::Abort
             },
             Err(CliError::Io(err)) => {
-                werr!("{err}");
+                werr!("io error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::Bad
             },
@@ -276,6 +290,21 @@ sponsored by datHere - Data Infrastructure Engineering
                 util::log_end(qsv_args, now);
                 QsvExitCode::Bad
             },
+            Err(CliError::Network(msg)) => {
+                werr!("network error: {msg}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::NetworkError
+            },
+            Err(CliError::OutOfMemory(msg)) => {
+                werr!("out of memory error: {msg}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::OutOfMemory
+            },
+            Err(CliError::Encoding(msg)) => {
+                werr!("encoding error: {msg}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::EncodingError
+            },
         },
     }
 }
@@ -308,8 +337,8 @@ enum Command {
     #[cfg(all(feature = "foreach", target_family = "unix", not(feature = "lite")))]
     ForEach,
     Frequency,
-    #[cfg(all(feature = "generate", feature = "feature_capable"))]
-    Generate,
+    #[cfg(all(feature = "geocode", feature = "feature_capable"))]
+    Geocode,
     Headers,
     Help,
     Index,
@@ -356,11 +385,12 @@ impl Command {
         let argv: Vec<_> = argv.iter().map(|s| &**s).collect();
         let argv = &*argv;
 
+        assert!(argv.len() > 1);
         if !argv[1].chars().all(char::is_lowercase) {
-            return Err(CliError::Other(format!(
+            return fail_incorrectusage_clierror!(
                 "qsv expects commands in lowercase. Did you mean '{}'?",
                 argv[1].to_lowercase()
-            )));
+            );
         }
 
         CURRENT_COMMAND.get_or_init(|| argv[1].to_lowercase());
@@ -390,8 +420,8 @@ impl Command {
             Command::Flatten => cmd::flatten::run(argv),
             Command::Fmt => cmd::fmt::run(argv),
             Command::Frequency => cmd::frequency::run(argv),
-            #[cfg(all(feature = "generate", feature = "feature_capable"))]
-            Command::Generate => cmd::generate::run(argv),
+            #[cfg(all(feature = "geocode", feature = "feature_capable"))]
+            Command::Geocode => cmd::geocode::run(argv),
             Command::Headers => cmd::headers::run(argv),
             Command::Help => {
                 wout!("{USAGE}");
