@@ -85,7 +85,7 @@ macro_rules! command_list {
 
     NOTE: qsvdp ignores the --progressbar option for all commands.
 
-    sponsored by datHere - Data Infrastructure Engineering
+sponsored by datHere - Data Infrastructure Engineering (https://qsv.datHere.com)
 "
     };
 }
@@ -112,7 +112,7 @@ Options:
     -v, --version        Print version info, mem allocator, features installed, 
                          max_jobs, num_cpus, build info then exit
 
-* sponsored by datHere - Data Infrastructure Engineering
+sponsored by datHere - Data Infrastructure Engineering (https://qsv.datHere.com)
 "#;
 #[derive(Deserialize)]
 struct Args {
@@ -164,7 +164,7 @@ fn main() -> QsvExitCode {
 Please choose one of the following commands:",
                 command_list!()
             ));
-            _ = util::qsv_check_for_update(true, false);
+
             util::log_end(qsv_args, now);
             QsvExitCode::Good
         },
@@ -174,22 +174,27 @@ Please choose one of the following commands:",
                 QsvExitCode::Good
             },
             Err(CliError::Flag(err)) => {
-                werr!("{err}");
+                werr!("usage error: {err}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::IncorrectUsage
+            },
+            Err(CliError::IncorrectUsage(err)) => {
+                werr!("usage error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::IncorrectUsage
             },
             Err(CliError::Csv(err)) => {
-                werr!("{err}");
+                werr!("csv error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::Bad
             },
             Err(CliError::Io(ref err)) if err.kind() == io::ErrorKind::BrokenPipe => {
-                werr!("Broken pipe: {err}");
+                werr!("broken pipe error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::Abort
             },
             Err(CliError::Io(err)) => {
-                werr!("{err}");
+                werr!("io error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::Bad
             },
@@ -202,6 +207,21 @@ Please choose one of the following commands:",
                 util::log_end(qsv_args, now);
                 QsvExitCode::Bad
             },
+            Err(CliError::Network(msg)) => {
+                werr!("network error: {msg}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::NetworkError
+            },
+            Err(CliError::OutOfMemory(msg)) => {
+                werr!("out of memory error: {msg}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::OutOfMemory
+            },
+            Err(CliError::Encoding(msg)) => {
+                werr!("encoding error: {msg}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::EncodingError
+            },
         },
     }
 }
@@ -247,11 +267,12 @@ impl Command {
         let argv: Vec<_> = argv.iter().map(|s| &**s).collect();
         let argv = &*argv;
 
+        assert!(argv.len() > 1);
         if !argv[1].chars().all(char::is_lowercase) {
-            return Err(CliError::Other(format!(
-                "qsvdp expects commands in lowercase. Did you mean '{}'?",
+            return fail_incorrectusage_clierror!(
+                "qsv expects commands in lowercase. Did you mean '{}'?",
                 argv[1].to_lowercase()
-            )));
+            );
         }
         CURRENT_COMMAND.get_or_init(|| argv[1].to_lowercase());
         match self {
