@@ -3,6 +3,7 @@ use std::{env, io, time::Instant};
 
 extern crate qsv_docopt as docopt;
 use docopt::Docopt;
+use rand::Rng;
 use serde::Deserialize;
 
 use crate::clitypes::{CliError, CliResult, QsvExitCode, CURRENT_COMMAND};
@@ -64,7 +65,7 @@ macro_rules! command_list {
     transpose   Transpose rows/columns of CSV data
     validate    Validate CSV data for RFC4180-compliance or with JSON Schema
 
-    sponsored by datHere - Data Infrastructure Engineering
+sponsored by datHere - Data Infrastructure Engineering (https://qsv.datHere.com)
 "
     };
 }
@@ -91,7 +92,7 @@ Options:
     -v, --version        Print version info, mem allocator, features installed, 
                          max_jobs, num_cpus, build info then exit
 
-* sponsored by datHere - Data Infrastructure Engineering
+sponsored by datHere - Data Infrastructure Engineering (https://qsv.datHere.com)
 "#;
 
 #[derive(Deserialize)]
@@ -144,7 +145,12 @@ fn main() -> QsvExitCode {
 Please choose one of the following commands:",
                 command_list!()
             ));
-            _ = util::qsv_check_for_update(true, false);
+
+            // if no command is specified, auto-check for updates 10% of the time
+            let mut rng = rand::thread_rng(); //DevSkim: ignore DS148264
+            if rng.gen_range(0..10) == 0 {
+                _ = util::qsv_check_for_update(true, false);
+            }
             util::log_end(qsv_args, now);
             QsvExitCode::Good
         },
@@ -154,22 +160,27 @@ Please choose one of the following commands:",
                 QsvExitCode::Good
             },
             Err(CliError::Flag(err)) => {
-                werr!("{err}");
+                werr!("usage error: {err}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::IncorrectUsage
+            },
+            Err(CliError::IncorrectUsage(err)) => {
+                werr!("usage error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::IncorrectUsage
             },
             Err(CliError::Csv(err)) => {
-                werr!("{err}");
+                werr!("csv error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::Bad
             },
             Err(CliError::Io(ref err)) if err.kind() == io::ErrorKind::BrokenPipe => {
-                werr!("Broken pipe: {err}");
+                werr!("broken pipe error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::Abort
             },
             Err(CliError::Io(err)) => {
-                werr!("{err}");
+                werr!("io error: {err}");
                 util::log_end(qsv_args, now);
                 QsvExitCode::Bad
             },
@@ -182,6 +193,21 @@ Please choose one of the following commands:",
                 util::log_end(qsv_args, now);
                 QsvExitCode::Bad
             },
+            Err(CliError::Network(msg)) => {
+                werr!("network error: {msg}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::NetworkError
+            },
+            Err(CliError::OutOfMemory(msg)) => {
+                werr!("out of memory error: {msg}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::OutOfMemory
+            },
+            Err(CliError::Encoding(msg)) => {
+                werr!("encoding error: {msg}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::EncodingError
+            },
         },
     }
 }
@@ -242,11 +268,12 @@ impl Command {
         let argv: Vec<_> = argv.iter().map(|s| &**s).collect();
         let argv = &*argv;
 
+        assert!(argv.len() > 1);
         if !argv[1].chars().all(char::is_lowercase) {
-            return Err(CliError::Other(format!(
-                "qsvlite expects commands in lowercase. Did you mean '{}'?",
+            return fail_incorrectusage_clierror!(
+                "qsv expects commands in lowercase. Did you mean '{}'?",
                 argv[1].to_lowercase()
-            )));
+            );
         }
         CURRENT_COMMAND.get_or_init(|| argv[1].to_lowercase());
         match self {
