@@ -8,6 +8,7 @@ use std::{
     path::{Path, PathBuf},
     str,
     sync::OnceLock,
+    time::SystemTime,
 };
 
 use docopt::Docopt;
@@ -18,7 +19,7 @@ use reqwest::Client;
 use serde::de::DeserializeOwned;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 use serde::de::{Deserialize, Deserializer, Error};
-use sysinfo::{System, SystemExt};
+use sysinfo::System;
 
 use crate::{
     config,
@@ -77,19 +78,24 @@ pub fn max_jobs() -> usize {
     }
 }
 
+/// Given a desired number of cores to use
+/// returns number of cores to actually use.
+/// If desired is None, zero, or greater than available cores,
+/// returns max_jobs, which is equal to number of available cores
+/// If desired is Some and less than available cores,
+/// returns desired number of cores
 pub fn njobs(flag_jobs: Option<usize>) -> usize {
     let max_jobs = max_jobs();
-    flag_jobs.map_or(max_jobs, |jobs| {
+    let jobs_to_use = flag_jobs.map_or(max_jobs, |jobs| {
         if jobs == 0 || jobs > max_jobs {
-            env::set_var("RAYON_NUM_THREADS", max_jobs.to_string());
-            log::info!("Using {max_jobs} max processors...");
             max_jobs
         } else {
-            env::set_var("RAYON_NUM_THREADS", jobs.to_string());
-            log::info!("Throttling to {max_jobs} processors...");
             jobs
         }
-    })
+    });
+    env::set_var("RAYON_NUM_THREADS", jobs_to_use.to_string());
+    log::info!("Using {jobs_to_use} jobs...");
+    jobs_to_use
 }
 
 pub fn timeout_secs(timeout: u16) -> Result<u64, String> {
@@ -99,7 +105,7 @@ pub fn timeout_secs(timeout: u16) -> Result<u64, String> {
     };
 
     if timeout > 3600 {
-        return fail!("Timeout cannot be more than 3,600 seconds (1 hour)");
+        return fail_format!("Timeout cannot be more than 3,600 seconds (1 hour): {timeout}");
     } else if timeout == 0 {
         return fail!("Timeout cannot be zero.");
     }
@@ -132,7 +138,7 @@ pub fn set_user_agent(user_agent: Option<String>) -> CliResult<String> {
 
     match HeaderValue::from_str(ua.as_str()) {
         Ok(_) => (),
-        Err(e) => return fail_clierror!("Invalid user-agent value: {e}"),
+        Err(e) => return fail_incorrectusage_clierror!("Invalid user-agent value: {e}"),
     };
 
     log::info!("set user agent: {ua}");
@@ -150,6 +156,8 @@ pub fn version() -> String {
     enabled_features.push_str("foreach;");
     #[cfg(all(feature = "generate", not(feature = "lite")))]
     enabled_features.push_str("generate;");
+    #[cfg(all(feature = "geocode", not(feature = "lite")))]
+    enabled_features.push_str("geocode;");
 
     #[cfg(all(feature = "luau", not(feature = "lite")))]
     {
@@ -245,7 +253,7 @@ pub fn version() -> String {
     }
 }
 
-const OTHER_ENV_VARS: &[&str] = &["no_proxy", "http_proxy", "https_proxy"];
+const OTHER_ENV_VARS: &[&str] = &["all_proxy", "no_proxy", "http_proxy", "https_proxy"];
 
 pub fn show_env_vars() {
     let mut env_var_set = false;
@@ -293,8 +301,9 @@ pub fn count_rows(conf: &Config) -> Result<u64, CliError> {
         let rc = ROW_COUNT.get_or_init(|| {
             if let Ok(mut rdr) = conf.reader() {
                 let mut count = 0_u64;
-                let mut record = csv::ByteRecord::new();
-                while rdr.read_byte_record(&mut record).unwrap_or_default() {
+                let mut _record = csv::ByteRecord::new();
+                #[allow(clippy::used_underscore_binding)]
+                while rdr.read_byte_record(&mut _record).unwrap_or_default() {
                     count += 1;
                 }
                 count
@@ -316,7 +325,7 @@ pub fn count_rows(conf: &Config) -> Result<u64, CliError> {
 pub fn prep_progress(progress: &ProgressBar, record_count: u64) {
     progress.set_style(
         ProgressStyle::default_bar()
-            .template("[{elapsed_precise}] [{bar:25} {percent}%{msg}] ({per_sec} - {eta})")
+            .template("[{elapsed_precise}] [{wide_bar} {percent}%{msg}] ({per_sec} - {eta})")
             .unwrap(),
     );
     progress.set_message(format!(" of {} records", HumanCount(record_count)));
@@ -331,7 +340,7 @@ pub fn prep_progress(progress: &ProgressBar, record_count: u64) {
 pub fn finish_progress(progress: &ProgressBar) {
     progress.set_style(
         ProgressStyle::default_bar()
-            .template("[{elapsed_precise}] [{bar:25} {percent}%{msg}] ({per_sec})")
+            .template("[{elapsed_precise}] [{wide_bar} {percent}%{msg}] ({per_sec})")
             .unwrap(),
     );
 
@@ -344,7 +353,7 @@ pub fn finish_progress(progress: &ProgressBar) {
     }
 }
 
-#[cfg(all(any(feature = "apply", feature = "fetch"), not(feature = "lite")))]
+#[cfg(all(any(feature = "fetch", feature = "geocode"), not(feature = "lite")))]
 macro_rules! update_cache_info {
     ($progress:expr, $cache_instance:expr) => {
         use cached::Cached;
@@ -352,13 +361,18 @@ macro_rules! update_cache_info {
 
         match $cache_instance.lock() {
             Ok(cache) => {
-                if cache.cache_size() > 0 {
+                let size = cache.cache_size();
+                if size > 0 {
                     let hits = cache.cache_hits().unwrap_or_default();
                     let misses = cache.cache_misses().unwrap_or(1);
                     let hit_ratio = (hits as f64 / (hits + misses) as f64) * 100.0;
+                    let capacity = cache.cache_capacity();
                     $progress.set_message(format!(
-                        " of {} records. Cache hit ratio: {hit_ratio:.2}%",
+                        " of {} records. Cache {:.2}% entries: {} capacity: {}.",
                         HumanCount($progress.length().unwrap()),
+                        hit_ratio,
+                        HumanCount(size as u64),
+                        HumanCount(capacity.unwrap() as u64),
                     ));
                 }
             },
@@ -376,7 +390,7 @@ macro_rules! update_cache_info {
     };
 }
 
-#[cfg(all(any(feature = "apply", feature = "fetch"), not(feature = "lite")))]
+#[cfg(all(any(feature = "fetch", feature = "geocode"), not(feature = "lite")))]
 pub(crate) use update_cache_info;
 
 pub fn get_args<T>(usage: &str, argv: &[&str]) -> CliResult<T>
@@ -392,12 +406,17 @@ where
         .map_err(From::from)
 }
 
+#[inline]
 pub fn many_configs(
-    inps: &[String],
+    inps: &[PathBuf],
     delim: Option<Delimiter>,
     no_headers: bool,
+    flexible: bool,
 ) -> Result<Vec<Config>, String> {
-    let mut inps = inps.to_vec();
+    let mut inps = inps
+        .iter()
+        .map(|p| p.to_str().unwrap_or("-").to_owned())
+        .collect::<Vec<_>>();
     if inps.is_empty() {
         inps.push("-".to_owned()); // stdin
     }
@@ -407,6 +426,7 @@ pub fn many_configs(
             Config::new(&Some(p))
                 .delimiter(delim)
                 .no_headers(no_headers)
+                .flexible(flexible)
         })
         .collect::<Vec<_>>();
     errif_greater_one_stdin(&confs)?;
@@ -495,7 +515,7 @@ pub fn mem_file_check(
             fs::metadata(path).map_err(|e| format!("Failed to get file size: {e}"))?;
         let fsize = file_metadata.len();
         if fsize > max_avail_mem {
-            return fail_clierror!(
+            return fail_OOM_clierror!(
                 "Not enough memory to process the file. qsv running in non-streaming {mode} mode. \
                  Total memory: {total_mem} Available memory: {avail_mem}. Free swap: {free_swap} \
                  Max Available memory/Max input file size: {max_avail_mem}. \
@@ -519,6 +539,7 @@ pub fn mem_file_check(
 }
 
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
+#[inline]
 pub fn condense(val: Cow<[u8]>, n: Option<usize>) -> Cow<[u8]> {
     match n {
         None => val,
@@ -751,12 +772,10 @@ pub fn qsv_check_for_update(check_only: bool, no_confirm: bool) -> Result<bool,
 
     let mut updated = false;
     let Ok(latest_release_sv) = semver::Version::parse(latest_release) else {
-        return fail!(format!(
-            "Can't parse latest release version: {latest_release}"
-        ));
+        return fail_format!("Can't parse latest release version: {latest_release}");
     };
     let Ok(curr_version_sv) = semver::Version::parse(curr_version) else {
-        return fail!(format!("Can't parse current version: {curr_version}"));
+        return fail_format!("Can't parse current version: {curr_version}");
     };
 
     if latest_release_sv > curr_version_sv {
@@ -773,6 +792,7 @@ pub fn qsv_check_for_update(check_only: bool, no_confirm: bool) -> Result<bool,
                 .show_output(false)
                 .no_confirm(no_confirm)
                 .current_version(curr_version)
+                .verifying_keys([*include_bytes!("qsv-zipsign-public.key")])
                 .build()
             {
                 Ok(update_job) => match update_job.update() {
@@ -835,7 +855,6 @@ fn send_hwsurvey(
     dry_run: bool,
 ) -> Result<reqwest::StatusCode, String> {
     use serde_json::json;
-    use sysinfo::CpuExt;
 
     static HW_SURVEY_URL: &str =
         "https://4dhmneehnl.execute-api.us-east-1.amazonaws.com/dev/qsv-hwsurvey";
@@ -843,12 +862,10 @@ fn send_hwsurvey(
     let mut sys = System::new();
     sys.refresh_all();
     let total_mem = sys.total_memory();
-    let kernel_version = sys
-        .kernel_version()
-        .unwrap_or_else(|| "Unknown kernel".to_string());
-    let long_os_version = sys
-        .long_os_version()
-        .unwrap_or_else(|| "Unknown OS version".to_string());
+    let kernel_version =
+        sysinfo::System::kernel_version().unwrap_or_else(|| "Unknown kernel".to_string());
+    let long_os_version =
+        sysinfo::System::long_os_version().unwrap_or_else(|| "Unknown OS version".to_string());
     let cpu_count = sys.cpus().len();
     let physical_cpu_count = sys.physical_core_count().unwrap_or_default();
     let cpu_vendor_id = sys.cpus()[0].vendor_id();
@@ -951,24 +968,29 @@ pub fn safe_header_names(
     let safename_regex = regex_oncelock!(r"[^A-Za-z0-9]");
     let mut changed_count = 0_u16;
     let mut name_vec: Vec<String> = Vec::with_capacity(headers.len());
+    let mut safe_name: String;
+    let mut safename_always: String;
+    let mut safename_candidate: String;
+    let mut final_candidate: String;
+    let mut buf_wrk = String::new();
 
     for header_name in headers {
         let reserved_found = if let Some(reserved_names_vec) = reserved_names.clone() {
-            let header_name = if keep_case {
-                header_name.to_string()
+            if keep_case {
+                buf_wrk = header_name.to_owned();
             } else {
-                header_name.to_lowercase()
+                to_lowercase_into(header_name, &mut buf_wrk);
             };
             reserved_names_vec
                 .iter()
-                .any(|reserved_name| reserved_name == header_name.as_str())
+                .any(|reserved_name| reserved_name == &buf_wrk)
         } else {
             false
         };
-        let safe_name = if conditional && is_safe_name(header_name) && !reserved_found {
+        safe_name = if conditional && is_safe_name(header_name) && !reserved_found {
             header_name.to_string()
         } else {
-            let mut safename_always = if header_name.is_empty() {
+            safename_always = if header_name.is_empty() {
                 prefix.to_string()
             } else {
                 safename_regex
@@ -979,14 +1001,14 @@ pub fn safe_header_names(
                 safename_always = format!("{prefix}{safename_always}");
             }
 
-            let safename_candidate = if reserved_found {
+            safename_candidate = if reserved_found {
                 log::warn!("\"{safename_always}\" is a reserved name: {reserved_names:?}");
                 format!("reserved_{safename_always}")
             } else {
                 safename_always
             };
 
-            let mut final_candidate = safename_candidate[..safename_candidate
+            final_candidate = safename_candidate[..safename_candidate
                 .chars()
                 .map(char::len_utf8)
                 .take(60)
@@ -1074,13 +1096,13 @@ fn test_hw_survey() {
     assert!(send_hwsurvey("qsv", false, "0.0.2", "0.0.1", true).is_ok());
 }
 
-#[cfg(any(feature = "apply", feature = "datapusher_plus"))]
+#[cfg(any(feature = "apply", feature = "datapusher_plus", feature = "geocode"))]
 pub struct ColumnNameParser {
     chars: Vec<char>,
     pos:   usize,
 }
 
-#[cfg(any(feature = "apply", feature = "datapusher_plus"))]
+#[cfg(any(feature = "apply", feature = "datapusher_plus", feature = "geocode"))]
 impl ColumnNameParser {
     pub fn new(s: &str) -> ColumnNameParser {
         ColumnNameParser {
@@ -1159,6 +1181,7 @@ impl ColumnNameParser {
     }
 }
 
+#[inline]
 pub fn round_num(dec_f64: f64, places: u32) -> String {
     use rust_decimal::{Decimal, RoundingStrategy};
 
@@ -1181,12 +1204,9 @@ pub fn round_num(dec_f64: f64, places: u32) -> String {
 pub fn transform(bs: &[u8], casei: bool) -> ByteString {
     if let Ok(s) = simdutf8::basic::from_utf8(bs) {
         if casei {
-            let norm: String = s
-                .trim()
-                .chars()
-                .map(|c| c.to_lowercase().next().unwrap())
-                .collect();
-            norm.into_bytes()
+            let mut buffer = String::with_capacity(bs.len());
+            to_lowercase_into(s.trim(), &mut buffer);
+            buffer.into_bytes()
         } else {
             s.trim().as_bytes().to_vec()
         }
@@ -1286,7 +1306,7 @@ pub fn decompress_snappy_file(
 /// if sample_size is Some, it will be used as the number of bytes to download
 pub async fn download_file(
     url: &str,
-    path: &str,
+    path: PathBuf,
     show_progress: bool,
     custom_user_agent: Option<String>,
     download_timeout: Option<u16>,
@@ -1319,11 +1339,7 @@ pub async fn download_file(
         },
     };
 
-    let res = client
-        .get(url)
-        .send()
-        .await
-        .map_err(|e| format!("Download failed: {e}"))?;
+    let res = client.get(url).send().await?;
 
     let total_size = match res.content_length() {
         Some(l) => l,
@@ -1361,14 +1377,13 @@ pub async fn download_file(
     let sample_size = sample_size.unwrap_or(0);
 
     // download chunks
-    let mut file = File::create(path).map_err(|_| format!("Failed to create file '{path}'"))?;
+    let mut file = File::create(path)?;
     let mut downloaded: u64 = 0;
     let mut stream = res.bytes_stream();
 
     while let Some(item) = stream.next().await {
-        let chunk = item.map_err(|_| "Error while downloading file")?;
-        file.write_all(&chunk)
-            .map_err(|_| "Error while writing to file")?;
+        let chunk = item?;
+        file.write_all(&chunk)?;
         let new = min(downloaded + (chunk.len() as u64), total_size);
         downloaded = new;
 
@@ -1385,16 +1400,28 @@ pub async fn download_file(
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     if show_progress {
         pb.finish_with_message(format!("Downloaded {url}"));
-        eprintln!(""); // newline after progress bar
+        eprintln!(); // newline after progress bar
     }
 
     Ok(())
 }
 
-/// load the first BUFFER*4 (128k) bytes of the file and check if it is utf8
+/// this is a non-allocating to_lowercase that uses an existing buffer
+/// and should be faster than the allocating std::to_lowercase
+#[inline]
+pub fn to_lowercase_into(s: &str, buf: &mut String) {
+    buf.clear();
+    for c in s.chars() {
+        for lc in c.to_lowercase() {
+            buf.push(lc);
+        }
+    }
+}
+
+/// load the first BUFFER*8 (1024k) bytes of the file and check if it is utf8
 pub fn isutf8_file(path: &Path) -> Result<bool, CliError> {
     let metadata = std::fs::metadata(path)?;
-    let buffer_len = config::DEFAULT_RDR_BUFFER_CAPACITY * 4;
+    let buffer_len = config::DEFAULT_RDR_BUFFER_CAPACITY * 8;
     let file_size = metadata.len() as usize;
     let bytes_to_read: usize = if file_size < buffer_len {
         file_size
@@ -1410,19 +1437,21 @@ pub fn isutf8_file(path: &Path) -> Result<bool, CliError> {
     Ok(simdutf8::basic::from_utf8(&buffer).is_ok())
 }
 
-/// Process the input files and return a vector of paths to the input files
+/// Process the input files and return a vector of paths to the input files.
 ///
-/// If the input is empty, try to copy stdin to a file named stdin in the passed temp directory
-/// If the input is empty and stdin is empty, return an error
-/// If it's not empty, check the input files if they exist, and return an error if they don't
+/// If the input is empty, try to copy stdin to a file named stdin in the passed temp directory.
+/// If the input is empty and stdin is empty, return an error.
+/// If it's not empty, check the input files if they exist, and return an error if they don't.
 ///
-/// If the input is a directory, add all the files in the directory to the input
-/// If the input is a file, add the file to the input
-/// If the input are snappy compressed files, uncompress them before adding them to the input
+/// If the input is a directory, add all the files in the directory to the input.
+/// If the input is a file with the extension ".infile-list", read the file & add each line as a
+/// file to the input.
+/// If the input is a file, add the file to the input.
+/// If the input are snappy compressed files, uncompress them before adding them to the input.
 pub fn process_input(
     mut arg_input: Vec<PathBuf>,
     tmpdir: &tempfile::TempDir,
-    empty_stdin_errmsg: &str,
+    custom_empty_stdin_errmsg: &str,
 ) -> Result<Vec<PathBuf>, CliError> {
     let mut processed_input = Vec::with_capacity(arg_input.len());
 
@@ -1440,11 +1469,36 @@ pub fn process_input(
         arg_input.remove(0);
     }
 
-    let work_input = if arg_input.len() == 1 && arg_input[0].is_dir() {
-        // if the input is a directory, add all the files in the directory to the input
-        std::fs::read_dir(&arg_input[0])?
-            .map(|entry| entry.map(|e| e.path()))
-            .collect::<Result<Vec<_>, _>>()?
+    let work_input = if arg_input.len() == 1 {
+        let input_path = &arg_input[0];
+        if input_path.is_dir() {
+            // if the input is a directory, add all the files in the directory to the input
+            std::fs::read_dir(input_path)?
+                .map(|entry| entry.map(|e| e.path()))
+                .collect::<Result<Vec<_>, _>>()?
+        } else if input_path.is_file() {
+            // if the input is a file and has the extension "infile-list" case-insensitive,
+            // read the file. Each line is a file path
+            if input_path
+                .extension()
+                .and_then(std::ffi::OsStr::to_str)
+                .map(str::to_lowercase)
+                == Some("infile-list".to_string())
+            {
+                let mut input_file = std::fs::File::open(input_path)?;
+                let mut input_file_contents = String::new();
+                input_file.read_to_string(&mut input_file_contents)?;
+                input_file_contents
+                    .lines()
+                    .map(PathBuf::from)
+                    .collect::<Vec<_>>()
+            } else {
+                // if the input is not an ".infile-list" file, add the file to the input
+                arg_input
+            }
+        } else {
+            arg_input
+        }
     } else {
         arg_input
     };
@@ -1478,8 +1532,36 @@ pub fn process_input(
     }
 
     if processed_input.is_empty() {
-        return fail_clierror!("{empty_stdin_errmsg}");
+        if custom_empty_stdin_errmsg.is_empty() {
+            return fail_clierror!(
+                "No data on stdin. Please provide at least one input file or pipe data to stdin."
+            );
+        }
+        return fail_clierror!("{custom_empty_stdin_errmsg}");
     }
     log::debug!("processed input: {:?}", processed_input);
     Ok(processed_input)
 }
+
+#[inline]
+pub fn replace_column_value(
+    record: &csv::StringRecord,
+    column_index: usize,
+    new_value: &str,
+) -> csv::StringRecord {
+    record
+        .into_iter()
+        .enumerate()
+        .map(|(i, v)| if i == column_index { new_value } else { v })
+        .collect()
+}
+
+pub fn format_systemtime(time: SystemTime, format_specifier: &str) -> String {
+    let timestamp = time
+        .duration_since(SystemTime::UNIX_EPOCH)
+        .unwrap()
+        .as_secs();
+    let naive = chrono::NaiveDateTime::from_timestamp_opt(timestamp as i64, 0).unwrap_or_default();
+    let datetime = chrono::DateTime::<chrono::Utc>::from_naive_utc_and_offset(naive, chrono::Utc);
+    format!("{datetime}", datetime = datetime.format(format_specifier))
+}
