@@ -14,7 +14,7 @@ fn apply_ops_unknown_operation() {
         .arg("data.csv");
 
     let got: String = wrk.output_stderr(&mut cmd);
-    assert_eq!(&*got, "Unknown 'obfuscate' operation\n")
+    assert_eq!(&*got, "usage error: Unknown 'obfuscate' operation\n")
 }
 
 #[test]
@@ -133,7 +133,7 @@ fn apply_ops_upper_rename_invalid() {
     let got: String = wrk.output_stderr(&mut cmd);
     assert_eq!(
         got,
-        "Number of new columns does not match input column selection.\n"
+        "usage error: Number of new columns does not match input column selection.\n"
     );
 }
 
@@ -408,6 +408,70 @@ fn apply_dynfmt_keepcase() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn apply_dynfmt_issue1458() {
+    let wrk = Workdir::new("apply_dynfmt_issue1458");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["FirstName", "MI", "LastName",],
+            svec!["Adam", "B", "Case"],
+            svec!["Derek", "E", "Foster"],
+            svec!["Gordon", "H", "Irvin"],
+            svec!["Jack", "K", "Lynch"],
+        ],
+    );
+    let mut cmd = wrk.command("apply");
+    cmd.arg("dynfmt")
+        .arg("--formatstr")
+        .arg("Sir/Madam {FirstName} {MI}. {LastName}")
+        .arg("-c")
+        .arg("FullName")
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["FirstName", "MI", "LastName", "FullName"],
+        svec!["Adam", "B", "Case", "Sir/Madam Adam B. Case"],
+        svec!["Derek", "E", "Foster", "Sir/Madam Derek E. Foster"],
+        svec!["Gordon", "H", "Irvin", "Sir/Madam Gordon H. Irvin"],
+        svec!["Jack", "K", "Lynch", "Sir/Madam Jack K. Lynch"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn apply_regex_replace_issue1469() {
+    let wrk = Workdir::new("apply_regex_replace_issue1469");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["col1", "col2", "col3",],
+            svec!["(Adam)", "B", "Case(hello)Name "],
+            svec!["Derek(foo)", "(bar)E", "Fos(this needs to go)ter"],
+            svec!["Gordon", "H", "(cmon)Irvin"],
+            svec!["Jack(ie)", "K", "Lynch(-Chan)"],
+        ],
+    );
+    let mut cmd = wrk.command("apply");
+    cmd.arg("operations")
+        .arg("regex_replace")
+        .arg("col1,col2,col3")
+        .args(["--comparand", r"\([^)]+\)"])
+        .args(["--replacement", "<Null>"])
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["col1", "col2", "col3"],
+        svec!["", "B", "CaseName "],
+        svec!["Derek", "E", "Foster"],
+        svec!["Gordon", "H", "Irvin"],
+        svec!["Jack", "K", "Lynch"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn apply_calcconv() {
     let wrk = Workdir::new("apply");
@@ -534,6 +598,58 @@ fn apply_calcconv_units() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn enum_apply_calconv_issue1458() {
+    let wrk = Workdir::new("enum_apply_calconv_issue1458");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["FirstName", "MI", "LastName",],
+            svec!["Adam", "B", "Case"],
+            svec!["Derek", "E", "Foster"],
+            svec!["Gordon", "H", "Irvin"],
+            svec!["Jack", "K", "Lynch"],
+        ],
+    );
+
+    // qsv enum file.csv | qsv apply calcconv --formatstr '{index} * {index}' -c result
+
+    let mut cmd = wrk.command("enum");
+    cmd.arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["FirstName", "MI", "LastName", "index"],
+        svec!["Adam", "B", "Case", "0"],
+        svec!["Derek", "E", "Foster", "1"],
+        svec!["Gordon", "H", "Irvin", "2"],
+        svec!["Jack", "K", "Lynch", "3"],
+    ];
+
+    assert_eq!(got, expected);
+
+    wrk.create("enum.csv", got);
+
+    let mut cmd2 = wrk.command("apply");
+    cmd2.arg("calcconv")
+        .arg("--formatstr")
+        .arg("{index} * {index}")
+        .arg("-c")
+        .arg("result")
+        .arg("enum.csv");
+
+    let got2: Vec<Vec<String>> = wrk.read_stdout(&mut cmd2);
+    let expected2 = vec![
+        svec!["FirstName", "MI", "LastName", "index", "result"],
+        svec!["Adam", "B", "Case", "0", "0"],
+        svec!["Derek", "E", "Foster", "1", "1"],
+        svec!["Gordon", "H", "Irvin", "2", "4"],
+        svec!["Jack", "K", "Lynch", "3", "9"],
+    ];
+
+    assert_eq!(got2, expected2);
+}
+
 #[test]
 fn apply_ops_empty_shortcircuit() {
     let wrk = Workdir::new("apply");
@@ -909,7 +1025,8 @@ fn apply_ops_replace_validation_error() {
     let got = wrk.output_stderr(&mut cmd);
     assert_eq!(
         got,
-        "--comparand (-C) and --replacement (-R) are required for replace operation.\n"
+        "usage error: --comparand (-C) and --replacement (-R) are required for replace \
+         operation.\n"
     );
     wrk.assert_err(&mut cmd);
 }
@@ -972,7 +1089,8 @@ fn apply_ops_regex_replace_validation_error() {
     let got = wrk.output_stderr(&mut cmd);
     assert_eq!(
         got,
-        "--comparand (-C) and --replacement (-R) are required for regex_replace operation.\n"
+        "usage error: --comparand (-C) and --replacement (-R) are required for regex_replace \
+         operation.\n"
     );
     wrk.assert_err(&mut cmd);
 }
@@ -1091,15 +1209,16 @@ fn apply_ops_chain_validation_error() {
     let got = wrk.output_stderr(&mut cmd);
     assert_eq!(
         got,
-        "you can only use censor(0), copy(0), eudex(0), regex_replace(0), replace(0), \
-         sentiment(0), similarity(2), strip(0), and whatlang(0) ONCE per operation series.\n"
+        "usage error: you can only use censor(0), copy(0), eudex(0), regex_replace(0), \
+         replace(0), sentiment(0), similarity(2), strip(0), and whatlang(0) ONCE per operation \
+         series.\n"
     );
     wrk.assert_err(&mut cmd);
 }
 
 #[test]
 fn apply_ops_chain_validation_error_missing_comparand() {
-    let wrk = Workdir::new("apply");
+    let wrk = Workdir::new("apply_ops_chain_validation_error_missing_comparand");
     wrk.create(
         "data.csv",
         vec![svec!["name"], svec!["   John       Paul   "], svec!["Mary"]],
@@ -1115,7 +1234,8 @@ fn apply_ops_chain_validation_error_missing_comparand() {
     let got = wrk.output_stderr(&mut cmd);
     assert_eq!(
         got,
-        "--comparand (-C) and --new_column (-c) is required for similarity operations.\n"
+        "usage error: --comparand (-C) and --new_column (-c) is required for similarity \
+         operations.\n"
     );
     wrk.assert_err(&mut cmd);
 }
@@ -2854,119 +2974,3 @@ fn apply_datefmt_fmtstring_notime() {
     ];
     assert_eq!(got, expected);
 }
-
-#[test]
-fn apply_geocode() {
-    let wrk = Workdir::new("apply");
-    wrk.create(
-        "data.csv",
-        vec![
-            svec!["Location"],
-            svec!["40.812126, -73.9041813"],
-            svec!["40.66472342, -73.93867227"],
-            svec!["(40.766672, -73.9568128)"],
-            svec!["(  40.819342, -73.9532127    )"],
-            svec!["< 40.819342,-73.9532127 >"],
-            svec!["This is not a Location and it will not be geocoded"],
-            svec![
-                "The treasure is at these coordinates 40.66472342, -73.93867227. This should be \
-                 geocoded."
-            ],
-            svec!["95.213424, 190,1234565"], // invalid lat, long
-            svec![
-                "The coordinates are 40.66472342 latitude, -73.93867227 longitudue. This should \
-                 NOT be geocoded."
-            ],
-        ],
-    );
-    let mut cmd = wrk.command("apply");
-    cmd.arg("geocode").arg("Location").arg("data.csv");
-
-    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    let expected = vec![
-        svec!["Location"],
-        svec!["The Bronx, New York"],
-        svec!["Brooklyn, New York"],
-        svec!["Manhattan, New York"],
-        svec!["Edgewater, New Jersey"],
-        svec!["Edgewater, New Jersey"],
-        svec!["This is not a Location and it will not be geocoded"],
-        svec!["Brooklyn, New York"],
-        svec!["95.213424, 190,1234565"], // invalid lat, long
-        svec![
-            "The coordinates are 40.66472342 latitude, -73.93867227 longitudue. This should NOT \
-             be geocoded."
-        ],
-    ];
-    assert_eq!(got, expected);
-}
-
-#[test]
-fn apply_geocode_fmtstring() {
-    let wrk = Workdir::new("apply");
-    wrk.create(
-        "data.csv",
-        vec![
-            svec!["Location"],
-            svec!["40.812126, -73.9041813"],
-            svec!["40.66472342, -73.93867227"],
-            svec!["(40.766672, -73.9568128)"],
-            svec!["(40.819342, -73.9532127)"],
-            svec!["This is not a Location and it will not be geocoded"],
-            svec!["95.213424, 190,1234565"], // invalid lat,long
-        ],
-    );
-    let mut cmd = wrk.command("apply");
-    cmd.arg("geocode")
-        .arg("Location")
-        .arg("--formatstr")
-        .arg("county-country")
-        .arg("data.csv");
-
-    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    let expected = vec![
-        svec!["Location"],
-        svec!["Bronx, US"],
-        svec!["Kings County, US"],
-        svec!["New York County, US"],
-        svec!["Bergen County, US"],
-        svec!["This is not a Location and it will not be geocoded"],
-        svec!["95.213424, 190,1234565"], // invalid lat,long
-    ];
-    assert_eq!(got, expected);
-}
-
-#[test]
-fn apply_geocode_fmtstring_intl() {
-    let wrk = Workdir::new("apply");
-    wrk.create(
-        "data.csv",
-        vec![
-            svec!["Location"],
-            svec!["41.390205, 2.154007"],
-            svec!["52.371807, 4.896029"],
-            svec!["(52.520008, 13.404954)"],
-            svec!["(14.55027,121.03269)"],
-            svec!["This is not a Location and it will not be geocoded"],
-            svec!["95.213424, 190,1234565"], // invalid lat,long
-        ],
-    );
-    let mut cmd = wrk.command("apply");
-    cmd.arg("geocode")
-        .arg("Location")
-        .arg("--formatstr")
-        .arg("city-admin1-country")
-        .arg("data.csv");
-
-    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-    let expected = vec![
-        svec!["Location"],
-        svec!["Barcelona, Catalonia ES"],
-        svec!["Amsterdam, North Holland NL"],
-        svec!["Mitte, Berlin DE"],
-        svec!["Makati City, Metro Manila PH"],
-        svec!["This is not a Location and it will not be geocoded"],
-        svec!["95.213424, 190,1234565"], // invalid lat,long
-    ];
-    assert_eq!(got, expected);
-}
