@@ -14,7 +14,7 @@ fn applydp_ops_unknown_operation() {
         .arg("data.csv");
 
     let got: String = wrk.output_stderr(&mut cmd);
-    assert_eq!(&*got, "Unknown 'obfuscate' operation\n")
+    assert_eq!(&*got, "usage error: Unknown 'obfuscate' operation\n")
 }
 
 #[test]
@@ -133,7 +133,7 @@ fn applydp_ops_upper_rename_invalid() {
     let got: String = wrk.output_stderr(&mut cmd);
     assert_eq!(
         got,
-        "Number of new columns does not match input column selection.\n"
+        "usage error: Number of new columns does not match input column selection.\n"
     );
 }
 
@@ -324,7 +324,8 @@ fn applydp_ops_replace_validation_error() {
     let got = wrk.output_stderr(&mut cmd);
     assert_eq!(
         got,
-        "--comparand (-C) and --replacement (-R) are required for replace operation.\n"
+        "usage error: --comparand (-C) and --replacement (-R) are required for replace \
+         operation.\n"
     );
     wrk.assert_err(&mut cmd);
 }
@@ -365,6 +366,38 @@ fn applydp_ops_regex_replace() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn applydp_regex_replace_issue1469() {
+    let wrk = Workdir::new("applydp_regex_replace_issue1469");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["col1", "col2", "col3",],
+            svec!["(Adam)", "B", "Case(hello)Name "],
+            svec!["Derek(foo)", "(bar)E", "Fos(this needs to go)ter"],
+            svec!["Gordon", "H", "(cmon)Irvin"],
+            svec!["Jack(ie)", "K", "Lynch(-Chan)"],
+        ],
+    );
+    let mut cmd = wrk.command("applydp");
+    cmd.arg("operations")
+        .arg("regex_replace")
+        .arg("col1,col2,col3")
+        .args(["--comparand", r"\([^)]+\)"])
+        .args(["--replacement", "<NULL>"])
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["col1", "col2", "col3"],
+        svec!["", "B", "CaseName "],
+        svec!["Derek", "E", "Foster"],
+        svec!["Gordon", "H", "Irvin"],
+        svec!["Jack", "K", "Lynch"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn applydp_ops_regex_replace_validation_error() {
     let wrk = Workdir::new("applydp");
@@ -387,7 +420,8 @@ fn applydp_ops_regex_replace_validation_error() {
     let got = wrk.output_stderr(&mut cmd);
     assert_eq!(
         got,
-        "--comparand (-C) and --replacement (-R) are required for regex_replace operation.\n"
+        "usage error: --comparand (-C) and --replacement (-R) are required for regex_replace \
+         operation.\n"
     );
     wrk.assert_err(&mut cmd);
 }
@@ -585,8 +619,8 @@ fn applydp_ops_chain_validation_error() {
     let got = wrk.output_stderr(&mut cmd);
     assert_eq!(
         got,
-        "you can only use copy(0), regex_replace(0), replace(0), and strip(2) ONCE per operation \
-         series.\n"
+        "usage error: you can only use copy(0), regex_replace(0), replace(0), and strip(2) ONCE \
+         per operation series.\n"
     );
     wrk.assert_err(&mut cmd);
 }
@@ -607,7 +641,10 @@ fn applydp_ops_chain_validation_error_missing_comparand() {
         .arg("data.csv");
 
     let got = wrk.output_stderr(&mut cmd);
-    assert_eq!(got, "--comparand (-C) is required for strip operations.\n");
+    assert_eq!(
+        got,
+        "usage error: --comparand (-C) is required for strip operations.\n"
+    );
     wrk.assert_err(&mut cmd);
 }
 
