@@ -26,16 +26,38 @@ fn count_width() {
     wrk.create_indexed(
         "in.csv",
         vec![
-            svec!["letter", "number"],
-            svec!["alpha", "13"],
-            svec!["beta", "24"],
+            svec!["letter", "number", "flag"],
+            svec!["alphabetic", "13", "true"],
+            svec!["beta", "24", "false"],
+            svec!["gamma", "37.1", "true"],
+            svec!("delta", "42.5", "false"),
         ],
     );
     let mut cmd = wrk.command("count");
     cmd.arg("--width").arg("in.csv");
 
     let got: String = wrk.stdout(&mut cmd);
-    let expected = "2;9";
+    let expected = "4;18";
+    assert_eq!(got, expected.to_string());
+}
+
+#[test]
+fn count_flexible() {
+    let wrk = Workdir::new("count_flexible");
+    wrk.create_from_string(
+        "in.csv",
+        r#"letter,number,flag
+alphabetic,13,true,extra column
+beta,24,false
+gamma,37.1
+delta,42.5,false
+"#,
+    );
+    let mut cmd = wrk.command("count");
+    cmd.arg("--flexible").arg("in.csv");
+
+    let got: String = wrk.stdout(&mut cmd);
+    let expected = "4";
     assert_eq!(got, expected.to_string());
 }
 
@@ -99,10 +121,10 @@ fn prop_count_len(
     cmd.arg("in.csv");
 
     if human_readable {
-        use thousands::Separable;
+        use indicatif::HumanCount;
 
         let got_count: String = wrk.stdout(&mut cmd);
-        let expected_count_commas = expected_count.separate_with_commas();
+        let expected_count_commas = HumanCount(expected_count as u64).to_string();
 
         rassert_eq!(got_count, expected_count_commas)
     } else {
