@@ -48,10 +48,7 @@ fn excel_open_xlsx_readpassword() {
     cmd.arg(xlsx_file);
 
     let got = wrk.output_stderr(&mut cmd);
-    assert!(got
-        .matches("password-protected-password123.xlsx may be a password-protected workbook:")
-        .min()
-        .is_some());
+    assert_eq!(got, "Xlsx error: Workbook is password protected\n");
     wrk.assert_err(&mut cmd);
 }
 
@@ -195,6 +192,27 @@ fn excel_date_xlsx_date_format() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn excel_xlsx_data_types() {
+    let wrk = Workdir::new("excel_xlsx_data_types");
+
+    let xlsx_file = wrk.load_test_file("excel-xlsx.xlsx");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--sheet").arg("data types").arg(xlsx_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["int", "float", "bool", "date", "duration", "string", "emojis", "foreign"], 
+        svec!["1", "1.1", "true", "2001-09-11", "0.4305555555555556", "The", "The", "敏捷的棕色狐狸在森林里奔跑"], 
+        svec!["2", "1.32434354545454", "false", "2023-10-07", "0.989849537037037", "quick", "🍔", "Franz jagt im komplett verwahrlosten Taxi quer durch Bayern"],
+        svec!["3", "0.423546456564534", "1", "1941-12-07", "1.2815162037037038", "brown", "is", "Le rusé goupil franchit d'un bond le chien somnolent."], 
+        svec!["4", "-54545.6565756785", "0", "2001-09-11 08:30:00", "0.9791666666666666", "fox", "💩", "El rápido zorro marrón"],
+        svec!["5", "-5446563454.43546", "true", "1945-08-06 08:15:00", "0.0004629629629629629", "jumped", "🙀", "いろはにほへとちりぬるをわかよたれそつねならむうゐのおくやまけふこえてあさきゆめみしゑひもせす"]
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn excel_date_xlsx() {
     let wrk = Workdir::new("excel_date_xls");
@@ -323,7 +341,7 @@ fn excel_invalid_sheet_index() {
     cmd.arg("--sheet").arg("100").arg(xls_file);
 
     let got = wrk.output_stderr(&mut cmd);
-    let expected = "sheet index 100 is greater than number of sheets 8\n".to_string();
+    let expected = "usage error: sheet index 100 is greater than number of sheets 8\n".to_string();
     assert_eq!(got, expected);
     wrk.assert_err(&mut cmd);
 }
@@ -419,6 +437,8 @@ fn excel_metadata() {
         svec![
             "index",
             "sheet_name",
+            "type",
+            "visible",
             "headers",
             "num_columns",
             "num_rows",
@@ -432,6 +452,8 @@ fn excel_metadata() {
             "0",
             "First",
             "[\"URL\", \"City\"]",
+            "WorkSheet",
+            "Visible",
             "2",
             "4",
             "[\"URL\", \"City\"]",
@@ -444,6 +466,8 @@ fn excel_metadata() {
             "1",
             "Flexibility Test",
             "[\"URL\", \"City\", \"\"]",
+            "WorkSheet",
+            "Visible",
             "3",
             "6",
             "[\"URL\", \"City\"]",
@@ -456,6 +480,8 @@ fn excel_metadata() {
             "2",
             "Middle",
             "[\"Middle sheet col1\", \"Middle-2\"]",
+            "WorkSheet",
+            "Visible",
             "2",
             "6",
             "[\"Middle sheet col1\", \"Middle-2\"]",
@@ -464,11 +490,26 @@ fn excel_metadata() {
             "0",
             "0"
         ],
-        svec!["3", "Sheet1", "[]", "0", "0", "[]", "0", "[]", "0", "0"],
+        svec![
+            "3",
+            "Sheet1",
+            "[]",
+            "WorkSheet",
+            "Visible",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
         svec![
             "4",
             "trim test",
             "[\"col1\", \"   col2\", \"col3\"]",
+            "WorkSheet",
+            "Visible",
             "3",
             "6",
             "[\"col1\", \"col3\"]",
@@ -481,6 +522,8 @@ fn excel_metadata() {
             "5",
             "date test",
             "[\"date_col\", \"num_col\", \"col_Petsa\", \"just another col\"]",
+            "WorkSheet",
+            "Visible",
             "4",
             "6",
             "[\"date_col\", \"num_col\", \"col_Petsa\", \"just another col\"]",
@@ -493,6 +536,8 @@ fn excel_metadata() {
             "6",
             "NoData",
             "[\"col1\", \"col2\", \"col3\", \"col4\"]",
+            "WorkSheet",
+            "Visible",
             "4",
             "1",
             "[\"col1\", \"col2\", \"col3\", \"col4\"]",
@@ -505,6 +550,8 @@ fn excel_metadata() {
             "7",
             "Last",
             "[\"Last sheet col1\", \"Last-2\"]",
+            "WorkSheet",
+            "Visible",
             "2",
             "6",
             "[\"Last sheet col1\", \"Last-2\"]",
@@ -528,12 +575,14 @@ fn excel_metadata_pretty_json() {
     cmd.arg("--metadata").arg("J").arg(xls_file);
 
     let got: String = wrk.stdout(&mut cmd);
-    let expected: &str = r#""format": "xls",
+    let expected: &str = r#""format": "Excel: xls",
   "num_sheets": 8,
   "sheet": [
     {
       "index": 0,
       "name": "First",
+      "typ": "WorkSheet",
+      "visible": "Visible",
       "headers": [
         "URL",
         "City"
@@ -552,6 +601,8 @@ fn excel_metadata_pretty_json() {
     {
       "index": 1,
       "name": "Flexibility Test",
+      "typ": "WorkSheet",
+      "visible": "Visible",
       "headers": [
         "URL",
         "City",
@@ -573,6 +624,8 @@ fn excel_metadata_pretty_json() {
     {
       "index": 2,
       "name": "Middle",
+      "typ": "WorkSheet",
+      "visible": "Visible",
       "headers": [
         "Middle sheet col1",
         "Middle-2"
@@ -591,6 +644,8 @@ fn excel_metadata_pretty_json() {
     {
       "index": 3,
       "name": "Sheet1",
+      "typ": "WorkSheet",
+      "visible": "Visible",
       "headers": [],
       "num_columns": 0,
       "num_rows": 0,
@@ -603,6 +658,8 @@ fn excel_metadata_pretty_json() {
     {
       "index": 4,
       "name": "trim test",
+      "typ": "WorkSheet",
+      "visible": "Visible",
       "headers": [
         "col1",
         "   col2",
@@ -624,6 +681,8 @@ fn excel_metadata_pretty_json() {
     {
       "index": 5,
       "name": "date test",
+      "typ": "WorkSheet",
+      "visible": "Visible",
       "headers": [
         "date_col",
         "num_col",
@@ -646,6 +705,8 @@ fn excel_metadata_pretty_json() {
     {
       "index": 6,
       "name": "NoData",
+      "typ": "WorkSheet",
+      "visible": "Visible",
       "headers": [
         "col1",
         "col2",
@@ -668,6 +729,8 @@ fn excel_metadata_pretty_json() {
     {
       "index": 7,
       "name": "Last",
+      "typ": "WorkSheet",
+      "visible": "Visible",
       "headers": [
         "Last sheet col1",
         "Last-2"
@@ -689,6 +752,437 @@ fn excel_metadata_pretty_json() {
     wrk.assert_success(&mut cmd);
 }
 
+#[test]
+fn ods_metadata() {
+    let wrk = Workdir::new("ods_metadata");
+
+    let xls_file = wrk.load_test_file("excel-ods.ods");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--metadata").arg("c").arg(xls_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "index",
+            "sheet_name",
+            "type",
+            "visible",
+            "headers",
+            "num_columns",
+            "num_rows",
+            "safe_headers",
+            "safe_headers_count",
+            "unsafe_headers",
+            "unsafe_headers_count",
+            "duplicate_headers_count"
+        ],
+        svec![
+            "0",
+            "Sheet1",
+            "[\"URL\", \"City\"]",
+            "WorkSheet",
+            "Visible",
+            "2",
+            "4",
+            "[\"URL\", \"City\"]",
+            "2",
+            "[]",
+            "0",
+            "0"
+        ],
+    ];
+
+    assert_eq!(got, expected);
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn ods_metadata_pretty_json() {
+    let wrk = Workdir::new("ods_metadata_pretty_json");
+
+    let xls_file = wrk.load_test_file("excel-ods.ods");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--metadata").arg("J").arg(xls_file);
+
+    let got: String = wrk.stdout(&mut cmd);
+    let expected: &str = r#"excel-ods.ods",
+  "format": "ODS",
+  "num_sheets": 1,
+  "sheet": [
+    {
+      "index": 0,
+      "name": "Sheet1",
+      "typ": "WorkSheet",
+      "visible": "Visible",
+      "headers": [
+        "URL",
+        "City"
+      ],
+      "num_columns": 2,
+      "num_rows": 4,
+      "safe_headers": [
+        "URL",
+        "City"
+      ],
+      "safe_headers_count": 2,
+      "unsafe_headers": [],
+      "unsafe_headers_count": 0,
+      "duplicate_headers_count": 0
+    }
+  ]
+}"#;
+    assert!(got.ends_with(expected));
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn excel_metadata_sheet_types() {
+    let wrk = Workdir::new("excel_metadata_sheet_types");
+
+    let xls_file = wrk.load_test_file("any_sheets.xls");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--metadata").arg("csv").arg(xls_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "index",
+            "sheet_name",
+            "type",
+            "visible",
+            "headers",
+            "num_columns",
+            "num_rows",
+            "safe_headers",
+            "safe_headers_count",
+            "unsafe_headers",
+            "unsafe_headers_count",
+            "duplicate_headers_count"
+        ],
+        svec![
+            "0",
+            "Visible",
+            "[\"1\", \"2\"]",
+            "WorkSheet",
+            "Visible",
+            "2",
+            "5",
+            "[]",
+            "0",
+            "[\"1\", \"2\"]",
+            "2",
+            "0"
+        ],
+        svec![
+            "1",
+            "Hidden",
+            "[\"1\", \"2\"]",
+            "WorkSheet",
+            "Hidden",
+            "2",
+            "3",
+            "[]",
+            "0",
+            "[\"1\", \"2\"]",
+            "2",
+            "0"
+        ],
+        svec![
+            "2",
+            "VeryHidden",
+            "[]",
+            "WorkSheet",
+            "VeryHidden",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+        svec![
+            "3",
+            "Chart",
+            "[\"1\", \"2\"]",
+            "ChartSheet",
+            "Visible",
+            "2",
+            "3",
+            "[]",
+            "0",
+            "[\"1\", \"2\"]",
+            "2",
+            "0"
+        ],
+    ];
+    assert_eq!(got, expected);
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn excel_metadata_sheet_types_xlsx() {
+    let wrk = Workdir::new("excel_metadata_sheet_types_xlsx");
+
+    let xlsx_file = wrk.load_test_file("any_sheets.xlsx");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--metadata").arg("csv").arg(xlsx_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "index",
+            "sheet_name",
+            "type",
+            "visible",
+            "headers",
+            "num_columns",
+            "num_rows",
+            "safe_headers",
+            "safe_headers_count",
+            "unsafe_headers",
+            "unsafe_headers_count",
+            "duplicate_headers_count"
+        ],
+        svec![
+            "0",
+            "Visible",
+            "[\"1\", \"2\"]",
+            "WorkSheet",
+            "Visible",
+            "2",
+            "5",
+            "[]",
+            "0",
+            "[\"1\", \"2\"]",
+            "2",
+            "0"
+        ],
+        svec![
+            "1",
+            "Hidden",
+            "[]",
+            "WorkSheet",
+            "Hidden",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+        svec![
+            "2",
+            "VeryHidden",
+            "[]",
+            "WorkSheet",
+            "VeryHidden",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+        // we don't get metadata for chart sheets in xlsx
+        svec![
+            "3",
+            "Chart",
+            "[]",
+            "ChartSheet",
+            "Visible",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+    ];
+    assert_eq!(got, expected);
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn excel_metadata_sheet_types_xlsb() {
+    let wrk = Workdir::new("excel_metadata_sheet_types_xlsb");
+
+    let xlsb_file = wrk.load_test_file("any_sheets.xlsb");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--metadata").arg("csv").arg(xlsb_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "index",
+            "sheet_name",
+            "type",
+            "visible",
+            "headers",
+            "num_columns",
+            "num_rows",
+            "safe_headers",
+            "safe_headers_count",
+            "unsafe_headers",
+            "unsafe_headers_count",
+            "duplicate_headers_count"
+        ],
+        svec![
+            "0",
+            "Visible",
+            "[\"1\", \"2\"]",
+            "WorkSheet",
+            "Visible",
+            "2",
+            "5",
+            "[]",
+            "0",
+            "[\"1\", \"2\"]",
+            "2",
+            "0"
+        ],
+        svec![
+            "1",
+            "Hidden",
+            "[]",
+            "WorkSheet",
+            "Hidden",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+        svec![
+            "2",
+            "VeryHidden",
+            "[]",
+            "WorkSheet",
+            "VeryHidden",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+        // we don't get metadata for chart sheets in xlsb
+        svec![
+            "3",
+            "Chart",
+            "[]",
+            "ChartSheet",
+            "Visible",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+    ];
+    assert_eq!(got, expected);
+    wrk.assert_success(&mut cmd);
+}
+
+#[test]
+fn excel_metadata_sheet_types_ods() {
+    let wrk = Workdir::new("excel_metadata_sheet_types_ods");
+
+    let ods_file = wrk.load_test_file("any_sheets.ods");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--metadata").arg("csv").arg(ods_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec![
+            "index",
+            "sheet_name",
+            "type",
+            "visible",
+            "headers",
+            "num_columns",
+            "num_rows",
+            "safe_headers",
+            "safe_headers_count",
+            "unsafe_headers",
+            "unsafe_headers_count",
+            "duplicate_headers_count"
+        ],
+        svec![
+            "0",
+            "Visible",
+            "[\"1\", \"2\"]",
+            "WorkSheet",
+            "Visible",
+            "2",
+            "5",
+            "[]",
+            "0",
+            "[\"1\", \"2\"]",
+            "2",
+            "0"
+        ],
+        svec![
+            "1",
+            "Hidden",
+            "[]",
+            "WorkSheet",
+            "Hidden",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+        svec![
+            "2",
+            "VeryHidden",
+            "[]",
+            "WorkSheet",
+            "Hidden",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+        svec![
+            "3",
+            "Chart",
+            "[]",
+            "WorkSheet",
+            "Visible",
+            "0",
+            "0",
+            "[]",
+            "0",
+            "[]",
+            "0",
+            "0"
+        ],
+    ];
+    assert_eq!(got, expected);
+    wrk.assert_success(&mut cmd);
+}
+
 #[test]
 fn excel_message() {
     let wrk = Workdir::new("excel_message");
@@ -725,7 +1219,8 @@ fn excel_empty_sheet2_message() {
     cmd.arg("--sheet").arg("Sheet1").arg(xls_file);
 
     let got = wrk.output_stderr(&mut cmd);
-    assert_eq!(got, "0 0-column rows exported from \"Sheet1\" sheet\n");
+    assert_eq!(got, "\"Sheet1\" sheet is empty.\n");
+    wrk.assert_err(&mut cmd);
 }
 
 #[test]
@@ -942,3 +1437,15 @@ fn excel_range_empty_sheet() {
         .is_some());
     wrk.assert_err(&mut cmd);
 }
+
+#[test]
+fn excel_formula_empty_string_value() {
+    let wrk = Workdir::new("formula_empty_string_value");
+
+    let xls_file = wrk.load_test_file("formula_empty_string_value.xls");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg(xls_file);
+
+    wrk.assert_success(&mut cmd);
+}
