@@ -5,6 +5,7 @@ macro_rules! sqlp_test {
         mod $name {
             use std::process;
 
+            #[allow(unused_imports)]
             use super::{make_rows, setup};
             use crate::workdir::Workdir;
 
@@ -93,28 +94,24 @@ sqlp_test!(
     |wrk: Workdir, mut cmd: process::Command| {
         cmd.arg("select * from cities full outer join places on cities.city = places.city");
         let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
-        let expected1 = make_rows(
-            false,
-            vec![
-                svec!["Boston", "MA", "Logan Airport"],
-                svec!["Boston", "MA", "Boston Garden"],
-                svec!["Buffalo", "NY", "Ralph Wilson Stadium"],
-                svec!["Orlando", "", "Disney World"],
-                svec!["San Francisco", "CA", ""],
-                svec!["New York", "NY", ""],
-            ],
-        );
-        let expected2 = make_rows(
-            false,
-            vec![
-                svec!["Boston", "MA", "Logan Airport"],
-                svec!["Boston", "MA", "Boston Garden"],
-                svec!["Buffalo", "NY", "Ralph Wilson Stadium"],
-                svec!["Orlando", "", "Disney World"],
-                svec!["New York", "NY", ""],
-                svec!["San Francisco", "CA", ""],
-            ],
-        );
+        let expected1 = vec![
+            svec!["city", "state", "city_right", "place"],
+            svec!["Boston", "MA", "Boston", "Logan Airport"],
+            svec!["Boston", "MA", "Boston", "Boston Garden"],
+            svec!["Buffalo", "NY", "Buffalo", "Ralph Wilson Stadium"],
+            svec!["", "", "Orlando", "Disney World"],
+            svec!["San Francisco", "CA", "", ""],
+            svec!["New York", "NY", "", ""],
+        ];
+        let expected2 = vec![
+            svec!["city", "state", "city_right", "place"],
+            svec!["Boston", "MA", "Boston", "Logan Airport"],
+            svec!["Boston", "MA", "Boston", "Boston Garden"],
+            svec!["Buffalo", "NY", "Buffalo", "Ralph Wilson Stadium"],
+            svec!["", "", "Orlando", "Disney World"],
+            svec!["New York", "NY", "", ""],
+            svec!["San Francisco", "CA", "", ""],
+        ];
         assert!(got == expected1 || got == expected2);
     }
 );
@@ -269,14 +266,14 @@ fn sqlp_boston311_groupby_orderby_with_table_alias() {
 }
 
 #[test]
-fn sqlp_boston311_null_value() {
-    let wrk = Workdir::new("sqlp_boston311_null_value");
+fn sqlp_boston311_wnull_value() {
+    let wrk = Workdir::new("sqlp_boston311_wnull_value");
     let test_file = wrk.load_test_file("boston311-100.csv");
 
     let mut cmd = wrk.command("sqlp");
 
     cmd.arg(&test_file)
-        .args(["--null-value", "Not Specified"])
+        .args(["--wnull-value", "Not Specified"])
         .arg(
             "select location_street_name, location_zipcode from _t_1 where location_zipcode is \
              null order by location_street_name limit 5",
@@ -314,14 +311,16 @@ fn sqlp_null_aware_equality_checks() {
 
     let mut cmd = wrk.command("sqlp");
 
-    cmd.arg("test_null.csv").args(["--null-value", "NULL"]).arg(
-        r#"SELECT (a = b) as "1_eq_unaware",
+    cmd.arg("test_null.csv")
+        .args(["--wnull-value", "NULL"])
+        .arg(
+            r#"SELECT (a = b) as "1_eq_unaware",
            (a != b) as "2_neq_unaware", 
            (a <=> b) as "3_eq_aware", 
            (a IS NOT DISTINCT FROM b) as "4_eq_aware", 
            (a IS DISTINCT FROM b) as "5_neq_aware" 
          FROM test_null"#,
-    );
+        );
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
@@ -341,6 +340,41 @@ fn sqlp_null_aware_equality_checks() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn sqlp_rnull_values() {
+    let wrk = Workdir::new("sqlp_rnull_values");
+    wrk.create(
+        "test_null.csv",
+        vec![
+            svec!["a", "b"],
+            svec!["1", "NULL"],
+            svec!["2", "NA"],
+            svec!["3", "Dunno"],
+            svec!["4", "4"],
+            svec!["5", ""],
+            svec!("6", "6"),
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+
+    cmd.arg("test_null.csv")
+        .args(["--rnull-values", "NULL,NA,Dunno"])
+        .arg("SELECT * FROM test_null");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["a", "b"],
+        svec!["1", ""],
+        svec!["2", ""],
+        svec!["3", ""],
+        svec!["4", "4"],
+        svec!["5", ""],
+        svec!["6", "6"],
+    ];
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn sqlp_regex_operators() {
     let wrk = Workdir::new("sqlp_regex_operators");
@@ -443,7 +477,7 @@ fn sqlp_string_functions() {
             svec!["abc"],
             svec!["    abc"],
             svec!["a"],
-            svec![""],
+            svec!["b"],
         ],
     );
 
@@ -491,7 +525,7 @@ fn sqlp_string_functions() {
         svec!["abc"],
         svec!["abc"],
         svec!["a"],
-        svec![""],
+        svec!["b"],
     ];
     assert_eq!(got, expected);
 
@@ -503,9 +537,9 @@ fn sqlp_string_functions() {
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["scol"],
-        svec!["cd"],
-        svec!["c"],
-        svec!["c"],
+        svec!["bc"],
+        svec!["bc"],
+        svec!["bc"],
         svec![""],
         svec![""],
     ];
@@ -523,7 +557,7 @@ fn sqlp_string_functions() {
         svec!["ABC"],
         svec!["ABC"],
         svec!["A"],
-        svec![""],
+        svec!["B"],
     ];
     assert_eq!(got, expected);
 
@@ -539,7 +573,7 @@ fn sqlp_string_functions() {
         svec!["abc"],
         svec!["abc"],
         svec!["a"],
-        svec![""],
+        svec!["b"],
     ];
     assert_eq!(got, expected);
 
@@ -555,7 +589,7 @@ fn sqlp_string_functions() {
         svec!["3"],
         svec!["3"],
         svec!["1"],
-        svec!["0"],
+        svec!["1"],
     ];
     assert_eq!(got, expected);
 
@@ -571,7 +605,7 @@ fn sqlp_string_functions() {
         svec!["3"],
         svec!["3"],
         svec!["1"],
-        svec!["0"],
+        svec!["1"],
     ];
     assert_eq!(got, expected);
 }
@@ -582,46 +616,49 @@ fn sqlp_boston311_try_parsedates() {
     let test_file = wrk.load_test_file("boston311-100.csv");
 
     let mut cmd = wrk.command("sqlp");
-    cmd.arg(&test_file).arg("--try-parsedates").arg(
-        "select ward, cast(avg(closed_dt - open_dt) as float) as avg_tat from _t_1 where \
-         case_status = 'Closed' group by ward order by avg_tat desc, ward asc",
-    );
+    cmd.arg(&test_file)
+        .arg("--try-parsedates")
+        .arg(
+            "select ward, cast(avg(closed_dt - open_dt) as float) as avg_tat from _t_1 where \
+             case_status = 'Closed' group by ward order by avg_tat desc, ward asc",
+        )
+        .arg("--ignore-errors");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["ward", "avg_tat"],
-        svec!["Ward 11", "4.84776e12"],
-        svec!["01", "4.81827e12"],
-        svec!["Ward 13", "1.5183657e12"],
-        svec!["Ward 15", "1.278926e12"],
-        svec!["Ward 21", "8.78446e11"],
-        svec!["Ward 14", "6.18933e11"],
-        svec!["Ward 3", "4.3769145e11"],
-        svec!["Ward 5", "4.119095e11"],
-        svec!["Ward 20", "3.67233e11"],
-        svec!["9", "3.53495e11"],
-        svec!["Ward 18", "2.49882e11"],
-        svec!["19", "2.12566e11"],
-        svec!["Ward 4", "1.128726e11"],
-        svec!["Ward 1", "1.0785067e11"],
-        svec!["Ward 10", "1.0411e11"],
-        svec!["16", "9.3557e10"],
-        svec!["Ward 19", "8.4164e10"],
-        svec!["10", "7.9101e10"],
-        svec!["21", "7.7717e10"],
-        svec!["7", "7.4611e10"],
-        svec!["17", "7.01175e10"],
-        svec!["3", "6.88366e10"],
-        svec!["Ward 9", "6.4097e10"],
-        svec!["Ward 12", "6.293e10"],
-        svec!["Ward 6", "5.4770168e10"],
-        svec!["Ward 7", "3.8346334e10"],
-        svec!["Ward 8", "3.27675e10"],
-        svec!["03", "2.98105e10"],
-        svec!["07", "2.5328001e10"],
-        svec!["22", "2.3919e10"],
-        svec!["14", "2.07865e10"],
-        svec!["Ward 22", "1.3524e10"],
+        svec!["Ward 11", "4847760000000.0"],
+        svec!["01", "4818270000000.0"],
+        svec!["Ward 13", "1518365700000.0"],
+        svec!["Ward 15", "1278926000000.0"],
+        svec!["Ward 21", "878446000000.0"],
+        svec!["Ward 14", "618933000000.0"],
+        svec!["Ward 3", "437691450000.0"],
+        svec!["Ward 5", "411909500000.0"],
+        svec!["Ward 20", "367233000000.0"],
+        svec!["9", "353495000000.0"],
+        svec!["Ward 18", "249882000000.0"],
+        svec!["19", "212566000000.0"],
+        svec!["Ward 4", "112872600000.0"],
+        svec!["Ward 1", "107850670000.0"],
+        svec!["Ward 10", "104110000000.0"],
+        svec!["16", "93557000000.0"],
+        svec!["Ward 19", "84164000000.0"],
+        svec!["10", "79101000000.0"],
+        svec!["21", "77717000000.0"],
+        svec!["7", "74611000000.0"],
+        svec!["17", "70117500000.0"],
+        svec!["3", "68836600000.0"],
+        svec!["Ward 9", "64097000000.0"],
+        svec!["Ward 12", "62930000000.0"],
+        svec!["Ward 6", "54770168000.0"],
+        svec!["Ward 7", "38346334000.0"],
+        svec!["Ward 8", "32767500000.0"],
+        svec!["03", "29810500000.0"],
+        svec!["07", "25328000000.0"],
+        svec!["22", "23919000000.0"],
+        svec!["14", "20786500000.0"],
+        svec!["Ward 22", "13524000000.0"],
         svec!["1", "9469000000.0"],
         svec!["06", "5290000000.0"],
         svec!["Ward 16", "4533667000.0"],
@@ -644,7 +681,8 @@ fn sqlp_boston311_try_parsedates_precision() {
         .arg(
             "select ward, cast(avg(closed_dt - open_dt) as float) as avg_tat from _t_1 where \
              case_status = 'Closed' group by ward order by avg_tat desc, ward asc limit 5",
-        );
+        )
+        .arg("--ignore-errors");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
@@ -667,7 +705,8 @@ fn sqlp_boston311_try_parsedates_format() {
     cmd.arg(&test_file)
         .arg("--try-parsedates")
         .args(["--datetime-format", "%a %Y-%m-%d %H:%M:%S"])
-        .arg("select closed_dt, open_dt from _t_1 where case_status = 'Closed' limit 5");
+        .arg("select closed_dt, open_dt from _t_1 where case_status = 'Closed' limit 5")
+        .arg("--ignore-errors");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
@@ -681,6 +720,77 @@ fn sqlp_boston311_try_parsedates_format() {
     assert_eq!(got, expected);
 }
 
+#[test]
+fn sqlp_comments() {
+    let wrk = Workdir::new("sqlp_comments");
+    wrk.create(
+        "comments.csv",
+        vec![
+            svec!["# test file to see how comments work", ""],
+            svec!["# this is another comment before the header", ""],
+            svec!["# DATA DICTIONARY", ""],
+            svec!["# column1 - alphabetic; id of the column", ""],
+            svec!["# column2 - numeric; just a number", ""],
+            svec!["column1", "column2"],
+            svec!["a", "1"],
+            svec!["#b", "2"],
+            svec!["c", "3"],
+            svec!["#d - this row is corrupted skip", "extra col2"],
+            svec!["e", "5"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.env("QSV_COMMENT_CHAR", "#");
+    cmd.arg("comments.csv")
+        .arg("select column1, column2 from comments order by column2 desc");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["column1", "column2"],
+        svec!["e", "5"],
+        svec!["c", "3"],
+        svec!["a", "1"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_compress() {
+    let wrk = Workdir::new("sqlp_compress");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["column1", "column2"],
+            svec!["a", "1"],
+            svec!["c", "3"],
+            svec!["e", "5"],
+        ],
+    );
+
+    let out_file = wrk.path("out.csv.sz").to_string_lossy().to_string();
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("data.csv")
+        .arg("select column1, column2 from data order by column2 desc")
+        .args(["-o", &out_file]);
+
+    wrk.assert_success(&mut cmd);
+
+    let mut cmd2 = wrk.command("snappy"); // DevSkim: ignore DS126858
+    cmd2.arg("decompress").arg(out_file); // DevSkim: ignore DS126858
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd2); // DevSkim: ignore DS126858
+    let expected = vec![
+        svec!["column1", "column2"],
+        svec!["e", "5"],
+        svec!["c", "3"],
+        svec!["a", "1"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn sqlp_boston311_explain() {
     let wrk = Workdir::new("sqlp_boston311_explain");
@@ -701,7 +811,8 @@ fn sqlp_boston311_explain() {
     assert!(got.starts_with(expected_begin));
 
     let expected_end = r#"boston311-100.csv
-        PROJECT 4/29 COLUMNS"#;
+        PROJECT 4/29 COLUMNS
+"        SELECTION: [(col(""case_status"")) == (String(Closed))]""#;
     assert!(got.ends_with(expected_end));
 }
 
@@ -831,14 +942,14 @@ fn sqlp_boston311_case_expression() {
               WHEN True THEN 'Yes' 
               WHEN False THEN 'No' 
               ELSE 'N/A'
-           END as grafitti_related
+           END as graffiti_related
            from _t_1
            where case_status = 'Open'"#,
     );
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
-        svec!["case_enquiry_id", "grafitti_related"],
+        svec!["case_enquiry_id", "graffiti_related"],
         svec!["101004143000", "No"],
         svec!["101004155594", "No"],
         svec!["101004154423", "No"],
@@ -899,3 +1010,173 @@ fn sqlp_boston311_case() {
 
     assert_eq!(got, expected);
 }
+
+#[test]
+fn sqlp_literal_pattern_match() {
+    let wrk = Workdir::new("sqlp_literal_pattern_match");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["idx", "val"],
+            svec!["0", "ABC"],
+            svec!["1", "abc"],
+            svec!["2", "000"],
+            svec!["3", "A0C"],
+            svec!["4", "a0c"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv")
+        .arg(r#"SELECT * FROM test WHERE val NOT REGEXP '.*c$'"#);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["idx", "val"],
+        svec!["0", "ABC"],
+        svec!["2", "000"],
+        svec!["3", "A0C"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_expression_pattern_match() {
+    let wrk = Workdir::new("sqlp_expression_pattern_match");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["idx", "val", "pat"],
+            svec!["0", "ABC", "^A"],
+            svec!["1", "abc", "^A"],
+            svec!["2", "000", "^A"],
+            svec!["3", "A0C", r#"[AB]\d.*$"#,],
+            svec!["4", "a0c", ".*xxx$"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv")
+        .arg("SELECT idx, val FROM test WHERE val REGEXP pat");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![svec!["idx", "val"], svec!["0", "ABC"], svec!["3", "A0C"]];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_sql_join_on_subquery() {
+    let wrk = Workdir::new("sqlp_sql_join_on_subquery");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["idx", "val"],
+            svec!["0", "ABC"],
+            svec!["1", "abc"],
+            svec!["2", "000"],
+            svec!["3", "A0C"],
+            svec!["4", "a0c"],
+        ],
+    );
+
+    wrk.create(
+        "test2.csv",
+        vec![
+            svec!["idx", "val"],
+            svec!["0", "ABC"],
+            svec!["1", "abc"],
+            svec!["2", "000"],
+            svec!["3", "A0C"],
+            svec!["4", "a0c"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg("test2.csv").arg(
+        "SELECT * FROM test t1 JOIN (SELECT idx, val FROM test2 WHERE idx > 2) t2 ON t1.idx = \
+         t2.idx",
+    );
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["idx", "val", "val_right"],
+        svec!["3", "A0C", "A0C"],
+        svec!["4", "a0c", "a0c"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_sql_from_subquery() {
+    let wrk = Workdir::new("sqlp_sql_from_subquery");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["idx", "val"],
+            svec!["0", "ABC"],
+            svec!["1", "abc"],
+            svec!["2", "000"],
+            svec!["3", "A0C"],
+            svec!["4", "a0c"],
+        ],
+    );
+
+    wrk.create(
+        "test2.csv",
+        vec![
+            svec!["idx", "val"],
+            svec!["0", "ABC"],
+            svec!["1", "abc"],
+            svec!["2", "000"],
+            svec!["3", "A0C"],
+            svec!["4", "a0c"],
+        ],
+    );
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv").arg("test2.csv").arg(
+        "SELECT * FROM (SELECT idx, val FROM test WHERE idx > 2) t1 JOIN test2 t2 ON t1.idx = \
+         t2.idx",
+    );
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["idx", "val", "val_right"],
+        svec!["3", "A0C", "A0C"],
+        svec!["4", "a0c", "a0c"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn sqlp_sql_tsv() {
+    let wrk = Workdir::new("sqlp_sql_tsv");
+    wrk.create(
+        "test.csv",
+        vec![
+            svec!["idx", "val"],
+            svec!["0", "ABC"],
+            svec!["1", "abc"],
+            svec!["2", "000"],
+            svec!["3", "A0C"],
+            svec!["4", "a0c"],
+        ],
+    );
+
+    let output_file = wrk.path("output.tsv").to_string_lossy().to_string();
+
+    let mut cmd = wrk.command("sqlp");
+    cmd.arg("test.csv")
+        .arg("SELECT * FROM test")
+        .args(["--output", &output_file]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got = wrk.read_to_string(&output_file);
+
+    let expected = "idx\tval\n0\tABC\n1\tabc\n2\t000\n3\tA0C\n4\ta0c\n";
+
+    assert_eq!(got, expected);
+}
