@@ -66,6 +66,8 @@ mod test_fmt;
 #[cfg(all(feature = "foreach", target_family = "unix"))]
 mod test_foreach;
 mod test_frequency;
+#[cfg(all(feature = "feature_capable", feature = "geocode"))]
+mod test_geocode;
 mod test_headers;
 mod test_index;
 mod test_input;
